@@ -51,7 +51,9 @@ from chalk.utils.string import oxford_comma_list
 if TYPE_CHECKING:
     import types
 
-    import sqlglot
+    from sqlglot.expressions import Select, Union
+
+    from chalk.features import FeatureWrapper
 
 
 class DiagnosticBuilder:
@@ -135,10 +137,37 @@ class LSPErrorBuilder:
     we can't depend on the __eq__ method of the exception
     object."""
 
+    _node_map: dict[tuple[FeatureWrapper, str], tuple[ast.AST, types.FrameType]] = {}
+
     @classmethod
     def has_errors(cls):
         return cls.lsp and len(cls.all_errors) > 0
 
+    @classmethod
+    def save_node(cls, wrapper: FeatureWrapper, item: str):
+        frame = inspect.currentframe()
+        if frame is None:
+            return
+        i = 0
+        while i < 2 and frame is not None:
+            frame = frame.f_back
+            i += 1
+        if frame is None:
+            return
+        try:
+            node = Source.executing(frame).node
+        except Exception:
+            return
+        if node is not None:
+            try:
+                cls._node_map[(wrapper, item)] = (node, frame)
+            except:
+                pass
+
+    @classmethod
+    def get_node(cls, wrapper: FeatureWrapper, item: str) -> tuple[ast.AST, types.FrameType] | None:
+        return cls._node_map.get((wrapper, item))
+
     @classmethod
     def save_exception(cls, e: Exception, uri: str, diagnostic: DiagnosticGQL):
         """Save an exception to be promoted to a diagnostic later.
@@ -206,11 +235,7 @@ class FeatureClassErrorBuilder:
         return get_class_definition_range(cls=self.node, filename=self.uri)
 
     def invalid_attribute(
-        self,
-        namespace: str,
-        item: str,
-        candidates: List[str],
-        back: int,
+        self, namespace: str, item: str, candidates: List[str], back: int, saved_frame: FeatureWrapper | None = None
     ):
         back = back + 1
         message = f"Invalid attribute '{item}' on feature class '{namespace}'."
@@ -218,25 +243,30 @@ class FeatureClassErrorBuilder:
             # Short circuit if we're not in an LSP context. What follows is expensive.
             raise AttributeError(message)
 
-        frame: Optional[types.FrameType] = inspect.currentframe()
-        i = 0
-        while i < back and frame is not None:
-            frame = frame.f_back
-            i += 1
+        if saved_frame is not None:
+            saved_node_and_frame = LSPErrorBuilder.get_node(saved_frame, item)
+            if saved_node_and_frame is None:
+                raise AttributeError(message)
+            node, frame = saved_node_and_frame
+        else:
+            frame: Optional[types.FrameType] = inspect.currentframe()
+            i = 0
+            while i < back and frame is not None:
+                frame = frame.f_back
+                i += 1
 
-        if frame is None or i != back:
-            raise AttributeError(message)
+            if frame is None or i != back:
+                raise AttributeError(message)
 
-        try:
-            node = Source.executing(frame).node
-        except Exception:
-            raise AttributeError(message)
+            try:
+                node = Source.executing(frame).node
+            except Exception:
+                raise AttributeError(message)
 
-        if "__file__" not in frame.f_locals:
-            raise AttributeError(message)
-
-        uri = frame.f_locals["__file__"]
+        uri = frame.f_locals.get("__file__")
         if isinstance(node, ast.Attribute):
+            if node.end_lineno is None or node.end_col_offset is None:
+                raise AttributeError(message)
             node = RangeGQL(
                 start=PositionGQL(
                     line=node.end_lineno,
@@ -322,9 +352,13 @@ class FeatureClassErrorBuilder:
             if raise_error is not None:
                 raise raise_error(message)
             return _dummy_builder
-
+        default_error = TypeError
+        if range is None:
+            raise raise_error(message) if raise_error else default_error(message)
         if isinstance(range, ast.AST):
             range = node_to_range(range)
+            if range is None:
+                raise raise_error(message) if raise_error else default_error(message)
 
         builder = DiagnosticBuilder(
             severity=severity,
@@ -345,10 +379,7 @@ class FeatureClassErrorBuilder:
                 LSPErrorBuilder.save_exception(error, uri, builder.diagnostic)
                 raise error
         else:
-            raise error or TypeError(message)
-
-        if error is not None:
-            raise error
+            raise error or default_error(message)
 
         return builder
 
@@ -458,9 +489,14 @@ class ResolverErrorBuilder:
                 raise raise_error(message)
             return _dummy_builder
         uri = self.uri if uri is None else uri
+        default_error = TypeError
+        if range is None:
+            raise raise_error(message) if raise_error else default_error(message)
 
         if isinstance(range, ast.AST):
             range = node_to_range(range)
+            if range is None:
+                raise raise_error(message) if raise_error else default_error(message)
 
         builder = DiagnosticBuilder(
             severity=severity,
@@ -481,10 +517,7 @@ class ResolverErrorBuilder:
                 LSPErrorBuilder.save_exception(error, uri, builder.diagnostic)
                 raise error
         else:
-            raise error or TypeError(message)
-
-        if error is not None:
-            raise error
+            raise error or default_error(message)
 
         return builder
 
@@ -525,7 +558,7 @@ class ResolverErrorBuilder:
     def function_arg_value_by_name(self, name: str) -> ast.AST | None:
         return self.function_arg_values().get(name)
 
-    def function_arg_value_by_index(self, index: int) -> ast.AST | None:
+    def function_arg_value_by_index(self, index: int) -> ast.AST | RangeGQL | None:
         if self.node is None:
             return None
 
@@ -552,7 +585,7 @@ class ResolverErrorBuilder:
             return self.node.args.args[index].annotation
         return None
 
-    def function_return_annotation(self) -> ast.AST | None:
+    def function_return_annotation(self) -> ast.AST | RangeGQL | None:
         if self.node is None:
             return None
 
@@ -574,6 +607,8 @@ class ResolverErrorBuilder:
     def string_in_node(self, node: ast.AST, string: str, text: list[str]) -> RangeGQL | None:
         start_line = range_or_node_to_start_line(node)
         end_line = range_or_node_to_end_line(node)
+        if start_line is None or end_line is None:
+            return None
         for i, line in enumerate(text):
             if i < start_line:
                 continue
@@ -581,10 +616,14 @@ class ResolverErrorBuilder:
                 return None
             if i == start_line:
                 start_char = range_or_node_to_start_char(node)
+                if start_char is None:
+                    return None
             else:
                 start_char = 0
             if i == end_line:
                 end_char = range_or_node_to_end_char(node)
+                if end_char is None:
+                    return None
             else:
                 end_char = len(line)
             starting_index = line.find(string, start_char, end_char)
@@ -614,6 +653,7 @@ def get_resolver_error_builder(fn: Callable) -> ResolverErrorBuilder:
 
 class SQLFileResolverErrorBuilder:
     def __init__(self, uri: str, sql_string: str):
+        super().__init__()
         self.uri = uri
         self.diagnostics: List[DiagnosticGQL] = []
         self.sql_string = sql_string
@@ -646,10 +686,15 @@ class SQLFileResolverErrorBuilder:
             if raise_error is not None:
                 raise raise_error(message)
             return _dummy_builder
+        default_error = TypeError
+        if range is None:
+            raise raise_error(message) if raise_error else default_error(message)
         uri = self.uri if uri is None else uri
 
         if isinstance(range, ast.AST):
             range = node_to_range(range)
+            if range is None:
+                raise raise_error(message) if raise_error else default_error(message)
 
         builder = DiagnosticBuilder(
             severity=severity,
@@ -670,9 +715,7 @@ class SQLFileResolverErrorBuilder:
                 LSPErrorBuilder.save_exception(error, uri, builder.diagnostic)
                 raise error
         else:
-            raise error or TypeError(message)
-        if error is not None:
-            raise error
+            raise error or default_error(message)
 
         return builder
 
@@ -685,9 +728,7 @@ class SQLFileResolverErrorBuilder:
     def variable_range_by_name(self, name: str) -> RangeGQL | None:
         return get_variable_range(self.sql_lines, name)
 
-    def value_range_by_name(
-        self, glot: sqlglot.expressions.Expression | sqlglot.expressions.Union, name: str
-    ) -> RangeGQL | None:
+    def value_range_by_name(self, glot: Select | Union, name: str) -> RangeGQL | None:
         return get_feature_range(self.sql_lines, glot, name)
 
     def custom_range(self, line_no: int, col: int, length: int | None = None) -> RangeGQL:
@@ -706,9 +747,68 @@ class SQLFileResolverErrorBuilder:
     def full_range(self) -> RangeGQL:
         return get_full_range(self.sql_lines)
 
-    def sql_range(self) -> RangeGQL:
+    def sql_range(self) -> RangeGQL | None:
         return get_sql_range(self.sql_lines)
 
+    def add_diagnostic_with_spellcheck(
+        self,
+        spellcheck_item: str,
+        spellcheck_candidates: List[str],
+        message: str,
+        label: str,
+        code: str,
+        range: RangeGQL | ast.AST | None,
+        code_href: str | None = None,
+        severity: DiagnosticSeverityGQL = DiagnosticSeverityGQL.Error,
+        raise_error: Type[Exception] | None = None,
+        uri: str | None = None,
+    ):
+        if not LSPErrorBuilder.lsp:
+            # Don't do anything special let's just add the diagnostic
+            return self.add_diagnostic(
+                message=message,
+                label=label,
+                code=code,
+                range=range,
+                code_href=code_href,
+                severity=severity,
+                raise_error=raise_error,
+                uri=uri,
+            )
+
+        candidates = [f"'{c}'" for c in spellcheck_candidates if not c.split(".")[-1].startswith("_")]
+        if len(candidates) > 0:
+            all_scores = [
+                (
+                    Levenshtein.distance(spellcheck_item, candidate),
+                    candidate,
+                )
+                for candidate in candidates
+            ]
+            all_scores.sort(key=lambda x: x[0])
+
+            if len(candidates) > 5:
+                prefix = "The closest options are"
+                candidates = [c for (_, c) in all_scores[:5]]
+            elif len(candidates) == 1:
+                prefix = "The only valid option is"
+            else:
+                prefix = "Valid options are"
+                candidates = [c for (_, c) in all_scores]
+
+            message += f" {prefix} {oxford_comma_list(candidates)}."
+
+        return self.add_diagnostic(
+            message=message,
+            label=label,
+            code=code,
+            range=range,
+            code_href=code_href,
+            severity=severity,
+            raise_error=raise_error,
+            uri=uri,
+        )
+
 
 """
 The following helper methods are empirically proven to be correct.
@@ -717,33 +817,41 @@ The character offsets are 1 indexed and non-inclusive
 """
 
 
-def range_or_node_to_start_line(range_or_node: RangeGQL | ast.AST) -> int:
+def range_or_node_to_start_line(range_or_node: RangeGQL | ast.AST) -> int | None:
     if isinstance(range_or_node, RangeGQL):
         i = range_or_node.start.line
     else:
         i = range_or_node.lineno
+        if i is None:
+            return None
     return i - 1
 
 
-def range_or_node_to_end_line(range_or_node: RangeGQL | ast.AST) -> int:
+def range_or_node_to_end_line(range_or_node: RangeGQL | ast.AST) -> int | None:
     if isinstance(range_or_node, RangeGQL):
         i = range_or_node.end.line
     else:
         i = range_or_node.end_lineno
+        if i is None:
+            return None
     return i - 1
 
 
-def range_or_node_to_start_char(range_or_node: RangeGQL | ast.AST) -> int:
+def range_or_node_to_start_char(range_or_node: RangeGQL | ast.AST) -> int | None:
     if isinstance(range_or_node, RangeGQL):
         i = range_or_node.start.character
     else:
         i = range_or_node.col_offset
+        if i is None:
+            return None
     return i
 
 
-def range_or_node_to_end_char(range_or_node: RangeGQL | ast.AST) -> int:
+def range_or_node_to_end_char(range_or_node: RangeGQL | ast.AST) -> int | None:
     if isinstance(range_or_node, RangeGQL):
         i = range_or_node.end.character
     else:
         i = range_or_node.end_col_offset
+        if i is None:
+            return None
     return max(i - 1, 0)
