@@ -12,7 +12,7 @@ from chalk._monitoring.charts_enums_codegen import (
     WindowFunctionKind,
 )
 from chalk.features import Feature
-from chalk.features.resolver import Resolver, ResolverProtocol
+from chalk.features.resolver import RESOLVER_REGISTRY, ResolverProtocol
 
 
 @dataclasses.dataclass
@@ -70,9 +70,6 @@ class SeriesBase:
         self._time_shift: Optional[int] = time_shift
         self._entity_kind = ChartLinkKind.manual
         self._entity_id = None
-        _resolver_names = [resolver.name for resolver in Resolver.registry]
-        _resolver_fqns = [resolver.fqn for resolver in Resolver.registry]
-        self._all_valid_resolver_strings = _resolver_fqns + _resolver_names
 
     @property
     def _default_name(self):
@@ -99,8 +96,9 @@ class SeriesBase:
     def name(self):
         return self._name or self._default_name
 
-    @staticmethod
-    def _get_window_function_type(key: Optional[Union[str, WindowFunctionKind]]) -> Union[WindowFunctionKind, None]:
+    def _get_window_function_type(
+        self, key: Optional[Union[str, WindowFunctionKind]]
+    ) -> Union[WindowFunctionKind, None]:
         if key is None or isinstance(key, WindowFunctionKind):
             return key
 
@@ -111,7 +109,7 @@ class SeriesBase:
         if uk in _WINDOW_FUNCTION_VALUE_DICT:
             return _WINDOW_FUNCTION_VALUE_DICT[uk]
 
-        raise ValueError(f"Window function '{key}' is not supported.")
+        raise ValueError(f"Window function '{key}' is not supported for Series '{self.name}'")
 
     def _where(
         self: TSeries,
@@ -174,13 +172,13 @@ class SeriesBase:
 
     def _feature_name_filter(self: TSeries, *features: Tuple[Any], equals: bool) -> TSeries:
         if not features:
-            raise ValueError("One or more Chalk Features must be supplied.")
+            raise ValueError(f"One or more Chalk Features must be supplied to Series '{self.name}'.")
         copy = self._copy_with()
         comparator = ComparatorKind.EQ if equals else ComparatorKind.NEQ
         if len(features) == 1 or not equals:
             for feature in features:
                 value = str(feature)
-                _validate_feature(value)
+                self._validate_feature(value)
                 metric_filter = MetricFilter(kind=FilterKind.FEATURE_NAME, comparator=comparator, value=[value])
                 copy._filters.append(metric_filter)
             if len(features) == 1:
@@ -189,14 +187,14 @@ class SeriesBase:
                 copy._entity_kind = ChartLinkKind.feature
         else:
             values = [str(feature) for feature in features]
-            [_validate_feature(value) for value in values]
+            [self._validate_feature(value) for value in values]
             metric_filter = MetricFilter(kind=FilterKind.FEATURE_NAME, comparator=ComparatorKind.ONE_OF, value=values)
             copy._filters.append(metric_filter)
         return copy
 
     def _resolver_name_filter(self: TSeries, *resolvers: Union[ResolverProtocol, str], equals: bool) -> TSeries:
         if not resolvers:
-            raise ValueError("One or more Chalk Resolvers must be supplied.")
+            raise ValueError(f"One or more Chalk Resolvers must be supplied for Series '{self.name}'.")
         copy = self._copy_with()
         comparator = ComparatorKind.EQ if equals else ComparatorKind.NEQ
         if len(resolvers) == 1 or not equals:
@@ -218,7 +216,7 @@ class SeriesBase:
 
     def _string_filter(self: TSeries, *strings: str, kind: FilterKind, equals: bool = True) -> TSeries:
         if not strings:
-            raise ValueError("One or more arguments must be supplied for this filter")
+            raise ValueError(f"One or more arguments must be supplied for this filter for Series '{self.name}'.")
         copy = self._copy_with()
         comparator = ComparatorKind.EQ if equals else ComparatorKind.NEQ
         if len(strings) == 1 or not equals:
@@ -237,9 +235,15 @@ class SeriesBase:
         self: TSeries, *resolver_types: Literal["online", "offline", "stream"], equals: bool = True
     ) -> TSeries:
         if not resolver_types:
-            raise ValueError("One or more resolver types from 'online', 'offline', or 'stream' must be supplied")
+            raise ValueError(
+                f"One or more resolver types from 'online', 'offline', or 'stream' must be supplied "
+                f"for Series '{self.name}'."
+            )
         if not set(resolver_types).issubset(["online", "offline", "stream"]):
-            raise ValueError(f"Resolver types '{resolver_types}' must be one of 'online', 'offline', or 'stream'")
+            raise ValueError(
+                f"Resolver types '{resolver_types}' must be one of 'online', 'offline', or 'stream' "
+                f"for Series '{self.name}'."
+            )
         copy = self._copy_with()
         comparator = ComparatorKind.EQ if equals else ComparatorKind.NEQ
         if len(resolver_types) == 1 or not equals:
@@ -312,8 +316,20 @@ class SeriesBase:
         return self_copy
 
     def _validate_resolver(self, resolver_string: str):
-        if resolver_string not in self._all_valid_resolver_strings:
-            raise ValueError(f"No resolver found with name or fqn '{resolver_string}'")
+        maybe_resolver = RESOLVER_REGISTRY.get_resolver(resolver_string)
+        if maybe_resolver is None:
+            raise ValueError(f"No resolver found with name or fqn '{resolver_string}' for Series '{self.name}'.")
+
+    def _validate_feature(self, feature_string: str):
+        split = feature_string.split(".")
+        try:
+            assert len(split) == 2
+            Feature.from_root_fqn(feature_string)
+        except Exception:
+            raise ValueError(
+                f"No feature found with name '{feature_string}' for Series '{self._name}'. "
+                f"Feature must be in the format 'user.name'"
+            )
 
     def __gt__(self, other: Union[float, int]) -> ThresholdFunction:
         return ThresholdFunction(self, ">", other)
@@ -361,12 +377,3 @@ class SeriesBase:
         )
 
         return zlib.crc32(series_string.encode())
-
-
-def _validate_feature(feature_string: str):
-    split = feature_string.split(".")
-    try:
-        assert len(split) == 2
-        Feature.from_root_fqn(feature_string)
-    except Exception:
-        raise ValueError(f"No feature found with name '{feature_string}'. Feature must be in the format 'user.name'")
