@@ -19,12 +19,12 @@ else:
 
 
 class BatchProgress(BaseModel):
-    total: int
-    computed: int
-    failed: int
+    total: int = 0
+    computed: int = 0
+    failed: int = 0
     start: Optional[datetime]
     end: Optional[datetime]
-    total_duration_s: float
+    total_duration_s: float = 0.0
 
     @classmethod
     def empty(cls) -> "BatchProgress":
@@ -142,3 +142,15 @@ class BatchReportResponse(BaseModel):
 
 class InitiateOfflineQueryResponse(BaseModel):
     revision_id: uuid.UUID
+
+
+class Heartbeat(BaseModel):
+    operation_id: str
+    operation_kind: BatchOpKind
+    project_id: str
+    environment_id: str
+    team_id: str
+    deployment_id: str
+    generated_at: datetime
+    computer_id: int
+    shard_id: int
