@@ -9,7 +9,10 @@ from typing import TYPE_CHECKING, List, Optional, cast
 
 from dataclasses_json import DataClassJsonMixin
 
+from chalk._gen.chalk.artifacts.v1 import export_pb2 as export_pb
 from chalk._lsp.error_builder import LSPErrorBuilder
+from chalk.parsed._proto.export import import_files_then_export_from_registry
+from chalk.parsed._proto.lsp import convert_lsp_gql_to_proto
 
 if TYPE_CHECKING:
     from pydantic import ValidationError
@@ -50,19 +53,47 @@ def get_list_results(directory: Optional[str], file_allowlist: Optional[List[str
         )
 
 
-def dump_cmd(filename: str, directory: Optional[str], filter_file: Optional[str]):
+def get_artifacts_export_safe(directory: Optional[str], file_allowlist: Optional[List[str]]) -> bytes:
+    try:
+        return import_files_then_export_from_registry(directory, file_allowlist).SerializeToString()
+    except Exception:
+        ex_type, ex_value, ex_traceback = sys.exc_info()
+        assert ex_type is not None
+        relevant_traceback = f"""{os.linesep.join(traceback.format_tb(ex_traceback))}
+\n{ex_type.__name__}: {str(ex_value)}
+"""
+        return export_pb.Export(
+            failed=[
+                export_pb.FailedImport(
+                    traceback=relevant_traceback,
+                )
+            ],
+            lsp=convert_lsp_gql_to_proto(get_lsp_gql()),
+        ).SerializeToString()
+
+
+def dump_cmd(filename: str, directory: Optional[str], filter_file: Optional[str], use_proto: bool = False):
     file_allowlist = None
     if filter_file is not None and os.path.exists(filter_file):
         with open(filter_file) as f:
             file_allowlist = [f.strip() for f in f.readlines()]
 
-    with open(filename, "w") as f:
-        f.write(
-            get_list_results(
-                directory=directory,
-                file_allowlist=file_allowlist,
+    if use_proto:
+        with open(filename, "wb") as f:
+            f.write(
+                get_artifacts_export_safe(
+                    directory=directory,
+                    file_allowlist=file_allowlist,
+                )
+            )
+    else:
+        with open(filename, "w") as f:
+            f.write(
+                get_list_results(
+                    directory=directory,
+                    file_allowlist=file_allowlist,
+                )
             )
-        )
 
 
 def config_cmd():
@@ -112,6 +143,9 @@ def cli(args_override: Optional[List[str]] = None):
     export_parser.add_argument("--no-lsp", dest="lsp", action="store_false")
     export_parser.set_defaults(lsp=False)
 
+    export_parser.add_argument("--proto", action="store_true")
+    export_parser.set_defaults(proto=False)
+
     # STUBGEN
     stubgen_parser = subparsers.add_parser("stubgen", help="Generate type stubs for feature set classes")
     configure_stubgen_argparse(stubgen_parser)
@@ -131,7 +165,7 @@ def cli(args_override: Optional[List[str]] = None):
 
     if args.command == "export":
         LSPErrorBuilder.lsp = args.lsp
-        dump_cmd(filename=args.filename, directory=args.directory, filter_file=args.file_filter)
+        dump_cmd(filename=args.filename, directory=args.directory, filter_file=args.file_filter, use_proto=args.proto)
 
     elif args.command == "config":
         config_cmd()
