@@ -8,6 +8,7 @@ from typing import TYPE_CHECKING, Any, Dict, Literal, Mapping, Optional, Protoco
 import requests
 from typing_extensions import TypeAlias
 
+from chalk._gen.chalk.auth.v1.permissions_pb2 import Permission
 from chalk.client.exc import ChalkAuthException, ChalkBaseException
 from chalk.client.models import (
     BranchDeployResponse,
@@ -22,6 +23,7 @@ from chalk.client.models import (
     FeatureObservationDeletionResponse,
     FeatureReference,
     FeatureResult,
+    GetIncrementalProgressResponse,
     OfflineQueryContext,
     OnlineQuery,
     OnlineQueryContext,
@@ -100,8 +102,16 @@ class DatasetRevision(Protocol):
     num_computers: int
     """Number of computers this query ran on."""
 
+    branch: str | None = None
+    """Name of branch"""
+
     errors: list[ChalkError] | None = None
 
+    @property
+    def url(self) -> str | None:
+        """url linking to relevant dashboard page"""
+        return self.dashboard_url
+
     def to_polars(
         self,
         output_id: bool = False,
@@ -119,9 +129,9 @@ class DatasetRevision(Protocol):
             Whether to return the primary key feature in a column
             named `"__chalk__.__id__"` in the resulting `pl.LazyFrame`.
         output_ts
-            Whether to return the timestamp feature in a column
+            Whether to return the input-time feature in a column
             named `"__chalk__.CHALK_TS"` in the resulting `pl.LazyFrame`.
-            If set to a non-empty `str`, used as the timestamp column name.
+            If set to a non-empty `str`, used as the input-time column name.
         ignore_errors
             Whether to ignore query errors upon fetching data
         show_progress
@@ -155,9 +165,9 @@ class DatasetRevision(Protocol):
             Whether to return the primary key feature in a column
             named `"__chalk__.__id__"` in the resulting `pl.LazyFrame`.
         output_ts
-            Whether to return the timestamp feature in a column
+            Whether to return the input-time feature in a column
             named `"__chalk__.CHALK_TS"` in the resulting `pl.LazyFrame`.
-            If set to a non-empty `str`, used as the timestamp column name.
+            If set to a non-empty `str`, used as the input-time column name.
         ignore_errors
             Whether to ignore query errors upon fetching data
         show_progress
@@ -191,9 +201,9 @@ class DatasetRevision(Protocol):
             Whether to return the primary key feature in a column
             named `"__chalk__.__id__"` in the resulting `pl.LazyFrame`.
         output_ts
-            Whether to return the timestamp feature in a column
+            Whether to return the input-time feature in a column
             named `"__chalk__.CHALK_TS"` in the resulting `pl.LazyFrame`.
-            If set to a non-empty `str`, used as the timestamp column name.
+            If set to a non-empty `str`, used as the input-time column name.
         ignore_errors
             Whether to ignore query errors upon fetching data
         show_progress
@@ -227,9 +237,9 @@ class DatasetRevision(Protocol):
             Whether to return the primary key feature in a column
             named `"__chalk__.__id__"` in the resulting `pd.DataFrame`.
         output_ts
-            Whether to return the timestamp feature in a column
+            Whether to return the input-time feature in a column
             named `"__chalk__.CHALK_TS"` in the resulting `pd.DataFrame`.
-            If set to a non-empty `str`, used as the timestamp column name.
+            If set to a non-empty `str`, used as the input-time column name.
         ignore_errors
             Whether to ignore query errors upon fetching data
         show_progress
@@ -264,9 +274,9 @@ class DatasetRevision(Protocol):
             Whether to return the primary key feature in a column
             named `"__chalk__.__id__"` in the resulting `DataFrame`.
         output_ts
-            Whether to return the timestamp feature in a column
+            Whether to return the input-time feature in a column
             named `"__chalk__.CHALK_TS"` in the resulting `DataFrame`.
-            If set to a non-empty `str`, used as the timestamp column name.
+            If set to a non-empty `str`, used as the input-time column name.
         ignore_errors
             Whether to ignore query errors upon fetching data\
         show_progress
@@ -299,6 +309,27 @@ class DatasetRevision(Protocol):
         """
         ...
 
+    def wait(
+        self,
+        timeout: float | timedelta | ellipsis | None = ...,
+        show_progress: bool = True,
+    ) -> None:
+        """
+        Waits for an offline query job to complete.
+        Raises if the query is unsuccessful, otherwise returns itself on success.
+
+        Parameters
+        ----------
+        timeout
+            How long to wait, in seconds, for job completion before raising a TimeoutError.
+            Jobs will continue to run in the background if they take longer than this timeout.
+            For no timeout, set to ``None``. If no timeout is specified, the client's default
+            timeout is used.
+        show_progress
+            Whether to show a progress bar
+        """
+        ...
+
     def download_data(
         self,
         path: str,
@@ -323,9 +354,9 @@ class DatasetRevision(Protocol):
             Whether to return the primary key feature in a column
             named `"__chalk__.__id__"` in the resulting `DataFrame`.
         output_ts
-            Whether to return the timestamp feature in a column
+            Whether to return the input-time feature in a column
             named `"__chalk__.CHALK_TS"` in the resulting `DataFrame`.
-            If set to a non-empty `str`, used as the timestamp column name.
+            If set to a non-empty `str`, used as the input-time column name.
         ignore_errors
             Whether to ignore query errors upon fetching data
         executor:
@@ -411,12 +442,25 @@ class DatasetRevision(Protocol):
         """
         ...
 
+    def ingest(self, store_online: bool = False, store_offline: bool = True) -> Dataset:
+        """
+        Saves this revision to Chalk's online and offline storage.
+
+        Parameters
+        ----------
+        store_online
+            Whether to store the revision in Chalk's online storage.
+        store_offline
+            Whether to store the revision in Chalk's offline storage.
+        """
+        ...
+
     def resolver_replay(
         self,
         resolver: ResolverProtocol,
         show_progress: bool = True,
         timeout: float | timedelta | None | ellipsis = ...,
-    ) -> Union[pl.DataFrame, pl.LazyFrame, Mapping[str, pl.DataFrame], Mapping[str, pl.LazyFrame]]:
+    ) -> Union[pl.DataFrame, pl.LazyFrame, Mapping[str, pl.DataFrame], Mapping[str, pl.LazyFrame],]:
         """
         Downloads the resolver replay data for the given resolver in the revision, provided the revision had store_plan_stages
         enabled.
@@ -508,9 +552,9 @@ class Dataset(Protocol):
             Whether to return the primary key feature in a column
             named `"__chalk__.__id__"` in the resulting `pl.LazyFrame`.
         output_ts
-            Whether to return the timestamp feature in a column
+            Whether to return the input-time feature in a column
             named `"__chalk__.CHALK_TS"` in the resulting `pl.LazyFrame`.
-            If set to a non-empty `str`, used as the timestamp column name.
+            If set to a non-empty `str`, used as the input-time column name.
         ignore_errors
             Whether to ignore query errors upon fetching data
         show_progress
@@ -544,9 +588,9 @@ class Dataset(Protocol):
             Whether to return the primary key feature in a column
             named `"__chalk__.__id__"` in the resulting `pl.LazyFrame`.
         output_ts
-            Whether to return the timestamp feature in a column
+            Whether to return the input-time feature in a column
             named `"__chalk__.CHALK_TS"` in the resulting `pl.LazyFrame`.
-            If set to a non-empty `str`, used as the timestamp column name.
+            If set to a non-empty `str`, used as the input-time column name.
         ignore_errors
             Whether to ignore query errors upon fetching data
         show_progress
@@ -581,7 +625,7 @@ class Dataset(Protocol):
         output_ts
             Whether to return the timestamp feature in a column
             named `"__chalk__.CHALK_TS"` in the resulting `pl.LazyFrame`.
-            If set to a non-empty `str`, used as the timestamp column name.
+            If set to a non-empty `str`, used as the input-time column name.
         ignore_errors
             Whether to ignore query errors upon fetching data
         show_progress
@@ -615,9 +659,9 @@ class Dataset(Protocol):
             Whether to return the primary key feature in a column
             named `"__chalk__.__id__"` in the resulting `pd.DataFrame`.
         output_ts
-            Whether to return the timestamp feature in a column
+            Whether to return the input-time feature in a column
             named `"__chalk__.CHALK_TS"` in the resulting `pd.DataFrame`.
-            If set to a non-empty `str`, used as the timestamp column name.
+            If set to a non-empty `str`, used as the input-time column name.
         ignore_errors
             Whether to ignore query errors upon fetching data
         show_progress
@@ -652,9 +696,9 @@ class Dataset(Protocol):
             Whether to return the primary key feature in a column
             named `"__chalk__.__id__"` in the resulting `DataFrame`.
         output_ts
-            Whether to return the timestamp feature in a column
+            Whether to return the input-time feature in a column
             named `"__chalk__.CHALK_TS"` in the resulting `DataFrame`.
-            If set to a non-empty `str`, used as the timestamp column name.
+            If set to a non-empty `str`, used as the input-time column name.
         ignore_errors
             Whether to ignore query errors upon fetching data
         show_progress
@@ -688,9 +732,9 @@ class Dataset(Protocol):
             Whether to return the primary key feature in a column
             named `"__chalk__.__id__"` in the resulting `pd.DataFrame`.
         output_ts
-            Whether to return the timestamp feature in a column
+            Whether to return the input-time feature in a column
             named `"__chalk__.CHALK_TS"` in the resulting `pd.DataFrame`.
-            If set to a non-empty `str`, used as the timestamp column name.
+            If set to a non-empty `str`, used as the input-time column name.
         ignore_errors
             Whether to ignore query errors upon fetching data
         show_progress
@@ -727,9 +771,9 @@ class Dataset(Protocol):
             Whether to return the primary key feature in a column
             named `"__chalk__.__id__"` in the resulting `pd.DataFrame`.
         output_ts
-            Whether to return the timestamp feature in a column
+            Whether to return the input-time feature in a column
             named `"__chalk__.CHALK_TS"` in the resulting `pd.DataFrame`.
-            If set to a non-empty `str`, used as the timestamp column name.
+            If set to a non-empty `str`, used as the input-time column name.
         ignore_errors
             Whether to ignore query errors upon fetching data
         show_progress
@@ -742,6 +786,26 @@ class Dataset(Protocol):
         """
         ...
 
+    def wait(
+        self,
+        timeout: float | timedelta | ellipsis | None = ...,
+        show_progress: bool = True,
+    ) -> Dataset:
+        """
+        Waits for an offline query job to complete. Returns a list of errors if unsuccessful, or None if successful.
+
+        Parameters
+        ----------
+        timeout
+            How long to wait, in seconds, for job completion before raising a TimeoutError.
+            Jobs will continue to run in the background if they take longer than this timeout.
+            For no timeout, set to ``None``. If no timeout is specified, the client's default
+            timeout is used.
+        show_progress
+            Whether to show a progress bar
+        """
+        ...
+
     def download_data(
         self,
         path: str,
@@ -841,8 +905,15 @@ class Dataset(Protocol):
         self,
         features: list[FeatureReference] | None = None,
         branch: str | None = None,
-        wait: bool = False,  # deprecated
-        show_progress: bool = True,
+        wait: bool = True,
+        show_progress: bool = False,
+        store_plan_stages: bool = False,
+        correlation_id: str | None = None,
+        explain: bool | Literal["only"] = False,
+        tags: list[str] | None = None,
+        required_resolver_tags: list[str] | None = None,
+        planner_options: Mapping[str, Union[str, int, bool]] | None = None,
+        use_multiple_computers: bool = False,
         timeout: float | timedelta | None | ellipsis = ...,
     ) -> Dataset:
         """Creates a new revision of this `Dataset` by recomputing the specified features.
@@ -853,6 +924,7 @@ class Dataset(Protocol):
         ------
         features
             A list of specific features to recompute. Features that don't exist in the dataset will be added.
+            Features that already exist in the dataset will be recomputed.
             If not provided, all the existing features in the dataset will be recomputed.
         branch
             If specified, Chalk will route your request to the relevant branch.
@@ -862,6 +934,23 @@ class Dataset(Protocol):
             If True, progress bars will be shown while recomputation is running.
             This flag will also be propogated to the methods of the resulting
             `Dataset`.
+        correlation_id
+            You can specify a correlation ID to be used in logs and web interfaces.
+            This should be globally unique, i.e. a `uuid` or similar. Logs generated
+            during the execution of your query will be tagged with this correlation id.
+        store_plan_stages
+            If True, the output of each of the query plan stages will be stored
+            in S3/GCS. This will dramatically impact the performance of the query,
+            so it should only be used for debugging.
+            These files will be visible in the web dashboard's query detail view, and
+            can be downloaded in full by clicking on a plan node in the query plan visualizer.
+        tags
+            The tags used to scope the resolvers.
+            See https://docs.chalk.ai/docs/resolver-tags for more information.
+        required_resolver_tags
+            If specified, *all* required_resolver_tags must be present on a resolver for it to be
+            considered eligible to execute.
+            See https://docs.chalk.ai/docs/resolver-tags for more information.
         timeout
             How long to wait, in seconds, for job completion before raising a TimeoutError.
             Jobs will continue to run in the background if they take longer than this timeout.
@@ -884,9 +973,25 @@ class Dataset(Protocol):
         """
         ...
 
+    def ingest(self, store_online: bool = False, store_offline: bool = True) -> Dataset:
+        """
+        Saves the latest revision of this dataset to Chalk's online and offline storage.
+
+        Parameters
+        ----------
+        store_online
+            Whether to store the revision in Chalk's online storage.
+        store_offline
+            Whether to store the revision in Chalk's offline storage.
+        """
+        ...
+
     def resolver_replay(
-        self, resolver: ResolverProtocol, show_progress: bool = True, timeout: float | timedelta | ellipsis | None = ...
-    ) -> Union[pl.DataFrame, pl.LazyFrame, Mapping[str, pl.DataFrame], Mapping[str, pl.LazyFrame]]:
+        self,
+        resolver: ResolverProtocol,
+        show_progress: bool = True,
+        timeout: float | timedelta | ellipsis | None = ...,
+    ) -> Union[pl.DataFrame, pl.LazyFrame, Mapping[str, pl.DataFrame], Mapping[str, pl.LazyFrame],]:
         """
         Downloads the resolver replay data for the given resolver in the latest revision of the dataset.
 
@@ -1012,7 +1117,7 @@ class OnlineQueryResult(Protocol):
         ...
 
 
-class ChalkClient(Protocol):
+class ChalkClient:
     """The `ChalkClient` is the primary Python interface for interacting with Chalk.
 
     You can use it to query data, trigger resolver runs, gather offline data, and more.
@@ -1020,8 +1125,8 @@ class ChalkClient(Protocol):
 
     def __init__(
         self,
-        client_id: str,
-        client_secret: str,
+        client_id: str | None = None,
+        client_secret: str | None = None,
         environment: EnvironmentId | None = None,
         api_server: str | None = None,
         branch: BranchId | None = None,
@@ -1105,6 +1210,7 @@ class ChalkClient(Protocol):
         encoding_options: FeatureEncodingOptions | None = None,
         required_resolver_tags: list[str] | None = None,
         planner_options: Mapping[str, Union[str, int, bool]] | None = None,
+        request_timeout: Optional[float] = None,
     ) -> OnlineQueryResult:
         """Compute features values using online resolvers.
         See https://docs.chalk.ai/docs/query-basics for more information.
@@ -1189,6 +1295,10 @@ class ChalkClient(Protocol):
             Values may be provided as part of conversations with Chalk support to
             to enable or disable specific functionality.
 
+        request_timeout
+            Float value indicating number of seconds that the request should wait before timing out
+            at the network level. May not cancel resources on the server processing the query.
+
         Returns
         -------
         OnlineQueryResult
@@ -1228,7 +1338,60 @@ class ChalkClient(Protocol):
         `Merchant` and there is no natural relationship object which is related to both of these classes, `multi_query`
         allows you to submit two independent queries.
 
+        Returns a BulkOnlineQueryResponse, which is functionally a list of query results. Each of these result
+        can be accessed by index. Individual results can be further checked for errors and converted
+        to pandas or polars DataFrames.
+
         In contrast, `query_bulk` executes a single query with multiple inputs/outputs.
+
+        Parameters
+        ----------
+        queries
+            A list of the OnlineQueries you'd like to execute.
+        environment
+            The environment under which to run the resolvers.
+            API tokens can be scoped to an environment.
+            If no environment is specified in the query,
+            but the token supports only a single environment,
+            then that environment will be taken as the scope
+            for executing the request.
+        branch
+            If specified, Chalk will route your request to the relevant branch.
+        preview_deployment_id
+            If specified, Chalk will route your request to the
+            relevant preview deployment.
+
+        Other Parameters
+        ----------------
+        query_name
+            The name for class of query you're making, for example, `"loan_application_model"`.
+        correlation_id
+            A globally unique ID for the query, used alongside logs and
+            available in web interfaces.
+        meta
+            Arbitrary `key:value` pairs to associate with a query.
+        compression
+            Which compression scheme to use pyarrow. Options are: {"zstd", "lz4", "uncompressed"}.
+
+        Returns
+        -------
+        BulkOnlineQueryResponse
+            An output containing results: list[BulkOnlineQueryResult], where each result contains
+            dataframes of the results of each query or any errors.
+
+        Examples
+        --------
+        >>> from chalk.client import ChalkClient, OnlineQuery
+        >>> queries =[
+        ...     OnlineQuery(input={User.name: ['Katherine Johnson'], output=[User.fico_score]}),
+        ...     OnlineQuery(input={Merchant.name': ['Myrrh Chant'], output=['Merchant.address']}),
+        ...     OnlineQuery(input={NonFeature.wrong': ['Wrong!'], output=['NonFeature.wrong']}),
+        ... ]
+        >>> result = ChalkClient().multi_query(
+        ...     queries=queries,
+        ... )
+        >>> result[0].get_feature_value(User.fico_score)
+        >>> queries_with_errors = [q for q, r in zip(queries, result) if r.errors is not None]
         """
         ...
 
@@ -1405,6 +1568,7 @@ class ChalkClient(Protocol):
         show_progress: bool = True,
         timeout: float | timedelta | ellipsis | None = ...,
         recompute_features: bool | list[FeatureReference] = False,
+        sample_features: list[FeatureReference] | None = None,
         lower_bound: datetime | None = None,
         upper_bound: datetime | None = None,
         store_plan_stages: bool = False,
@@ -1413,6 +1577,7 @@ class ChalkClient(Protocol):
         required_resolver_tags: list[str] | None = None,
         planner_options: Mapping[str, Union[str, int, bool]] | None = None,
         spine_sql_query: str | None = None,
+        spine_sql_primary_key_column_name: str | None = None,
     ) -> Dataset:
         """Compute feature values from the offline store or by running offline/online resolvers.
         See `Dataset` for more information.
@@ -1438,6 +1603,9 @@ class ChalkClient(Protocol):
             If False, all output features will be sampled from the offline store.
             If a list, all output features in recompute_features will be recomputed,
             and all other output features will be sampled from the offline store.
+        sample_features
+            A list of features that will always be sampled, and thus always excluded from recompute.
+            Should not overlap with any features used in "recompute_features" argument.
         environment
             The environment under which to run the resolvers.
             API tokens can be scoped to an environment.
@@ -1625,6 +1793,9 @@ class ChalkClient(Protocol):
         branch: BranchId | None = ...,
         upper_bound: datetime | None = None,
         lower_bound: datetime | None = None,
+        store_online: bool = True,
+        store_offline: bool = True,
+        timestamping_mode: Literal["feature_time", "online_store_write_time"] = "feature_time",
     ) -> ResolverRunResponse:
         """Triggers a resolver to run.
         See https://docs.chalk.ai/docs/runs for more information.
@@ -1647,6 +1818,10 @@ class ChalkClient(Protocol):
             If specified, the resolver will only ingest data observed before this timestamp.
         lower_bound
             If specified, the resolver will only ingest data observed after this timestamp.
+        store_online
+            If True, the resolver run output will be stored in the online store.
+        store_offline
+            If True, the resolver run output will be stored in the offline store.
         branch
 
         Returns
@@ -1721,7 +1896,7 @@ class ChalkClient(Protocol):
         --------
         >>> from chalk.client import ChalkClient
         >>> ChalkClient().whoami()
-        WhoAmIResponse(user="44")
+        WhoAmIResponse(user="...", environment_id='...', team_id='...')
         """
         ...
 
@@ -1972,9 +2147,9 @@ class ChalkClient(Protocol):
             where instead the row would be included, but the feature
             value would be `None`.
         output_ts
-            Whether to return the timestamp feature in a column
+            Whether to return the input-time feature in a column
             named `"__chalk__.CHALK_TS"` in the resulting `DataFrame`.
-            If set to a non-empty `str`, used as the timestamp column name.
+            If set to a non-empty `str`, used as the input-time column name.
         output_id
             Whether to return the primary key feature in a column
             named `"__chalk__.__id__"` in the resulting `DataFrame`.
@@ -2131,6 +2306,73 @@ class ChalkClient(Protocol):
         """
         ...
 
+    def set_incremental_cursor(
+        self,
+        *,
+        resolver: str | Resolver | None = None,
+        scheduled_query: str | None = None,
+        max_ingested_timestamp: datetime | None = None,
+        last_execution_timestamp: datetime | None = None,
+    ) -> None:
+        """
+        Sets the incremental cursor for a resolver or scheduled query.
+
+        Parameters
+        ---------
+        resolver
+            The resolver. Can be a function or the string name of a function.
+            Exactly one of `resolver` and `scheduled_query` is required.
+        scheduled_query
+            The name of the scheduled query. Exactly one of `resolver` and `scheduled_query`
+            is required.
+        max_ingested_timestamp
+            Set the maximum timestamp of the data ingested by the resolver.
+        last_execution_timestamp
+            Override the last execution timestamp of the resolver.
+
+        Examples
+        --------
+        >>> from chalk.client import ChalkClient
+        >>> client = ChalkClient()
+        >>> client.set_incremental_cursor(
+        ...     resolver="my_resolver",
+        ...     max_ingested_timestamp=datetime.now(),
+        ... )
+        """
+        ...
+
+    def get_incremental_cursor(
+        self,
+        *,
+        resolver: str | Resolver | None = None,
+        scheduled_query: str | None = None,
+    ) -> GetIncrementalProgressResponse:
+        """
+        Gets the incremental cursor for a resolver or scheduled query.
+
+        Parameters
+        ---------
+        resolver
+            The resolver. Can be a function or the string name of a function.
+            Exactly one of `resolver` and `scheduled_query` is required.
+        scheduled_query
+            If updating incremental status of a resolver in the context of a
+            scheduled query, the name of the scheduled query.
+            Exactly one of `resolver` and `scheduled_query` is required.
+
+        Returns
+        ------
+        IncrementalStatus
+            An object containing the `max_ingested_timestamp` and `incremental_timestamp`.
+
+        Examples
+        --------
+        >>> from chalk.client import ChalkClient
+        >>> client = ChalkClient()
+        >>> client.get_incremental_cursor(resolver="my_resolver")
+        """
+        ...
+
     def test_streaming_resolver(
         self,
         resolver: str | Resolver,
@@ -2184,8 +2426,8 @@ class ChalkClient(Protocol):
         -------
         StreamResolverTestResponse
             A simple wrapper around a status and optional error message.
-            Calling `.features()` will return the test results, if they exist.
-            Otherwise, check `.errors` and `.message` for errors.
+            Inspecting `StreamResolverTestResponse.features` will return the test results, if they exist.
+            Otherwise, check `StreamResolverTestResponse.errors` and `StreamResolverTestResponse.message` for errors.
 
         Examples
         --------
@@ -2193,20 +2435,19 @@ class ChalkClient(Protocol):
         >>> from chalk.client import ChalkClient
         >>> from chalk.features import Features, features
         >>> import pydantic
-        >>>
-        >>> # This code is an example of a simple streaming feature setup.
+        >>> # This code is an example of a simple streaming feature setup. Define the source
         >>> stream_source=KafkaSource(...)
-        >>>
+        >>> # Define the features
         >>> @features(etl_offline_to_online=True, max_staleness="7d")
         >>> class StreamingFeature:
         >>>     id: str
         >>>     user_id: str
         >>>     card_id: str
-        >>>
+        >>> # Define the streaming message model
         >>> class StreamingMessage(pydantic.BaseModel):
         >>>     card_id: str
         >>>     user_id: str
-        >>>
+        >>> # Define the mapping resolver
         >>> @stream(source=stream_source)
         >>> def our_stream_resolver(
         >>>     m: StreamingMessage,
@@ -2216,7 +2457,6 @@ class ChalkClient(Protocol):
         >>>        card_id=m.card_id,
         >>>        user_id=m.user_id,
         >>>    )
-        >>>
         >>> # Once you have done a `chalk apply`, you can test the streaming resolver with custom messages as follows
         >>> client = ChalkClient()
         >>> keys = ["my_key"] * 10
@@ -2259,4 +2499,5 @@ __all__ = [
     "QueryStatus",
     "ResolverRunResponse",
     "WhoAmIResponse",
+    "Permission",
 ]
