@@ -11,7 +11,7 @@ from concurrent.futures import Future, ThreadPoolExecutor
 from datetime import datetime, timedelta, timezone
 from enum import IntEnum
 from sys import stderr
-from typing import TYPE_CHECKING, Any, Dict, List, Mapping, Optional, Sequence, Union, cast
+from typing import TYPE_CHECKING, Any, Dict, List, Literal, Mapping, Optional, Sequence, Union, cast
 from urllib.parse import urlparse
 
 import pandas as pd
@@ -26,6 +26,7 @@ from chalk.client.models import (
     DatasetRecomputeResponse,
     DatasetResponse,
     DatasetRevisionResponse,
+    IngestDatasetRequest,
     OfflineQueryContext,
     QueryStatus,
 )
@@ -36,7 +37,6 @@ from chalk.features.filter import freeze_time
 from chalk.features.pseudofeatures import CHALK_TS_FEATURE, ID_FEATURE, OBSERVED_AT_FEATURE, PSEUDONAMESPACE
 from chalk.features.resolver import Resolver
 from chalk.features.tag import BranchId, EnvironmentId
-from chalk.utils.collections import get_unique_item
 from chalk.utils.df_utils import read_parquet
 from chalk.utils.log_with_context import get_logger
 from chalk.utils.missing_dependency import missing_dependency_exception
@@ -290,14 +290,10 @@ def _extract_df_columns(
             # All dataframes have an __id__ column
             expected_cols[str(ID_FEATURE)] = id_col.alias(str(ID_FEATURE))
 
-        ts_col = to_utc(
-            df, str(OBSERVED_AT_FEATURE), pl.col(str(OBSERVED_AT_FEATURE)).fill_null(pl.col(str(CHALK_TS_FEATURE)))
-        )
         if output_ts:
             ts_col_name = str(CHALK_TS_FEATURE) if output_ts is True else output_ts
-            # For the ts feature, we want `__oat__` if not null else `__ts__`
-            # it is not null; otherwise, we want
-            expected_cols[ts_col_name] = ts_col.alias(ts_col_name)
+            input_ts_col = to_utc(df, str(CHALK_TS_FEATURE), pl.col(str(CHALK_TS_FEATURE)))
+            expected_cols[ts_col_name] = input_ts_col.alias(ts_col_name)
 
         if output_feature_fqns is None:
             # If not provided, return all columns, except for the OBSERVED_AT_FEATURE
@@ -309,17 +305,34 @@ def _extract_df_columns(
         else:
             # Make a best-effort attempt to determine the pkey and ts column fqn from the root namespace
             # of the other features
-            root_ns = get_unique_item(
-                (x.split(".")[0] for x in df.columns if not x.startswith(f"{PSEUDONAMESPACE}.")), "root_ns"
-            )
+
+            root_namespaces: "set[str]" = set()
+            for x in df.columns:
+                if not x.startswith(f"{PSEUDONAMESPACE}.") and "." in x:
+                    root_namespaces.add(x.split(".")[0])
+
+            if len(root_namespaces) == 1:
+                # There is a unique root namespace.
+                root_ns = root_namespaces.pop()
+            else:
+                # There are zero or multiple root namespaces, so none is best to choose.
+                root_ns = None
+
             ts_feature = None
             pkey_feature = None
             features_cls = None
-            if root_ns in FeatureSetBase.registry and version != DatasetVersion.NATIVE_COLUMN_NAMES:
+            if (
+                root_ns is not None
+                and root_ns in FeatureSetBase.registry
+                and version != DatasetVersion.NATIVE_COLUMN_NAMES
+            ):
                 """For native column names, we don't need to decipher column names at all"""
                 features_cls = FeatureSetBase.registry[root_ns]
                 ts_feature = features_cls.__chalk_ts__
                 pkey_feature = features_cls.__chalk_primary__
+            ts_col = to_utc(
+                df, str(OBSERVED_AT_FEATURE), pl.col(str(OBSERVED_AT_FEATURE)).fill_null(pl.col(str(CHALK_TS_FEATURE)))
+            )
             for x in output_feature_fqns:
                 if features_cls is not None and x in [f.fqn for f in features_cls.features if f.is_has_one]:
                     for col in df.columns:
@@ -328,7 +341,7 @@ def _extract_df_columns(
                     continue
                 if x == root_ns:
                     for col in df.columns:
-                        if col.startswith(root_ns) and not col.startswith("__"):
+                        if root_ns is not None and col.startswith(root_ns) and not col.startswith("__"):
                             expected_cols[col] = pl.col(col)
                     continue
                 if x in expected_cols:
@@ -535,7 +548,7 @@ class DatasetRevisionImpl(DatasetRevision):
         self.dataset_id = dataset_id
         self.dashboard_url = dashboard_url
         self._client = client
-        self._branch = BranchId(branch) if branch is not None else None
+        self.branch = BranchId(branch) if branch is not None else None
         self._hydrated = self.status == QueryStatus.SUCCESSFUL
         self.num_computers = num_computers
         self.errors = errors
@@ -583,7 +596,7 @@ class DatasetRevisionImpl(DatasetRevision):
             output_id=output_id,
             outputs=self.outputs,
             output_ts=output_ts,
-            branch=self._branch,
+            branch=self.branch,
             ignore_errors=ignore_errors,
             query_inputs=False,
         )
@@ -643,7 +656,7 @@ class DatasetRevisionImpl(DatasetRevision):
                 job_id=self.revision_id,
                 outputs=self.outputs,
                 context=context,
-                branch=self._branch,
+                branch=self.branch,
                 ignore_errors=ignore_errors,
                 query_inputs=False,
             )
@@ -688,7 +701,7 @@ class DatasetRevisionImpl(DatasetRevision):
                 output_ts=output_ts,
                 outputs=self.outputs,
                 context=context,
-                branch=self._branch,
+                branch=self.branch,
                 ignore_errors=ignore_errors,
                 query_inputs=False,
             )
@@ -705,7 +718,8 @@ class DatasetRevisionImpl(DatasetRevision):
     ) -> list[str]:
         from chalk.client.client_impl import DatasetJobStatusRequest
 
-        self._hydrate(caller_method=caller_name, timeout=timeout, show_progress=show_progress)
+        if not ignore_errors:
+            self._hydrate(caller_method=caller_name, timeout=timeout, show_progress=show_progress)
         context = OfflineQueryContext(environment=self.environment)
         status = self._client.get_job_status_v4(
             request=DatasetJobStatusRequest(
@@ -714,10 +728,31 @@ class DatasetRevisionImpl(DatasetRevision):
                 query_inputs=False,
             ),
             environment=context and context.environment,
-            branch=self._branch,
+            branch=self.branch,
         )
         return status.urls
 
+    def wait(
+        self,
+        timeout: float | timedelta | ellipsis | None = ...,
+        show_progress: bool = True,
+        caller_name: str = "wait",
+    ) -> None:
+        from chalk.client.client_impl import DatasetJobStatusRequest
+
+        self._hydrate(caller_method=caller_name, timeout=timeout, show_progress=show_progress)
+        context = OfflineQueryContext(environment=self.environment)
+        status = self._client.get_job_status_v4(
+            request=DatasetJobStatusRequest(
+                job_id=str(self.revision_id),
+                query_inputs=False,
+            ),
+            environment=context and context.environment,
+            branch=self.branch,
+        )
+        if status.errors is not None and len(status.errors) > 0:
+            raise ChalkBaseException(errors=status.errors)
+
     def download_data(
         self,
         path: str,
@@ -768,7 +803,7 @@ class DatasetRevisionImpl(DatasetRevision):
             output_id=False,
             outputs=None,
             output_ts=False,
-            branch=self._branch,
+            branch=self.branch,
             query_inputs=True,
             ignore_errors=ignore_errors,
         )
@@ -781,6 +816,20 @@ class DatasetRevisionImpl(DatasetRevision):
             webbrowser.open_new_tab(url)
         return url
 
+    def ingest(self, store_online: bool = False, store_offline: bool = True) -> DatasetImpl:
+        if not self._hydrated:
+            print("Waiting for dataset to complete before ingesting...")
+            self.wait(show_progress=False, caller_name="ingest")
+        context = OfflineQueryContext(environment=self.environment)
+        request = IngestDatasetRequest(
+            revision_id=str(self.revision_id),
+            branch=self.branch,
+            outputs=self.outputs,
+            store_online=store_online,
+            store_offline=store_offline,
+        )
+        return self._client.ingest_dataset(request, context)
+
     def resolver_replay(
         self,
         resolver: ResolverProtocol,
@@ -803,7 +852,6 @@ class DatasetRevisionImpl(DatasetRevision):
         _logger.info(f"loading Chalk DataFrame for DatasetRevision {self.revision_id}")
 
         if not self._hydrated:
-
             self._client.await_operation_completion(
                 operation_id=self.revision_id,
                 show_progress=show_progress,
@@ -818,7 +866,7 @@ class DatasetRevisionImpl(DatasetRevision):
             environment_id=self.environment,
             revision_id=revision_id,
             resolver_fqn=resolver_fqn,
-            branch=self._branch,
+            branch=self.branch,
             timeout=timeout,
         )
 
@@ -966,7 +1014,7 @@ This occurred during the actual execution of resolver {resolver.fqn}.
         dataset = self._client.get_anonymous_dataset(
             revision_id=str(self.revision_id),
             environment=self.environment,
-            branch=self._branch,
+            branch=self.branch,
         )
         completed_revision = dataset.revisions[-1]
         assert isinstance(completed_revision, DatasetRevisionImpl)
@@ -974,7 +1022,7 @@ This occurred during the actual execution of resolver {resolver.fqn}.
         self.outputs = completed_revision.outputs
         self.environment = completed_revision.environment
         self.revision_id = completed_revision.revision_id
-        self._branch = completed_revision._branch
+        self.branch = completed_revision.branch
         self.terminated_at = completed_revision.terminated_at
         self.started_at = completed_revision.started_at
         self.created_at = completed_revision.created_at
@@ -1171,6 +1219,15 @@ class DatasetImpl(Dataset):
             caller_name=caller_name,
         )
 
+    def wait(
+        self,
+        timeout: float | timedelta | ellipsis | None = ...,
+        show_progress: bool = True,
+        caller_name: str = "wait",
+    ) -> DatasetImpl:
+        self.revisions[-1].wait(timeout=timeout, show_progress=show_progress, caller_name=caller_name)
+        return self
+
     def download_data(
         self,
         path: str,
@@ -1214,35 +1271,49 @@ class DatasetImpl(Dataset):
         self,
         features: Optional[List[Union[Feature, Any]]] = None,
         branch: Optional[str] = None,
-        wait: bool = False,
+        wait: bool = True,
         show_progress: bool = False,
+        store_plan_stages: bool = False,
+        correlation_id: str | None = None,
+        explain: Union[bool, Literal["only"]] = False,
+        tags: Optional[List[str]] = None,
+        required_resolver_tags: Optional[List[str]] = None,
+        planner_options: Optional[Mapping[str, Union[str, int, bool]]] = None,
+        use_multiple_computers: bool = False,
         timeout: float | timedelta | ellipsis | None = ...,
     ) -> Dataset:
         if len(self.revisions) == 0:
             raise IndexError("No revisions exist for dataset")
-        branch = branch or self._client.config.branch
+        branch = branch or self._client.config.branch or self.revisions[-1].branch
         if branch is None:
             raise ValueError("A branch was not provided to the chalk client or as an argument to recompute.")
 
         recompute_response = self._client.recompute_dataset(
             dataset_name=self.dataset_name,
-            dataset_id=self.dataset_id,
             revision_id=self.revisions[-1].revision_id,
-            features=features or [],
+            features=features,
             branch=branch,
             environment=self.environment,
+            wait=wait,
+            show_progress=show_progress,
+            correlation_id=correlation_id,
+            store_plan_stages=store_plan_stages,
+            explain=explain,
+            tags=tags,
+            required_resolver_tags=required_resolver_tags,
+            planner_options=planner_options,
+            use_multiple_computers=use_multiple_computers,
         )
-        if recompute_response.errors:
-            raise ChalkBaseException(errors=recompute_response.errors)
-        new_revision = dataset_revision_from_response(recompute_response, self._client)
-        self.revisions.append(new_revision)
-
-        if not wait:
-            return self
-
-        new_revision.wait_for_completion(show_progress=show_progress, timeout=timeout)
+        self.revisions.append(recompute_response.revisions[-1])
         return self
 
+    def ingest(
+        self,
+        store_online: bool = False,
+        store_offline: bool = True,
+    ) -> DatasetImpl:
+        return self.revisions[-1].ingest(store_online=store_online, store_offline=store_offline)
+
     def resolver_replay(
         self,
         resolver: ResolverProtocol,
@@ -1266,6 +1337,14 @@ class DatasetImpl(Dataset):
             return f"Dataset(name='{self.dataset_name}', version='{self.version}')"
         return f"Dataset(name=<unnamed>)"
 
+    @property
+    def url(self) -> str | None:
+        return self.revisions[-1].dashboard_url
+
+    @property
+    def dashboard_url(self) -> str | None:
+        return self.revisions[-1].dashboard_url
+
 
 def dataset_revision_from_response(
     revision: Union[DatasetRevisionResponse, DatasetRecomputeResponse], client: ChalkAPIClientImpl
