@@ -1,10 +1,13 @@
+from __future__ import annotations
+
 from typing import List, Optional
 
 from typing_extensions import Self
 
 from chalk.client.models import ChalkError
 
-_CHALK_TRACE_ID_KEY = "trace"
+CHALK_TRACE_ID_KEY = "trace"
+_CHALK_TRACE_ID_KEY = CHALK_TRACE_ID_KEY  # for backcompat
 _CHALK_TRACE_ID_MESSAGE = "Please contact Chalk Support with the Trace ID"
 _CHALK_TRACE_ID_MISSING = "<trace ID unexpectedly missing>"
 
@@ -71,9 +74,15 @@ class ChalkBaseException(Exception):
 
 
 class ChalkCustomException(ChalkBaseException):
-    def __init__(self, message: str, errors: Optional[List[ChalkError]] = None, **kwargs):
+    def __init__(
+        self,
+        message: str,
+        errors: Optional[List[ChalkError]] = None,
+        trace_id: Optional[str] = None,
+        detail: Optional[str] = None,
+    ):
         self._message = message
-        super().__init__(errors=errors, **kwargs)
+        super().__init__(errors=errors, trace_id=trace_id, detail=detail)
 
     @property
     def message(self) -> str:
