@@ -1,6 +1,7 @@
 from __future__ import annotations
 
 import dataclasses
+import os
 import uuid
 from datetime import datetime
 from enum import Enum, IntEnum
@@ -10,6 +11,7 @@ import numpy as np
 from typing_extensions import TypeAlias
 
 from chalk.byte_transmit.model import ByteBaseModel, ByteDict
+from chalk.client._internal_models.models import OfflineQueryGivensVersion
 from chalk.features import Feature
 from chalk.features._encoding.json import FeatureEncodingOptions
 from chalk.features.resolver import Resolver
@@ -32,6 +34,8 @@ MAX_STR_LENGTH = 10_000
 
 FeatureReference: TypeAlias = Union[str, Any]
 
+_CHALK_DEBUG_FULL_TRACE = os.getenv("CHALK_DEBUG_FULL_TRACE") == "1"
+
 
 def _category_for_error_code(c: Union[ErrorCode, str]) -> ErrorCodeCategory:
     c = ErrorCode[c]
@@ -243,7 +247,7 @@ class ChalkError(BaseModel, frozen=True):
         """
         Returns True if the error indicates an issue with user's resolver, rather than an internal Chalk failure.
         """
-        return self.code in [ErrorCode.RESOLVER_FAILED, ErrorCode.RESOLVER_TIMED_OUT]
+        return self.code in [ErrorCode.RESOLVER_FAILED, ErrorCode.RESOLVER_TIMED_OUT, ErrorCode.UPSTREAM_FAILED]
 
     def copy_for_feature(self, feature: str) -> "ChalkError":
         return self.copy(update={"feature": feature})
@@ -254,7 +258,37 @@ class ChalkError(BaseModel, frozen=True):
     @root_validator
     def _validate_category(cls, values: Dict[str, Any]):
         values["category"] = _category_for_error_code(values["code"])
-        values["message"] = values["message"][0:MAX_STR_LENGTH]
+
+        if not _CHALK_DEBUG_FULL_TRACE:
+            # Truncate the message to a specified maximum length.
+            values["message"] = values["message"][0:MAX_STR_LENGTH]
+
+        _HAS_CHALK_TRACE = "[has chalk trace]"
+        if _CHALK_DEBUG_FULL_TRACE and _HAS_CHALK_TRACE not in values["message"]:
+            # Include a stack trace if it's not already present and the super-verbose
+            # full trace flag is enabled.
+            import traceback
+
+            formatted_stack = traceback.format_stack()[:-1]  # Exclude this validation function.
+            start_stack_from = 0
+            for i in range(len(formatted_stack)):
+                if "run_endpoint_function" in formatted_stack[i]:
+                    # This function occurs in the stack trace before the actual entry into the engine-
+                    # everything before it is boilerplate.
+                    start_stack_from = i + 1
+            values["message"] = (
+                values["message"]
+                + "\n"
+                + _HAS_CHALK_TRACE
+                + "\n"
+                + "[" * 200
+                + "\n"
+                + "\n".join(formatted_stack[start_stack_from:])
+                + "\n"
+                + "]" * 200
+                + "\n"
+            )
+
         return values
 
     if TYPE_CHECKING:
@@ -293,6 +327,12 @@ class WhoAmIResponse(BaseModel):
     user: str
     """The ID of the user or service token making the query."""
 
+    environment_id: Optional[str] = None
+    """The environment under which the client's queries will be run, unless overridden"""
+
+    team_id: Optional[str] = None
+    """The team ID pertaining to the client"""
+
 
 class FeatureResolutionMeta(BaseModel, frozen=True):
     """Detailed metadata about the execution of an online query."""
@@ -438,10 +478,17 @@ class MultiUploadFeaturesResponse(BaseModel):
     errors: List[ChalkError]
 
 
+class PersistenceSettings(BaseModel):
+    persist_online_storage: Optional[bool] = None
+    persist_offline_storage: Optional[bool] = None
+
+
 class TriggerResolverRunRequest(BaseModel):
     resolver_fqn: str
     upper_bound: Optional[str] = None
     lower_bound: Optional[str] = None
+    timestamping_mode: Literal["feature_time", "online_store_write_time"] = "feature_time"
+    persistence_settings: Optional[PersistenceSettings] = None
 
 
 class QueryMeta(BaseModel):
@@ -555,6 +602,20 @@ class BulkOnlineQueryResponse:
         return self.results[item]
 
 
+class SpineSqlRequest(BaseModel):
+    sql_query: str
+    primary_feature_column_name: Optional[str] = None
+
+
+class UploadedParquetShardedOfflineQueryInput(BaseModel):
+    """
+    Offline query input that is sharded parquet files uploaded to cloud storage.
+    """
+
+    filenames: Tuple[str, ...] = Field("A list of filenames of the sharded parquet files")
+    version: OfflineQueryGivensVersion = Field("Version of how the inputs is represented in a table")
+
+
 class CreateOfflineQueryJobRequest(BaseModel):
     output: List[str] = Field(description="A list of output feature root fqns to query")
     required_output: List[str] = Field(default_factory=list, description="A list of required output feature root fqns")
@@ -566,7 +627,9 @@ class CreateOfflineQueryJobRequest(BaseModel):
             "then jobs with the same ID will be rejected."
         ),
     )
-    input: Union[OfflineQueryInput, Tuple[OfflineQueryInput, ...], None] = Field(default=None, description="Any givens")
+    input: Union[
+        OfflineQueryInput, Tuple[OfflineQueryInput, ...], None, UploadedParquetShardedOfflineQueryInput
+    ] = Field(default=None, description="Any givens")
     max_samples: Optional[int] = Field(
         default=None,
         description="The maximum number of samples. If None, no limit",
@@ -589,7 +652,8 @@ class CreateOfflineQueryJobRequest(BaseModel):
     )
     dataset_name: Optional[str] = None
     branch: Optional[str] = None
-    recompute_features: Union[bool, List[FeatureReference]] = False
+    recompute_features: Union[bool, List[str]] = False
+    sample_features: Optional[List[str]] = None
     store_plan_stages: bool = False
     explain: Union[bool, Literal["only"]] = False
     tags: Optional[List[str]] = None
@@ -604,12 +668,19 @@ class CreateOfflineQueryJobRequest(BaseModel):
     planner_options: Optional[Any] = None
     use_multiple_computers: bool = False
 
-    spine_sql_query: Optional[str] = None
+    spine_sql_query: Optional[Union[str, SpineSqlRequest]] = None
+
+    recompute_request_revision_id: Optional[str] = None
 
     @root_validator
     def _validate_multiple_computers(cls, values: Dict[str, Any]):
-        if values["input"] is not None and values["use_multiple_computers"] != isinstance(values["input"], tuple):
-            raise ValueError("input should be tuple exactly when use_multiple_computers is True")
+        if values["input"] is None:
+            return values
+        expected_use_multiple_computers = isinstance(values["input"], tuple) or isinstance(
+            values["input"], UploadedParquetShardedOfflineQueryInput
+        )
+        if values["use_multiple_computers"] != expected_use_multiple_computers:
+            raise ValueError("input should be tuple or uploaded shards exactly when use_multiple_computers is True")
         return values
 
 
@@ -680,6 +751,8 @@ class CreateOfflineQueryJobResponse(BaseModel):
 
 
 class ColumnMetadata(BaseModel):
+    """This entire model is deprecated."""
+
     feature_fqn: str = Field(description="The root FQN of the feature for a column")
 
     column_name: str = Field(description="The name of the column that corresponds to this feature")
@@ -703,6 +776,7 @@ class GetOfflineQueryJobResponse(BaseModel):
         description="A list of short-lived, authenticated URLs that the client can download to retrieve the exported data."
     )
     errors: Optional[List[ChalkError]] = None
+    # deprecated
     columns: Optional[List[ColumnMetadata]] = Field(
         description="Expected columns for the dataframe, including data type information",
         default=None,
@@ -930,6 +1004,8 @@ class GetIncrementalProgressResponse(BaseModel):
     The fully qualified name of the given resolver
     """
 
+    query_name: Optional[str] = None
+
     max_ingested_timestamp: Optional[datetime]
     """
     The latest timestamp found in ingested data.
@@ -1051,6 +1127,11 @@ class StreamResolverTestResponse(BaseModel):
         return read_parquet(self.data_uri)
 
 
+class FeatherBodyType(str, Enum):
+    TABLE = "TABLE"
+    RECORD_BATCHES = "RECORD_BATCHES"
+
+
 class OnlineQueryResultFeather(ByteBaseModel):
     has_data: bool
     scalar_data: bytes
@@ -1109,4 +1190,21 @@ class PlanQueryResponse(BaseModel):
     errors: List[ChalkError]
 
 
-##
+class IngestDatasetRequest(BaseModel):
+    revision_id: str = Field(description="The ID of the dataset revision to ingest")
+    branch: Optional[str] = Field(description="The branch to ingest the dataset into")
+    outputs: List[str] = Field(description="The output features to return from the dataset")
+    store_online: bool = Field(description="Whether to store the dataset into the online store")
+    store_offline: bool = Field(description="Whether to store the dataset into the offline store")
+
+
+class AnnotatedSignedUploadURL(BaseModel):
+    signed_url: str = Field(description="Signed URLs which can be uploaded to using PUT requests")
+    filename: str = Field(description="Filenames which the signed URLs correspond to")
+
+
+class OfflineQueryParquetUploadURLResponse(BaseModel):
+    # there is one pair of url for each partition
+    urls: Tuple[AnnotatedSignedUploadURL, ...] = Field(
+        description="Signed URLs which can be uploaded to using PUT requests"
+    )
