@@ -1,3 +1,5 @@
+from __future__ import annotations
+
 import os
 from functools import lru_cache
 from pathlib import Path
@@ -31,13 +33,11 @@ class AuthConfig(BaseModel):
     tokens: Optional[Mapping[str, TokenConfig]]
 
 
-def _load_global_config_uncached():
-    home = os.getenv("XDG_CONFIG_HOME", os.path.expanduser("~"))
+def _load_global_config_uncached() -> AuthConfig | None:
     path = None
-    for filename in [".chalk.yml"]:
-        p = os.path.join(home, filename)
-        if os.path.exists(p):
-            path = p
+    p = os.path.join(os.getenv("XDG_CONFIG_HOME", os.path.expanduser("~")), ".chalk.yml")
+    if os.path.exists(p):
+        path = p
 
     if path is None:
         return None
@@ -49,11 +49,11 @@ def _load_global_config_uncached():
 
 # using lru_cache for 3.8 compat
 @lru_cache(maxsize=None)
-def _load_global_config() -> Optional[AuthConfig]:
+def _load_global_config() -> AuthConfig | None:
     return _load_global_config_uncached()
 
 
-def _en(current_env_name: str, old_env_name: Optional[str] = None) -> Optional[str]:
+def _en(current_env_name: str, old_env_name: str | None = None) -> str | None:
     old_env_name = old_env_name or f"_{current_env_name}"
     # Backwards compatability for old env vars.
     deprecated = os.getenv(old_env_name)
@@ -71,7 +71,7 @@ def load_token(
     active_environment: Optional[str],
     api_server: Optional[str],
     skip_cache: bool = False,
-) -> Optional[TokenConfig]:
+) -> TokenConfig | None:
     token_name = "Explicit" if client_id and client_secret else "Environment"
 
     client_id = client_id or _en("CHALK_CLIENT_ID")
