@@ -1,12 +1,18 @@
+from __future__ import annotations
+
 import ast
+import builtins
 import inspect
 from types import FrameType
-from typing import Any, Callable, Dict, Optional, Union
+from typing import Any, Callable, Dict, Optional, Type, Union
 
 from executing import Source
 
-from chalk.features import Filter
 from chalk.features._chalkop import op
+from chalk.features.feature_field import Feature
+from chalk.features.feature_wrapper import FeatureWrapper
+from chalk.features.filter import Filter
+from chalk.utils.collections import get_unique_item
 
 
 class ExecutingException(Exception):
@@ -41,9 +47,16 @@ def _get_node_by_frame(frame: FrameType, raise_exc: bool = True) -> Optional[ast
 
 
 def _get_func_frame_and_nodes(condition: Callable[[Optional[ast.AST]], bool]):
-    frames = inspect.stack()
-    for frame_info in frames[3:]:
-        frame = frame_info.frame
+    frame = inspect.currentframe()
+    assert frame is not None
+    # We want to go back 3 frames -- get out of this function, out of parse_feature_iter, and out
+    # of df __getitem__
+    frame = frame.f_back
+    assert frame is not None
+    frame = frame.f_back
+    assert frame is not None
+    frame = frame.f_back
+    while frame is not None:
         func_node = _get_node_by_frame(frame)
         if condition(func_node):
             # This is the correct getitem frame.
@@ -53,11 +66,12 @@ def _get_func_frame_and_nodes(condition: Callable[[Optional[ast.AST]], bool]):
             # def __getitem__(self, item):
             #    return self.df[item]  # <--- item is of type ast.name. We need to go one frame higher!
             return frame, func_node
+        frame = frame.f_back
     raise RuntimeError("Condition not found in stack")
 
 
-def parse_feature_iter(f):
-    func_frame, func_node = _get_func_frame_and_nodes(lambda node: isinstance(node, ast.Call))
+def parse_feature_iter(f: Feature | FeatureWrapper | Any):
+    _, func_node = _get_func_frame_and_nodes(lambda node: isinstance(node, ast.Call))
     if not isinstance(func_node, ast.Call):
         raise RuntimeError("Could not evaluate function")
     if not isinstance(func_node.func, ast.Name):
@@ -86,6 +100,35 @@ def parse_dataframe_getitem():
     return eval_converted_expr(converted_slice, glbs=func_frame.f_globals, lcls=func_frame.f_locals)
 
 
+def parse_inline_setattr_annotation(key: str) -> Optional[Type[Any]]:
+    """Parses the type annotation for inline feature definitions."""
+    # Get the frame when the attribute is set
+    frame = inspect.currentframe()
+    for _ in range(2):
+        assert frame is not None
+        frame = frame.f_back
+    assert frame is not None
+
+    try:
+        source = Source.executing(frame)
+        node = source.node
+        parent_node = node.parent
+
+        if isinstance(parent_node, ast.AnnAssign) and isinstance(node, ast.Attribute):
+            attribute_name = node.attr
+            if attribute_name == key:
+                if isinstance(parent_node.annotation, ast.Name):
+                    type_name = parent_node.annotation.id
+
+                    if hasattr(builtins, type_name):
+                        return getattr(builtins, type_name)
+                    elif type_name in frame.f_globals:
+                        return frame.f_globals[type_name]
+    except:
+        raise TypeError(f"Failed to parse type annotation for feature {key}.")
+    return None
+
+
 def parse_when() -> Optional[Filter]:
     func_frame, func_node = _get_func_frame_and_nodes(lambda node: isinstance(node, ast.Call))
     assert isinstance(func_node, ast.Call)
@@ -162,9 +205,8 @@ def _convert_ops(stmt: ast.expr):
     if isinstance(stmt, ast.Compare):
         if len(stmt.ops) == 1:
             lhs = stmt.left
-            assert len(stmt.comparators) == 1
-            rhs = stmt.comparators[0]
-            compare_op: ast.cmpop = stmt.ops[0]
+            rhs = get_unique_item(stmt.comparators)
+            compare_op: ast.cmpop = get_unique_item(stmt.ops)
             # Replace is with == and isnot with !=
             # It doesn't make sense to have identity checks in a dataframe filter
 
