@@ -1,5 +1,4 @@
-from collections.abc import Mapping as MappingABC
-from typing import Any, Dict, List, Mapping, Tuple
+from typing import Any, Dict, List, Mapping, Tuple, cast
 
 from chalk.features import Features, TPrimitive, ensure_feature
 from chalk.features._encoding.json import unstructure_primitive_to_json
@@ -8,9 +7,9 @@ from chalk.utils.json import TJSON
 
 
 def _recursive_unstructure_primitive_to_json(val: TPrimitive) -> TJSON:
-    if isinstance(val, MappingABC):
-        return {k: _recursive_unstructure_primitive_to_json(v) for k, v in val.items()}
-    elif isinstance(val, List):
+    if isinstance(val, dict):
+        return {cast(str, k): _recursive_unstructure_primitive_to_json(v) for k, v in val.items()}
+    elif isinstance(val, (list, tuple)):
         return [_recursive_unstructure_primitive_to_json(x) for x in val]
     else:
         return unstructure_primitive_to_json(val)
@@ -40,7 +39,7 @@ def recursive_encode_inputs(inputs: Mapping[str, Any]) -> Tuple[Dict[str, TJSON]
                 # The value can be either a feature instance or a dict
                 if isinstance(item, Features):
                     item = dict(item.items())
-                if not isinstance(item, MappingABC):
+                if not isinstance(item, dict):
                     raise TypeError(
                         (
                             f"Has-many feature '{feature.root_fqn}' must be a list of dictionaries or feature set instances, "
@@ -64,7 +63,7 @@ def recursive_encode_inputs(inputs: Mapping[str, Any]) -> Tuple[Dict[str, TJSON]
             # The value can be either a feature instance or a dict
             if isinstance(v, Features):
                 v = dict(v.items())
-            if not isinstance(v, MappingABC):
+            if not isinstance(v, dict):
                 raise TypeError(
                     (
                         f"Has-one feature '{feature.root_fqn}' must be a list of dictionaries or feature set instances, "
