@@ -7,7 +7,7 @@ import ipaddress
 import typing
 import uuid
 from datetime import date, datetime, time, timedelta
-from typing import TYPE_CHECKING, Any, Dict, FrozenSet, List, Set, Tuple, Type, Union, cast
+from typing import TYPE_CHECKING, Any, Dict, FrozenSet, List, Set, Tuple, Type, cast
 
 import attrs
 import pyarrow as pa
@@ -45,18 +45,13 @@ def _is_chalk_dataframe(typ: Any) -> TypeGuard[type[DataFrame]]:
     return isinstance(unwrapped, type) and issubclass(unwrapped, DataFrame)
 
 
-def rich_to_pyarrow(
-    python_type: Type, name: str, in_struct: bool = False, parent_namespaces: Tuple[str, ...] = ()
-) -> pa.DataType:
+def rich_to_pyarrow(python_type: Type, name: str, in_struct: bool = False) -> pa.DataType:
     """
     Recursively convert a python type to a PyArrow dtype.
     :param python_type:
     :param name:
     :param in_struct: Whether this function is being called recursively for a member of a struct type.
     Certain datatypes need to be handled differently if nested inside a struct.
-    :param parent_namespaces: List of parent namespaces for nested has-ones/has-manys.
-    If we run into an infinite loop, skip the field whose namespace we're already in
-    (e.g. When serializing Card.customer, skip Card.customer.card....)
     """
     # Polars seems to allow optional for any dtype, so we ignore it when computing dtypes
     python_type = unwrap_optional_and_annotated_if_needed(python_type)
@@ -70,7 +65,7 @@ def rich_to_pyarrow(
                 raise TypeError(
                     "Annotated types must contain the underlying type as the first argument -- e.g. Annotated[int, 'annotation']."
                 )
-            return rich_to_pyarrow(args[0], name, in_struct=in_struct, parent_namespaces=parent_namespaces)
+            return rich_to_pyarrow(args[0], name, in_struct=in_struct)
         if origin in (list, List, set, Set, frozenset, FrozenSet):
             typ_name = origin.__name__
             if len(args) == 0:
@@ -82,9 +77,7 @@ def rich_to_pyarrow(
                     f"{typ_name} annotations should only take one argument -- e.g. {typ_name}[int]. Instead, got {typ_name}[{', '.join(args)}]."
                 )
             arg = args[0]
-            return pa.large_list(
-                rich_to_pyarrow(arg, name=f"{name}[]", in_struct=in_struct, parent_namespaces=parent_namespaces)
-            )
+            return pa.large_list(rich_to_pyarrow(arg, name=f"{name}[]", in_struct=in_struct))
         if origin in (tuple, Tuple):
             if len(args) == 0:
                 raise TypeError(
@@ -93,9 +86,7 @@ def rich_to_pyarrow(
             if len(args) == 2 and args[1] is ...:
                 # Treat a variable-sized, homogenous tuple like a list
                 arg = args[0]
-                return pa.large_list(
-                    rich_to_pyarrow(arg, name=f"{name}[]", in_struct=in_struct, parent_namespaces=parent_namespaces)
-                )
+                return pa.large_list(rich_to_pyarrow(arg, name=f"{name}[]", in_struct=in_struct))
             raise TypeError(
                 (
                     "Tuple features must have a fixed type and be variable-length tuples (e.g. `Tuple[int, ...]`). "
@@ -118,11 +109,11 @@ def rich_to_pyarrow(
             try:
                 pa_value_type = rich_to_pyarrow(value_type, "value")
             except TypeError as e:
-                raise TypeError(f"Failed to determine type of dict value from the annotation `{value_type}`: {e}")
+                raise TypeError(f"Failed to determine type of dict value from the annotation `{value_type}`") from e
             else:
                 value_field = pa.field(name="value", type=pa_value_type, nullable=is_optional(value_type))
 
-            return pa.map_(key_type=rich_to_pyarrow(key_type, "key"), item_type=value_field)
+                return pa.map_(key_type=rich_to_pyarrow(key_type, "key"), item_type=value_field)
 
         raise TypeError(f"Unsupported varardic type annotation: {origin}")
     else:
@@ -142,22 +133,20 @@ def rich_to_pyarrow(
         raise TypeError(f"Type annotations must be a type. Instead, got {python_type}.")
     if issubclass(python_type, enum.Enum):
         # For enums, require all members to have the same type
-        return rich_to_pyarrow(
-            get_enum_value_type(python_type), name, in_struct=in_struct, parent_namespaces=parent_namespaces
-        )
+        return rich_to_pyarrow(get_enum_value_type(python_type), name, in_struct=in_struct)
     # First, handle the recursive types -- list, tuple, typeddict, namedtuple, dataclass, pydantic model
     if _is_features_cls(python_type):
-        parent_namespaces = (parent_namespaces or ()) + (python_type.namespace,)
         annotations = cached_get_type_hints(python_type)
         fields: List[pa.Field] = []
         for field_name, type_annotation in annotations.items():
-            if _is_infinitely_recursive_features_class(parent_namespaces, type_annotation):
+            if _is_features_cls(type_annotation) or _is_chalk_dataframe(type_annotation):
                 continue
-            underlying_dtype = rich_to_pyarrow(
-                type_annotation, name=f"{name}.{field_name}", in_struct=True, parent_namespaces=parent_namespaces
-            )
+            features_cls = unwrap_optional_and_annotated_if_needed(python_type)
+            feature_wrapper = getattr(features_cls, field_name)
+            underlying_dtype = feature_wrapper._chalk_get_underlying().converter.pyarrow_dtype
             # For a @features class, struct field names should include the namespace (i.e. be the root FQN)
             fields.append(pa.field(f"{python_type.namespace}.{field_name}", underlying_dtype))
+
         return pa.struct(fields)
     elif (
         dataclasses.is_dataclass(python_type)
@@ -169,26 +158,19 @@ def rich_to_pyarrow(
         annotations = cached_get_type_hints(python_type)
         fields: List[pa.Field] = []
         for field_name, type_annotation in annotations.items():
-            underlying_dtype = rich_to_pyarrow(
-                type_annotation, name=f"{name}.{field_name}", in_struct=True, parent_namespaces=parent_namespaces
-            )
+            if field_name.startswith("__"):
+                # Skip all dunders, like __slots__
+                continue
+            underlying_dtype = rich_to_pyarrow(type_annotation, name=f"{name}.{field_name}", in_struct=True)
             fields.append(pa.field(field_name, underlying_dtype))
         return pa.struct(fields)
     if _is_chalk_dataframe(python_type):
         if python_type.__pydantic_model__ is None:
-            assert python_type.namespace is not None
-            parent_namespaces = (parent_namespaces or ()) + (python_type.namespace,)
-            column_fields = []
-            for col in python_type.columns:
-                if _is_infinitely_recursive_features_class(parent_namespaces, col.typ.parsed_annotation):
-                    continue
-                else:
-                    column_fields.append((col.fqn, col.typ.parsed_annotation))
-            pa_fields = [
-                pa.field(fqn, rich_to_pyarrow(typ, name=fqn, in_struct=True, parent_namespaces=parent_namespaces))
-                for fqn, typ in column_fields
-            ]
-            return pa.large_list(pa.struct(pa_fields))
+            # `columns` is set to never contain has-ones or has-many's, so we don't need to worry about
+            # infinite recursion when calling `Feature.converter.pyarrow_dtype`
+            return pa.large_list(
+                pa.struct([pa.field(col.root_fqn, col.converter.pyarrow_dtype) for col in python_type.columns])
+            )
         else:
             return pa.large_list(rich_to_pyarrow(python_type=python_type.__pydantic_model__, name="item"))
     if issubclass(python_type, str):
@@ -234,35 +216,35 @@ def rich_to_pyarrow(
 
 def pyarrow_to_primitive(pyarrow_typ: pa.DataType, name: str) -> Type[TPrimitive]:
     if pa.types.is_null(pyarrow_typ):
-        return cast(Type[TPrimitive], None)
+        return type(None)
     if pa.types.is_boolean(pyarrow_typ):
-        return cast(Type[TPrimitive], bool)
+        return bool
     if pa.types.is_unsigned_integer(pyarrow_typ) or pa.types.is_signed_integer(pyarrow_typ):
-        return cast(Type[TPrimitive], int)
+        return int
     if pa.types.is_floating(pyarrow_typ):
-        return cast(Type[TPrimitive], float)
+        return float
     if pa.types.is_time(pyarrow_typ):
-        return cast(Type[TPrimitive], time)
+        return time
     if pa.types.is_timestamp(pyarrow_typ):
-        return cast(Type[TPrimitive], datetime)
+        return datetime
     if pa.types.is_date(pyarrow_typ):
-        return cast(Type[TPrimitive], date)
+        return date
     if pa.types.is_duration(pyarrow_typ):
-        return cast(Type[TPrimitive], timedelta)
+        return timedelta
     if (
         pa.types.is_binary(pyarrow_typ)
         or pa.types.is_fixed_size_binary(pyarrow_typ)
         or pa.types.is_large_binary(pyarrow_typ)
     ):
-        return cast(Type[TPrimitive], bytes)
+        return bytes
     if pa.types.is_string(pyarrow_typ) or pa.types.is_large_string(pyarrow_typ):
-        return cast(Type[TPrimitive], str)
+        return str
     if pa.types.is_list(pyarrow_typ) or pa.types.is_large_list(pyarrow_typ) or pa.types.is_fixed_size_list(pyarrow_typ):
-        pyarrow_typ = cast(Union[pa.ListType, pa.LargeListType, pa.FixedSizeListType], pyarrow_typ)
+        assert isinstance(pyarrow_typ, (pa.ListType, pa.LargeListType, pa.FixedSizeListType))
         underlying = pyarrow_typ.value_type
-        return cast(Type[TPrimitive], List[pyarrow_to_primitive(underlying, name=f"{name}[]")])
+        return List[pyarrow_to_primitive(underlying, name=f"{name}[]")]
     if pa.types.is_struct(pyarrow_typ):
-        pyarrow_typ = cast(pa.StructType, pyarrow_typ)
+        assert isinstance(pyarrow_typ, pa.StructType)
 
         annotations: Dict[str, Type[TPrimitive]] = {}
         schema = pa.schema(pyarrow_typ)
@@ -273,38 +255,19 @@ def pyarrow_to_primitive(pyarrow_typ: pa.DataType, name: str) -> Type[TPrimitive
             #     field_type = typing.Optional[field_type]
             annotations[sub_name] = field_type
         struct_type = ChalkStructType(f"__chalk_struct__{name}", (object,), annotations)
-        return cast(Type[TPrimitive], struct_type)
+        return cast("type[ChalkStructType]", struct_type)
     if pa.types.is_map(pyarrow_typ):
-        pyarrow_typ = cast(pa.MapType, pyarrow_typ)
+        assert isinstance(pyarrow_typ, pa.MapType)
         value_typ = pyarrow_to_primitive(pyarrow_typ.item_type, name=f"{name}::value")
         if pyarrow_typ.item_field.nullable:
             value_typ = typing.Optional[value_typ]
-        return cast(
-            Type[TPrimitive],
-            Dict[
-                pyarrow_to_primitive(pyarrow_typ.key_type, name=f"{name}::key"),
-                value_typ,
-            ],
-        )
+        return Dict[
+            pyarrow_to_primitive(pyarrow_typ.key_type, name=f"{name}::key"),
+            value_typ,
+        ]
     raise TypeError(f"Unsupported PyArrow type '{pyarrow_typ}' for field '{name}'.")
 
 
-def _is_infinitely_recursive_features_class(parent_namespaces: Tuple[str, ...], typ: object) -> bool:
-    """
-    If a python type is a Features Class, make sure it's not an infinite loop of nested has-manys or has-ones.
-    For instance, if there is a has-one relationship between Card and Customer, trying to serialize Card.customer.card
-    will lead to an infinite loop.
-    :param parent_namespaces: Series of namespaces that contain the current field. For Card.customer.card, this will be ["card", "customer"]
-    :param typ: Same format as Feature.typ.parsed_annotation
-    :return: True if `typ` is a nested has-one/many and its namespace is one of the parent_namespaces
-    """
-    typ = unwrap_optional_and_annotated_if_needed(typ)
-    if not (_is_chalk_dataframe(typ) or _is_features_cls(typ)):
-        return False
-    assert typ.namespace is not None
-    return typ.namespace in parent_namespaces
-
-
 def is_map_in_dtype_tree(dtype: pa.DataType) -> bool:
     if isinstance(dtype, pa.MapType):
         return True
@@ -355,13 +318,13 @@ def pyarrow_to_polars(pa_type: pa.DataType, name: str | None = None) -> pl.Polar
     if pa.types.is_time(pa_type):
         return pl.Time()
     if pa.types.is_timestamp(pa_type):
-        pa_type = cast(pa.TimestampType, pa_type)
+        assert isinstance(pa_type, pa.TimestampType)
         assert pa_type.unit in ("ms", "us", "ns")
         return pl.Datetime(pa_type.unit, pa_type.tz)
     if pa.types.is_date(pa_type):
         return pl.Date()
     if pa.types.is_duration(pa_type):
-        pa_type = cast(pa.DurationType, pa_type)
+        assert isinstance(pa_type, pa.DurationType)
         assert pa_type.unit in ("ms", "us", "ns")
         return pl.Duration(pa_type.unit)
     if pa.types.is_binary(pa_type) or pa.types.is_fixed_size_binary(pa_type) or pa.types.is_large_binary(pa_type):
@@ -369,22 +332,22 @@ def pyarrow_to_polars(pa_type: pa.DataType, name: str | None = None) -> pl.Polar
     if pa.types.is_string(pa_type) or pa.types.is_large_string(pa_type):
         return pl.Utf8()
     if pa.types.is_list(pa_type) or pa.types.is_large_list(pa_type) or pa.types.is_fixed_size_list(pa_type):
-        pa_type = cast(Union[pa.ListType, pa.LargeListType, pa.FixedSizeListType], pa_type)
+        assert isinstance(pa_type, (pa.ListType, pa.LargeListType, pa.FixedSizeListType))
         underlying = pa_type.value_type
         return pl.List(pyarrow_to_polars(underlying, name=f"{name}[]"))
     if pa.types.is_struct(pa_type):
-        pa_type = cast(pa.StructType, pa_type)
+        assert isinstance(pa_type, pa.StructType)
         schema = pa.schema(pa_type)
-
         fields = [
             pl.Field(sub_name, pyarrow_to_polars(sub_typ, name=f"{name}.{sub_name}"))
             for sub_name, sub_typ in zip(schema.names, schema.types)
         ]
         return pl.Struct(fields)
     if pa.types.is_decimal(pa_type):
+        assert isinstance(pa_type, (pa.Decimal128Type, pa.Decimal256Type))
         return pl.Decimal(precision=pa_type.precision, scale=pa_type.scale)
     if pa.types.is_map(pa_type):
-        pa_type = cast(pa.MapType, pa_type)
+        assert isinstance(pa_type, pa.MapType)
         fields = [
             pl.Field("key", pyarrow_to_polars(pa_type.key_type, name=f"{name}::key")),
             pl.Field("value", pyarrow_to_polars(pa_type.item_type, name=f"{name}::value")),
