@@ -14,19 +14,16 @@ import attrs
 import cattrs
 import dateutil.parser
 import isodate
-import pydantic
-from pydantic import BaseModel
+
+# Need the *real* BaseModel here (not the v1 one) so that we can register the right hook
+from pydantic import BaseModel as BaseBaseModel
 from typing_extensions import get_args, get_origin, is_typeddict
 
-from chalk.features._encoding.primitive import TPrimitive
+from chalk.features._encoding.primitive import ChalkStructType, TPrimitive
 from chalk.utils.cached_type_hints import cached_get_type_hints
 from chalk.utils.collections import is_namedtuple, unwrap_optional_and_annotated_if_needed
 from chalk.utils.enum import get_enum_value_type
-
-try:
-    import numpy as np
-except ImportError:
-    np = None
+from chalk.utils.pydanticutil.pydantic_compat import is_pydantic_basemodel
 
 TRich = TypeVar("TRich")
 T = TypeVar("T")
@@ -34,7 +31,6 @@ T = TypeVar("T")
 # Converter to go from TPython to a TPrimitive
 _rich_converter = cattrs.Converter()
 
-
 #########
 # Structs
 #########
@@ -47,7 +43,7 @@ _rich_converter.register_unstructure_hook_func(
     lambda x: {field.name: _rich_converter.unstructure(getattr(x, field.name)) for field in dataclasses.fields(x)},
 )
 _rich_converter.register_unstructure_hook(
-    BaseModel, lambda x: {k: _rich_converter.unstructure(v) for (k, v) in x.dict().items()}
+    BaseBaseModel, lambda x: {k: _rich_converter.unstructure(v) for (k, v) in x.dict().items()}
 )
 _rich_converter.register_unstructure_hook_func(
     attrs.has,
@@ -61,7 +57,7 @@ def _is_struct(typ: Type):
         is_namedtuple(typ)
         or dataclasses.is_dataclass(typ)
         or is_typeddict(typ)
-        or (isinstance(typ, type) and issubclass(typ, BaseModel))
+        or (isinstance(typ, type) and is_pydantic_basemodel(typ))
         or attrs.has(typ)
     )
 
@@ -80,7 +76,7 @@ def _structure_struct(obj: Any, typ: Type):
     if not isinstance(obj, collections.abc.Mapping):
         raise TypeError(f"Unable to structure object `{obj}` into type `{typ}`. Only dictionaries are supported.")
     kwargs = {k: _rich_converter.structure(v, type_hints[k]) for (k, v) in obj.items()}
-    if isinstance(typ, type) and issubclass(typ, BaseModel):
+    if isinstance(typ, type) and is_pydantic_basemodel(typ):
         # Using .construct to bypass pydantic validation, which could fail if there are null values in kwargs
         # and the field is not annotated as null
         return typ.construct(**kwargs)
@@ -88,7 +84,25 @@ def _structure_struct(obj: Any, typ: Type):
         return typ(**kwargs)
 
 
+def is_chalk_struct(typ: Type):
+    return isinstance(typ, ChalkStructType)
+
+
+def _structure_chalk_struct(obj: Any, typ: Type):
+    if not isinstance(typ, ChalkStructType):
+        raise TypeError(f"Expected ChalkStructType instead of {typ}")
+    type_hints = typ.__chalk_type_hints__
+    if obj is None:
+        obj = {k: None for k in type_hints}
+    if isinstance(obj, collections.abc.Sequence):
+        obj = {k: v for (k, v) in zip(type_hints.keys(), obj)}
+    if not isinstance(obj, collections.abc.Mapping):
+        raise TypeError(f"Unable to structure object `{obj}` into type `{typ}`. Only dictionaries are supported.")
+    return {k: _rich_converter.structure(v, type_hints[k]) for (k, v) in obj.items()}
+
+
 _rich_converter.register_structure_hook_func(_is_struct, _structure_struct)
+_rich_converter.register_structure_hook_func(is_chalk_struct, _structure_chalk_struct)
 
 #############
 # Collections
@@ -100,6 +114,7 @@ _rich_converter.register_structure_hook_func(_is_struct, _structure_struct)
 _rich_converter.register_unstructure_hook(set, lambda x: sorted(list(x)))
 _rich_converter.register_unstructure_hook(frozenset, lambda x: sorted(list(x)))
 
+
 # Lists and tuples do not need an unstructure hook -- the default is fine
 
 
@@ -144,7 +159,16 @@ def _structure_collection(
 
 def _is_collection(typ: Type):
     origin = get_origin(typ)
-    return origin in (set, Set, frozenset, FrozenSet, list, List, tuple, Tuple)
+    return origin in (
+        set,
+        Set,
+        frozenset,
+        FrozenSet,
+        list,
+        List,
+        tuple,
+        Tuple,
+    )
 
 
 _rich_converter.register_structure_hook_func(_is_collection, _structure_collection)
@@ -194,7 +218,6 @@ def _structure_decimal(obj: Any, typ: Type[decimal.Decimal]) -> decimal.Decimal:
 
 _rich_converter.register_structure_hook(decimal.Decimal, _structure_decimal)
 
-
 #######
 # Bytes
 #######
@@ -233,7 +256,6 @@ def _structure_timedelta(x: Any, typ: Type[timedelta]) -> timedelta:
 
 _rich_converter.register_structure_hook(timedelta, _structure_timedelta)
 
-
 ######
 # Date
 ######
@@ -258,7 +280,8 @@ def _structure_date(x: Any, typ: Type[date]) -> date:
 
 
 _rich_converter.register_structure_hook_func(
-    lambda x: isinstance(x, type) and issubclass(x, date) and not issubclass(x, datetime), _structure_date
+    lambda x: isinstance(x, type) and issubclass(x, date) and not issubclass(x, datetime),
+    _structure_date,
 )
 
 ######
@@ -280,7 +303,6 @@ def _structure_time(x: Any, typ: Type[time]) -> time:
 
 _rich_converter.register_structure_hook(time, _structure_time)
 
-
 ##########
 # Datetime
 ##########
@@ -302,7 +324,6 @@ def _structure_datetime(x: Any, typ: Type[datetime]) -> datetime:
 
 _rich_converter.register_structure_hook(datetime, _structure_datetime)
 
-
 ##########
 # UUID
 ##########
@@ -322,7 +343,6 @@ def _structure_uuid(x: Any, typ: Type[uuid.UUID]) -> uuid.UUID:
 
 _rich_converter.register_structure_hook(uuid.UUID, _structure_uuid)
 
-
 """
 ipaddress.IPv4Network
 """
@@ -363,13 +383,17 @@ def _structure_ipv6(x: Any, typ: Type[ipaddress.IPv6Address]) -> ipaddress.IPv6A
 
 _rich_converter.register_structure_hook(ipaddress.IPv6Address, _structure_ipv6)
 
-
 #############
 # Numpy types
 #############
-_rich_converter.register_unstructure_hook_func(lambda x: np is not None and np.issubdtype(x, np.bool_), bool)
-_rich_converter.register_unstructure_hook_func(lambda x: np is not None and np.issubdtype(x, np.integer), int)
-_rich_converter.register_unstructure_hook_func(lambda x: np is not None and np.issubdtype(x, np.floating), float)
+try:
+    import numpy as np
+except ImportError:
+    pass
+else:
+    _rich_converter.register_unstructure_hook_func(lambda x: np.issubdtype(x, np.bool_), bool)
+    _rich_converter.register_unstructure_hook_func(lambda x: np.issubdtype(x, np.integer), int)
+    _rich_converter.register_unstructure_hook_func(lambda x: np.issubdtype(x, np.floating), float)
 
 
 ##############
@@ -405,7 +429,7 @@ def _structure_rich(obj: Any, typ: Type):
         )
 
     if (
-        isinstance(obj, (list, set, frozenset, dict, tuple, pydantic.BaseModel))
+        isinstance(obj, (list, set, frozenset, dict, tuple, BaseBaseModel))
         or dataclasses.is_dataclass(obj)
         or attrs.has(type(obj))
     ):
