@@ -4,7 +4,6 @@ import collections.abc
 import datetime
 import enum
 import functools
-import weakref
 from typing import TYPE_CHECKING, Any, List, Mapping, Optional, Protocol, Sequence, TypeVar, Union
 
 import pyarrow as pa
@@ -13,7 +12,7 @@ from typing_extensions import Self, TypeGuard
 from chalk.features._encoding.converter import pyarrow_to_polars
 from chalk.features.feature_field import Feature
 from chalk.features.feature_wrapper import FeatureWrapper, unwrap_feature
-from chalk.features.filter import Filter, TimeDelta, _get_filter_now
+from chalk.features.filter import Filter, TimeDelta, get_filter_now
 from chalk.utils.collections import ensure_tuple
 
 if TYPE_CHECKING:
@@ -55,108 +54,6 @@ class ParsedExpr(Protocol):
         ...
 
 
-class PyArrowParsedExpr(ParsedExpr):
-    def __init__(self, col: Union[pa.Array, pa.ChunkedArray, pa.Scalar], table: weakref.ReferenceType[pa.Table] | None):
-        self._col = col
-        assert isinstance(self._col, pa.Scalar) == (
-            table is None
-        ), "The table binding should be provided iff an array is given"
-        self._table = table
-
-    @property
-    def col(self):
-        return self._col
-
-    @property
-    def table(self):
-        if self._table is None:
-            # Scalar value; not bound to anything
-            return None
-        tbl = self._table()
-        if tbl is None:
-            raise ValueError("Table went out of scope; this expression will be invalid")
-        return tbl
-
-    def _validate(self, __value: PyArrowParsedExpr, /):
-        if self.table is not None and __value.table is not None and self.table is not __value.table:
-            raise ValueError("The LHS is bound to a different table than the RHS")
-
-    def __eq__(self, __value: object, /) -> Self:
-        if not isinstance(__value, PyArrowParsedExpr):
-            raise TypeError("A PyArrowParsedExpr can only be compared to a PyArrowParsedExpr")
-        self._validate(__value)
-        __value = __value.col
-        return PyArrowParsedExpr(pa.compute.equal(self._col, __value), self._table)
-
-    def __ne__(self, __value: object, /) -> Self:
-        if not isinstance(__value, PyArrowParsedExpr):
-            raise TypeError("A PyArrowParsedExpr can only be compared to a PyArrowParsedExpr")
-        self._validate(__value)
-        __value = __value.col
-        return PyArrowParsedExpr(pa.compute.not_equal(self._col, __value), self._table)
-
-    def __gt__(self, __value: object, /) -> Self:
-        if not isinstance(__value, PyArrowParsedExpr):
-            raise TypeError("A PyArrowParsedExpr can only be compared to a PyArrowParsedExpr")
-        self._validate(__value)
-        __value = __value.col
-        return PyArrowParsedExpr(pa.compute.greater(self._col, __value), self._table)
-
-    def __lt__(self, __value: object, /) -> Self:
-        if not isinstance(__value, PyArrowParsedExpr):
-            raise TypeError("A PyArrowParsedExpr can only be compared to a PyArrowParsedExpr")
-        self._validate(__value)
-        __value = __value.col
-        return PyArrowParsedExpr(pa.compute.less(self._col, __value), self._table)
-
-    def __le__(self, __value: object, /) -> Self:
-        if not isinstance(__value, PyArrowParsedExpr):
-            raise TypeError("A PyArrowParsedExpr can only be compared to a PyArrowParsedExpr")
-        self._validate(__value)
-        __value = __value.col
-        return PyArrowParsedExpr(pa.compute.less_equal(self._col, __value), self._table)
-
-    def __ge__(self, __value: object, /) -> Self:
-        if not isinstance(__value, PyArrowParsedExpr):
-            raise TypeError("A PyArrowParsedExpr can only be compared to a PyArrowParsedExpr")
-        self._validate(__value)
-        __value = __value.col
-        return PyArrowParsedExpr(pa.compute.greater_equal(self._col, __value), self._table)
-
-    def __invert__(self) -> Self:
-        return PyArrowParsedExpr(pa.compute.invert(self._col), self._table)
-
-    def __add__(self, __value: object, /) -> Self:
-        if not isinstance(__value, PyArrowParsedExpr):
-            raise TypeError("A PyArrowParsedExpr can only be compared to a PyArrowParsedExpr")
-        if self.table is not None and __value.table is not None and self.table is not __value.table:
-            raise ValueError("The LHS is bound to a different table than the RHS")
-        __value = __value.col
-        return PyArrowParsedExpr(pa.compute.add_checked(self._col, __value), self._table)
-
-    def __sub__(self, __value: object, /) -> Self:
-        if not isinstance(__value, PyArrowParsedExpr):
-            raise TypeError("A PyArrowParsedExpr can only be compared to a PyArrowParsedExpr")
-        if self.table is not None and __value.table is not None and self.table is not __value.table:
-            raise ValueError("The LHS is bound to a different table than the RHS")
-        __value = __value.col
-        return PyArrowParsedExpr(pa.compute.subtract_checked(self._col, __value), self._table)
-
-    def __and__(self, __value: object, /) -> Self:
-        if not isinstance(__value, PyArrowParsedExpr):
-            raise TypeError("A PyArrowParsedExpr can only be compared to a PyArrowParsedExpr")
-        self._validate(__value)
-        __value = __value.col
-        return PyArrowParsedExpr(pa.compute.and_(self._col, __value), self._table)
-
-    def __or__(self, __value: object, /) -> Self:
-        if not isinstance(__value, PyArrowParsedExpr):
-            raise TypeError("A PyArrowParsedExpr can only be compared to a PyArrowParsedExpr")
-        self._validate(__value)
-        __value = __value.col
-        return PyArrowParsedExpr(pa.compute.or_(self._col, __value), self._table)
-
-
 TExpr = TypeVar("TExpr", bound=ParsedExpr)
 
 
@@ -166,6 +63,9 @@ class FilterConverter(Protocol[TExpr]):
     that can be used to filter tables of the corresponding type.
     """
 
+    def cast(self, expr: TExpr, dtype: pa.DataType) -> TExpr:
+        ...
+
     def _is_expr(self, o: Any) -> TypeGuard[TExpr]:
         """Need this for isinstance() checks, TypeVar is insufficient :-("""
         ...
@@ -174,7 +74,7 @@ class FilterConverter(Protocol[TExpr]):
         """Expression for selecting a column by the given name"""
         ...
 
-    def lit(self, value: Any, dtype: pa.DataType) -> TExpr:
+    def lit(self, value: Any, dtype: pa.DataType[Any]) -> TExpr:
         """Expression for a literal value"""
         ...
 
@@ -222,6 +122,7 @@ class FilterConverter(Protocol[TExpr]):
     def _eq_missing(self, lhs: TExpr, rhs: TExpr, dtype: pa.DataType) -> TExpr:
         # If either side is a struct, then recursively do eq_missing on each struct member
         if pa.types.is_struct(dtype):
+            assert isinstance(dtype, pa.StructType)
             sub_exprs: List[TExpr] = []
             sa = self._get_struct_adapter(dtype)
             for f in dtype:
@@ -241,11 +142,15 @@ class FilterConverter(Protocol[TExpr]):
             sub_ans = lhs == rhs
         return self.if_else(
             self.is_null(lhs) == self.is_null(rhs),
-            self.fill_null(  # If both sides have the same nullability, return the sub answer, coalescing null values to true (b/c we want null==null)
-                sub_ans, self.lit(True, pa.bool_())
+            self.fill_null(
+                # If both sides have the same nullability, return the sub answer, coalescing null values to true (b/c we want null==null)
+                sub_ans,
+                self.lit(True, pa.bool_()),
             ),
-            self.lit(  # If both sides have different nullability, then return False, because Null != (anything other than null)
-                False, pa.bool_()
+            self.lit(
+                # If both sides have different nullability, then return False, because Null != (anything other than null)
+                False,
+                pa.bool_(),
             ),
         )
 
@@ -280,8 +185,9 @@ class FilterConverter(Protocol[TExpr]):
         if isinstance(lhs, Feature):
             lhs_converter = lhs.converter
             col_name = str(lhs)
-            if col_name not in df_schema:
-                raise KeyError(f"Feature {col_name} not in DataFrame with columns [{', '.join(df_schema.keys())}]")
+            key_error_or_none = dataframe_missing_key_error([col_name], tuple(df_schema.keys()))
+            if key_error_or_none is not None:
+                raise key_error_or_none
             lhs = self.col(col_name)
             dtype = lhs_converter.pyarrow_dtype
 
@@ -289,8 +195,9 @@ class FilterConverter(Protocol[TExpr]):
         if isinstance(rhs, Feature):
             rhs_converter = rhs.converter
             col_name = str(rhs)
-            if col_name not in df_schema:
-                raise KeyError(f"Feature {col_name} not in DataFrame with columns [{', '.join(df_schema.keys())}]")
+            key_error_or_none = dataframe_missing_key_error([col_name], tuple(df_schema.keys()))
+            if key_error_or_none is not None:
+                raise key_error_or_none
             rhs = self.col(col_name)
             dtype = rhs_converter.pyarrow_dtype
         assert dtype is not None, "One side must be an expression"
@@ -415,6 +322,9 @@ class PolarsFilterConverter(FilterConverter["pl.Expr"]):
 
         return isinstance(o, pl.Expr)
 
+    def cast(self, expr: pl.Expr, dtype: pa.DataType) -> pl.Expr:
+        raise NotImplemented
+
     def col(self, name: str) -> pl.Expr:
         import polars as pl
 
@@ -474,86 +384,6 @@ class PolarsFilterConverter(FilterConverter["pl.Expr"]):
         return _PolarsStructAdapter(dtype)
 
 
-class PyArrowFilterConverter(FilterConverter[ParsedExpr]):
-    def __init__(self, tbl: pa.Table):
-        self._table = tbl
-
-    def _is_expr(self, o: Any) -> TypeGuard[PyArrowParsedExpr]:
-        return isinstance(o, PyArrowParsedExpr)
-
-    def col(self, name: str) -> ParsedExpr:
-        return PyArrowParsedExpr(self._table.column(name), weakref.ref(self._table))
-
-    def lit(self, value: Any, dtype: pa.DataType) -> ParsedExpr:
-        return PyArrowParsedExpr(pa.scalar(value, dtype), None)
-
-    def is_null(self, expr: ParsedExpr) -> ParsedExpr:
-        if not isinstance(expr, PyArrowParsedExpr):
-            raise TypeError(f"Expected a PyArrow filter expression; got {type(expr)}")
-        if isinstance(expr.col, pa.Scalar):
-            is_null = pa.scalar(not expr.col.is_valid, pa.bool_())
-            ref = None
-        else:
-            is_null = expr.col.is_null()
-            if expr.table is not self._table:
-                raise ValueError("Table references are different")
-            ref = weakref.ref(self._table)
-        return PyArrowParsedExpr(is_null, ref)
-
-    def is_not_null(self, expr: ParsedExpr) -> ParsedExpr:
-        if not isinstance(expr, PyArrowParsedExpr):
-            raise TypeError(f"Expected a PyArrow filter expression; got {type(expr)}")
-        if isinstance(expr.col, pa.Scalar):
-            is_not_null = pa.scalar(expr.col.is_valid, pa.bool_())
-            ref = None
-        else:
-            is_not_null = expr.col.is_valid()
-            if expr.table is not self._table:
-                raise ValueError("Table references are different")
-            ref = weakref.ref(self._table)
-        return PyArrowParsedExpr(is_not_null, ref)
-
-    def if_else(self, cond: ParsedExpr, true_case: ParsedExpr, false_case: ParsedExpr):
-        if not isinstance(cond, PyArrowParsedExpr):
-            raise TypeError(f"Expected a PyArrow filter expression for the condition; got {type(cond)}")
-        table_ref = cond.table
-        if table_ref is not None and table_ref != self._table:
-            raise ValueError("Table references are different")
-        if not isinstance(true_case, PyArrowParsedExpr):
-            raise TypeError(f"Expected a PyArrow filter expression for the true case; got {type(true_case)}")
-        if true_case.table is not None and true_case.table != self._table:
-            raise ValueError("Table references are different")
-        if not isinstance(false_case, PyArrowParsedExpr):
-            raise TypeError(f"Expected a PyArrow filter expression for the false case; got {type(false_case)}")
-        if false_case.table is not None and false_case.table != self._table:
-            raise ValueError("Table references are different")
-        return PyArrowParsedExpr(
-            pa.compute.if_else(cond.col, true_case.col, false_case.col),
-            None if table_ref is None else weakref.ref(table_ref),
-        )
-
-    def _get_struct_adapter(self, dtype: pa.StructType) -> StructAdapter[ParsedExpr]:
-        return _PyArrowStructAdapter(dtype)
-
-    def convert_filters(
-        self,
-        filters: Sequence[Filter],
-        df_schema: Mapping[str, pa.DataType],
-        timestamp_feature: Optional[Feature] = None,
-        now: Optional[Union[datetime.datetime, TExpr]] = None,
-        now_col_name: Optional[str] = None,
-    ) -> Optional[Union[pa.Array, pa.ChunkedArray]]:
-        ans = super().convert_filters(filters, df_schema, timestamp_feature, now, now_col_name)
-        if ans is None:
-            return None
-        # Need to convert from a PyArrowParsedExpr to a column, so it can be passed directly into a PyArrow table
-        assert isinstance(ans, PyArrowParsedExpr)
-        if isinstance(ans.col, pa.Scalar):
-            # If it's a scalar, extend it to an array
-            return pa.nulls(len(self._table), pa.bool_()).fill_null(ans.col)
-        return ans.col
-
-
 def convert_filters_to_pl_expr(
     filters: Sequence[Filter],
     df_schema: Mapping[str, pl.PolarsDataType],
@@ -581,6 +411,7 @@ class StructAdapter(Protocol[TExpr]):
 
 class _PolarsStructAdapter(StructAdapter["pl.Expr"]):
     def __init__(self, struct_type: pa.StructType):
+        super().__init__()
         self._struct_type = struct_type
         self._name_to_dtype = {f.name: f.type for f in struct_type}
 
@@ -588,31 +419,6 @@ class _PolarsStructAdapter(StructAdapter["pl.Expr"]):
         return expr.struct.field(name), self._name_to_dtype[name]
 
 
-class _PyArrowStructAdapter(StructAdapter[ParsedExpr]):
-    def __init__(self, struct_type: pa.StructType):
-        self._struct_type = struct_type
-        self._name_to_dtype = {f.name: f.type for f in struct_type}
-
-    def get_expr_for_field(self, expr: ParsedExpr, name: str) -> tuple[ParsedExpr, pa.DataType]:
-        if not isinstance(expr, PyArrowParsedExpr):
-            raise TypeError(f"Expected a PyArrow filter expression; got {type(expr)}")
-        table_ref = None if expr.table is None else weakref.ref(expr.table)
-
-        if isinstance(expr.col, pa.Scalar):
-            assert isinstance(expr.col, pa.StructScalar)
-            return PyArrowParsedExpr(expr.col.get(name), table_ref), self._name_to_dtype[name]
-        if isinstance(expr.col, pa.Array):
-            assert isinstance(expr.col, pa.StructArray)
-            return PyArrowParsedExpr(expr.col.field(name), table_ref), self._name_to_dtype[name]
-        else:
-            assert isinstance(expr.col, pa.ChunkedArray)
-            projected_chunks: list[pa.Array] = []
-            for chunk in expr.col.chunks:
-                assert isinstance(chunk, pa.StructArray)
-                projected_chunks.append(chunk.field(name))
-            return PyArrowParsedExpr(pa.chunked_array(projected_chunks), table_ref), self._name_to_dtype[name]
-
-
 def filter_data_frame(
     item: Any,
     underlying: Union[pl.DataFrame, pl.LazyFrame],
@@ -634,7 +440,11 @@ def filter_data_frame(
             raise TypeError(
                 "When indexing by Filters or Features, it is not simultaneously possible to perform other indexing operations."
             )
-    now = _get_filter_now()
+    now = get_filter_now()
+    if len(projections) > 0:
+        key_error_or_none = dataframe_missing_key_error(projections, underlying.columns)
+        if key_error_or_none is not None:
+            raise key_error_or_none
     # now = datetime.datetime.now(tz=datetime.timezone.utc)
     timestamp_feature = None if namespace is None else FeatureSetBase.registry[namespace].__chalk_ts__
     pl_expr = convert_filters_to_pl_expr(filters, underlying.schema, timestamp_feature, now)
@@ -643,15 +453,18 @@ def filter_data_frame(
         df = df.filter(pl_expr)
     # Do the projection
     if len(projections) > 0:
-        polars_cols_set = set(df.columns)
-        missing_cols = [c for c in projections if c not in polars_cols_set]
-        if len(missing_cols) > 0:
-            raise KeyError(
-                (
-                    f"Attempted to select missing columns [{', '.join(sorted(missing_cols))}] "
-                    f"from DataFrame with columns [{', '.join(sorted(list(polars_cols_set)))}]"
-                )
-            )
-
         df = df.select(projections)
     return df
+
+
+def dataframe_missing_key_error(prospective_columns: Sequence[str], schema: Sequence[str]) -> Union[None, KeyError]:
+    cols_set = set(schema)
+    missing_cols = [c for c in prospective_columns if c not in cols_set]
+    if len(missing_cols) > 0:
+        return KeyError(
+            (
+                f"Attempted to select missing columns [{', '.join(sorted(missing_cols))}] "
+                f"from DataFrame with columns [{', '.join(sorted(list(cols_set)))}]"
+            )
+        )
+    return None
