@@ -112,6 +112,8 @@ class DataFrameMeta(type):
                     context = cls.references_feature_set
                     is_pydantic = False
                 a = parse_underscore_in_context(exp=a, context=context, is_pydantic=is_pydantic)
+            if isinstance(a, FeatureWrapper):
+                a = unwrap_feature(a, raise_error=False)
             if isinstance(a, Filter):
                 new_filters.append(a)
             elif isinstance(a, type) and issubclass(a, Features):
@@ -122,8 +124,6 @@ class DataFrameMeta(type):
                 new_references_feature_set = a
             elif isinstance(a, Feature):
                 new_columns.append(a)
-            elif isinstance(a, FeatureWrapper):
-                new_columns.append(unwrap_feature(a))
             elif isinstance(a, bool):
                 # If we encounter a bool, that means we are evaluating the type annotation before
                 # the ResolverAstParser had a chance to extract the source and rewrite the and/or/in operations
@@ -194,11 +194,12 @@ class DataFrameMeta(type):
         from chalk.features.feature_field import Feature
 
         cls = cast(Type[DataFrame], cls)
-        columns: List[Feature] = []
+        # Using a dict to preserve ordering
+        columns: Dict[Feature, None] = {}
         for x in cls.__columns__:
             assert isinstance(x, Feature)
             if x not in columns:
-                columns.append(x)
+                columns[x] = None
         if cls.__references_feature_set__ is not None:
             # Only include the first-level feature types
             # Do not recurse has-ones and has-many as that could create an infinite loop
@@ -206,8 +207,8 @@ class DataFrameMeta(type):
             # then they definitely did not mean to include a feature they didn't know about
             for x in cls.__references_feature_set__.features:
                 assert isinstance(x, Feature)
-                if not x.is_has_many and not x.is_has_one and x not in columns and not x.is_autogenerated:
-                    columns.append(x)
+                if not x.is_autogenerated and not x.is_has_many and not x.is_has_one and x not in columns:
+                    columns[x] = None
         return tuple(columns)
 
     @property
@@ -658,6 +659,8 @@ class DataFrame(metaclass=DataFrameMeta):
         cols = []
         operation: Aggregation
         for alias, operation in agg.items():
+            alias = unwrap_feature(alias, raise_error=False)
+            operation = unwrap_feature(operation, raise_error=False)
             c = operation.col
             if len(operation.filters) > 0:
                 f = convert_filters_to_pl_expr(
@@ -1141,7 +1144,8 @@ class DataFrame(metaclass=DataFrameMeta):
             # if isinstance(operation, int):
             #     operation = op.count(FeatureWrapper.len_registry[operation])
             # c = pl.col(str(operation.col))
-
+            if isinstance(operation, FeatureWrapper):
+                operation = unwrap_feature(operation, raise_error=False)
             if isinstance(operation, Aggregation):
                 col = operation.col
                 if len(operation.filters) > 0:
@@ -1155,7 +1159,7 @@ class DataFrame(metaclass=DataFrameMeta):
                         col = col.filter(f)
 
                 cols.append(operation.fn(col).alias(str(alias)))
-            elif isinstance(operation, FeatureWrapper):
+            elif isinstance(operation, Feature):
                 cols.append(pl.col(str(operation)).alias(str(alias)))
             elif isinstance(operation, pl.Expr):
                 cols.append(operation.alias(str(alias)))
