@@ -3,7 +3,6 @@ from __future__ import annotations
 import copy
 import dataclasses
 import functools
-import itertools
 import re
 from datetime import timedelta
 from typing import (
@@ -17,36 +16,34 @@ from typing import (
     Optional,
     Sequence,
     Set,
-    Tuple,
     Type,
     TypeVar,
     Union,
     cast,
 )
 
+import itertools
 import numpy as np
 import pyarrow as pa
 
 from chalk._lsp.error_builder import FeatureClassErrorBuilder
-from chalk.utils.pydanticutil.pydantic_compat import is_pydantic_basemodel, is_pydantic_basemodel_instance
 from chalk._validation.feature_validation import FeatureValidation
 from chalk._validation.validation import Validation
 from chalk.features._encoding.converter import FeatureConverter, TDecoder, TEncoder
 from chalk.features._encoding.primitive import TPrimitive
-from chalk.features._vector import Vector
-from chalk.utils.collections import get_unique_item
-from chalk.features.filter import Filter, TimeDelta, disable_filter_bool
+from chalk.features.feature_set import FeatureSetBase
+from chalk.features.feature_wrapper import FeatureWrapper
+from chalk.features.filter import Filter, TimeDelta
 from chalk.features.tag import Tags
 from chalk.features.underscore import Underscore
 from chalk.serialization.parsed_annotation import ParsedAnnotation
 from chalk.utils.collections import ensure_tuple
+from chalk.utils.collections import get_unique_item
 from chalk.utils.duration import Duration, parse_chalk_duration
 from chalk.utils.log_with_context import get_logger
+from chalk.utils.pydanticutil.pydantic_compat import is_pydantic_basemodel, is_pydantic_basemodel_instance
 from chalk.utils.string import to_snake_case
 
-from chalk.features.feature_wrapper import FeatureWrapper
-from chalk.features.feature_set import FeatureSetBase
-
 if TYPE_CHECKING:
     from chalk.features.feature_set import Features
     from google.protobuf.message import Message as ProtobufMessage
@@ -115,6 +112,51 @@ class Feature(Generic[_TPrim, _TRich]):
         assert self.features_cls is not None
         return self.features_cls.__chalk_error_builder__
 
+    __slots__ = (
+        "_all_validations",
+        "_converter",
+        "_decoder",
+        "_default",
+        "_encoder",
+        "_fqn",
+        "_hash",
+        "_is_feature_time",
+        "_is_has_many_subfeature",
+        "_join",
+        "_path",
+        "_primary",
+        "_pyarrow_dtype",
+        "_raw_max_staleness",
+        "_root_fqn",
+        "_root_namespace",
+        "_typ",
+        "_validations",
+        "attribute_name",
+        "description",
+        "etl_offline_to_online",
+        "features_cls",
+        "hook",
+        "is_autogenerated",
+        "is_distance_pseudofeature",
+        "is_pseudofeature",
+        "is_singleton",
+        "last_for",
+        "last",
+        "max_staleness",
+        "name",
+        "namespace",
+        "no_display",
+        "offline_ttl",
+        "owner",
+        "raw_etl_offline_to_online",
+        "tags",
+        "underlying",
+        "underscore_expression",
+        "version",
+        "window_duration",
+        "window_durations",
+    )
+
     def __init__(
         self,
         name: str | None = None,
@@ -136,7 +178,6 @@ class Feature(Generic[_TPrim, _TRich]):
         decoder: TDecoder[_TPrim, _TRich] | None = None,
         pyarrow_dtype: pa.DataType | None = None,
         join: Callable[[], Filter] | Filter | None = None,
-        path: Tuple[HasOnePathObj, ...] = (),
         is_feature_time: bool | None = None,
         is_autogenerated: bool = False,
         validations: FeatureValidation | None = None,
@@ -159,12 +200,19 @@ class Feature(Generic[_TPrim, _TRich]):
         self.last = None
         self.last_for = last_for
         self._typ = typ if typ is None or isinstance(typ, ParsedAnnotation) else ParsedAnnotation(underlying=typ)
+        self._converter = None
         self.features_cls = features_cls
-        self._name = name
+        if name is None:
+            name = attribute_name
+        if name is not None:
+            self.name = name
         # the attribute name for the feature in the @features class (in case if the name is specified differently)
-        self._attribute_name = attribute_name
-        self._namespace = namespace
-        self.path = path
+        if attribute_name is not None:
+            self.attribute_name = attribute_name
+        if namespace is not None:
+            self.namespace = namespace
+        self._path: tuple[HasOnePathObj, ...] = ()
+        self._is_has_many_subfeature: bool | None = None
 
         # if primary is True and version is not None:
         #     self.lsp_error_builder.add_diagnostic(
@@ -213,7 +261,8 @@ class Feature(Generic[_TPrim, _TRich]):
             max_staleness = timedelta(0)
         elif isinstance(max_staleness, str):
             max_staleness = parse_chalk_duration(max_staleness)
-        self._max_staleness = max_staleness
+        if max_staleness is not ...:
+            self.max_staleness: timedelta = max_staleness
 
         if offline_ttl is None:
             offline_ttl = timedelta(0)
@@ -222,10 +271,11 @@ class Feature(Generic[_TPrim, _TRich]):
             offline_ttl = timedelta.max
         elif isinstance(offline_ttl, str):
             offline_ttl = parse_chalk_duration(offline_ttl)
-        self._offline_ttl = offline_ttl
+        self.offline_ttl = offline_ttl
 
-        self._etl_offline_to_online = etl_offline_to_online
-        self._raw_etl_offline_to_online = etl_offline_to_online
+        if etl_offline_to_online is not None:
+            self.etl_offline_to_online = etl_offline_to_online
+        self.raw_etl_offline_to_online = etl_offline_to_online
         self._is_feature_time = is_feature_time
         self.is_autogenerated = is_autogenerated
         self.no_display = no_display
@@ -235,7 +285,7 @@ class Feature(Generic[_TPrim, _TRich]):
         if self._validations is not None:
             self._all_validations.append(self._validations)
         self.window_durations = window_durations
-        self._window_duration = window_duration
+        self.window_duration = window_duration
         if last_for is not None:
             assert last_for.features_cls is not None
             for f in last_for.features_cls.features:
@@ -243,28 +293,21 @@ class Feature(Generic[_TPrim, _TRich]):
                     f.last = last_for
                     break
         self.hook = hook
+        self.underlying: Feature | None = None
+        self._root_fqn: str | None = None
+        self._root_namespace: str | None = None
+        self._fqn: str | None = None
+        self._hash: int | None = None
+
+    @property
+    def fqn(self):
+        if self._fqn is None:
+            self._fqn = f"{self.namespace}.{self.name}"
+        return self._fqn
 
     def __str__(self):
         return self.root_fqn
 
-    @property
-    def offline_ttl(self) -> timedelta:
-        if self._offline_ttl is ...:
-            raise RuntimeError("The offline TTL was never finalized. Is this feature in a features class?")
-        return self._offline_ttl
-
-    @property
-    def max_staleness(self) -> timedelta:
-        if self._max_staleness is ...:
-            raise RuntimeError("The max staleness was never finalized. Is this feature in a features class?")
-        return self._max_staleness
-
-    @property
-    def etl_offline_to_online(self) -> bool:
-        if self._etl_offline_to_online is None:
-            raise RuntimeError("ETL offline_to_online was never finalized. Is this feature in a features class?")
-        return self._etl_offline_to_online
-
     @property
     def typ(self) -> ParsedAnnotation:
         if self._typ is None:
@@ -282,28 +325,38 @@ class Feature(Generic[_TPrim, _TRich]):
     def primary(self) -> bool:
         if self._primary is not None:
             return self._primary
-
-        return (
-            self.features_cls is not None
-            and self.features_cls.__chalk_primary__ is not None
-            and self.features_cls.__chalk_primary__.name == self.name
-        ) or self.typ.is_primary()
+        if self.underlying is None:
+            return (
+                self.features_cls is not None
+                and self.features_cls.__chalk_primary__ is not None
+                and self.features_cls.__chalk_primary__.name == self.name
+            ) or self.typ.is_primary()
+        else:
+            return self.underlying.primary
 
     @property
     def is_feature_time(self) -> bool:
         if self._is_feature_time is not None:
             return self._is_feature_time
 
-        return (
-            self.features_cls is not None
-            and self.features_cls.__chalk_ts__ is not None
-            and self.features_cls.__chalk_ts__.name == self.name
-        ) or self.typ.is_feature_time()
+        if self.underlying is None:
+            return (
+                self.features_cls is not None
+                and self.features_cls.__chalk_ts__ is not None
+                and self.features_cls.__chalk_ts__.name == self.name
+            ) or self.typ.is_feature_time()
+        else:
+            return self.underlying.is_feature_time
 
-    @functools.cached_property
+    @property
     def converter(self) -> FeatureConverter:
         from chalk.features import DataFrame, Vector
 
+        if self.underlying is not None:
+            return self.underlying.converter
+        if self._converter is not None:
+            return self._converter
+
         encoder = self._encoder
         decoder = self._decoder
         pyarrow_dtype = self._pyarrow_dtype
@@ -335,14 +388,12 @@ class Feature(Generic[_TPrim, _TRich]):
         proto_typ = self.typ.as_proto()
         if proto_typ is not None:
             if encoder is None:
-
                 def proto_encoder(x: ProtobufMessage):
                     return x.SerializeToString()
 
                 encoder = proto_encoder
 
             if decoder is None:
-
                 def proto_decoder(x: bytes):
                     message = proto_typ()
                     message.ParseFromString(x)
@@ -355,7 +406,6 @@ class Feature(Generic[_TPrim, _TRich]):
 
         dataframe_typ = self.typ.as_dataframe()
         if dataframe_typ is not None:
-            # Don't explicitly get col.converter here to avoid infinite recursion
             # the has_many signature doesn't have fields for the encoder/decoder/pyarrow_dtype, so these fields should always be null
             assert decoder is None
             assert encoder is None
@@ -403,7 +453,6 @@ class Feature(Generic[_TPrim, _TRich]):
                 return DataFrame(results)
 
             decoder = frame_decoder
-            pyarrow_dtype = pa.large_list(pa.struct({k: v.converter.pyarrow_dtype for k, v in column_features.items()}))
 
         feature_set_typ = self.typ.as_features_cls()
         if feature_set_typ is not None:
@@ -500,68 +549,32 @@ class Feature(Generic[_TPrim, _TRich]):
             non_null_non_scalar_type = next((typ for typ in non_scalar_types if typ is not None), None)
             if non_null_non_scalar_type is not None:
                 raise TypeError(
-                    f"When annotating a feature's type with another feature to specify a join, the feature being "
-                    + f"used as a type must be a scalar type itself. Here, feature '{self.fqn}' is annotated with "
-                    + f"type '{feature_typ.fqn}', but '{feature_typ.fqn}' is annotated with complex type "
-                    + f"{non_null_non_scalar_type}. Feature '{feature_typ.fqn}' should be annotated with a type 'str'"
-                    + f"or 'int', for example."
+                    (
+                        "When annotating a feature's type with another feature to specify a join, the feature being "
+                        f"used as a type must be a scalar type itself. Here, feature '{self.fqn}' is annotated with "
+                        f"type '{feature_typ.fqn}', but '{feature_typ.fqn}' is annotated with complex type "
+                        f"{non_null_non_scalar_type}. Feature '{feature_typ.fqn}' should be annotated with "
+                        "a type 'str' or 'int', for example."
+                    )
                 )
             rich_type = feature_typ.converter.rich_type
             encoder = feature_typ.converter.encoder
             decoder = feature_typ.converter.decoder
             pyarrow_dtype = feature_typ.converter.pyarrow_dtype
 
-        return FeatureConverter(
+        self._converter = FeatureConverter(
             name=self.fqn,
             rich_type=rich_type,
             is_nullable=self.typ.is_nullable,
             rich_default=self._default,
             pyarrow_dtype=pyarrow_dtype,
             encoder=encoder,
-            decoder=decoder,
+            decoder=cast(Any, decoder),
         )
-
-    @property
-    def attribute_name(self):
-        if self._attribute_name is None:
-            raise RuntimeError(
-                "Feature.attribute_name is not yet defined. Is the feature being constructed outside of a Features class?"
-            )
-        return self._attribute_name
-
-    @attribute_name.setter
-    def attribute_name(self, attribute_name: str):
-        self._attribute_name = attribute_name
-        if self._name is None:
-            # If there is no name, also set the name to the attribute name
-            self._name = attribute_name
-
-    @property
-    def name(self):
-        if self._name is None:
-            raise RuntimeError(
-                "Feature.name is not yet defined. Is the feature being constructed outside of a Features class?"
-            )
-        return self._name
-
-    @name.setter
-    def name(self, name: str):
-        self._name = name
+        return self._converter
 
     def is_name_set(self):
-        return self._name is not None
-
-    @property
-    def namespace(self):
-        if self._namespace is None:
-            raise RuntimeError(
-                "Feature.namespace is not yet defined. Is the feature being constructed outside of a Features class?"
-            )
-        return self._namespace
-
-    @namespace.setter
-    def namespace(self, namespace: str):
-        self._namespace = namespace
+        return hasattr(self, "name")
 
     @classmethod
     @functools.lru_cache(None)
@@ -608,7 +621,7 @@ class Feature(Generic[_TPrim, _TRich]):
 
             for x in features_cls.features:
                 assert isinstance(x, Feature)
-                if x.name == feature_name or feature_name.lower() in {x.name, x.window_alias_name}:
+                if x.name == feature_name or feature_name.lower() in {x.name, *x.window_alias_name}:
                     assert x.attribute_name is not None
                     found_feature = True
                     feat = x if feat is None else feat.copy_with_path(x)
@@ -627,28 +640,47 @@ class Feature(Generic[_TPrim, _TRich]):
         return feat
 
     @property
-    def root_namespace(self) -> str:
-        if len(self.path) > 0:
-            assert self.path[0].parent.namespace is not None, "parent namespace is None"
-            return self.path[0].parent.namespace
-        assert self.namespace is not None, "namespace is None"
-        return self.namespace
+    def root_namespace(self):
+        if self._root_namespace is None:
+            self.path = self._path
+        assert self._root_namespace is not None
+        return self._root_namespace
 
     @property
-    def root_fqn(self) -> str:
-        assert self.name is not None, "Missing name on feature"
-        if len(self.path) > 0:
-            return ".".join(
+    def root_fqn(self):
+        if self._root_fqn is None:
+            self.path = self._path
+        assert self._root_fqn is not None
+        return self._root_fqn
+
+    @property
+    def path(self):
+        return self._path
+
+    @path.setter
+    def path(self, path: tuple[HasOnePathObj, ...]):
+        self._path = path
+        if len(path) > 0:
+            assert path[0].parent.namespace is not None, "parent namespace is None"
+            self._root_namespace = path[0].parent.namespace
+            self._root_fqn = ".".join(
                 itertools.chain(
-                    (self.root_namespace,),
+                    (self._root_namespace,),
                     (x.parent.name for x in self.path),
                     (self.name,),
                 )
             )
-        return f"{self.namespace}.{self.name}"
+        else:
+            self._root_namespace = self.namespace
+            self._root_fqn = f"{self.namespace}.{self.name}"
+        self._hash = hash(self.root_fqn)
+        self._is_has_many_subfeature = any(p.parent.is_has_many for p in self.path)
 
     def __hash__(self) -> int:
-        return hash(self.root_fqn)
+        if self._hash is None:
+            self.path = self._path
+        assert self._hash is not None
+        return self._hash
 
     def __eq__(self, other: object) -> bool:
         if self.is_has_many:
@@ -675,29 +707,37 @@ class Feature(Generic[_TPrim, _TRich]):
     def __repr__(self):
         return f"Feature(fqn={self.root_fqn}, typ={self.typ})"
 
-    @property
-    def fqn(self) -> str:
-        return f"{self.namespace}.{self.name}"
-
     @property
     def is_has_one(self) -> bool:
         # A feature is a has-one relationship if the type is
         # another singleton features cls and there is a join condition
         # Need to short-circuit if it is a dataframe, as DataFrames
         # might not have an underlying
-        return self.typ.as_features_cls() is not None
+        if self.underlying is None:
+            return self.typ.as_features_cls() is not None
+        else:
+            return self.underlying.is_has_one
 
     @property
     def is_proto(self):
-        return self.typ.as_proto() is not None
+        if self.underlying is None:
+            return self.typ.as_proto() is not None
+        else:
+            return self.underlying.is_proto
 
     @property
     def is_has_many(self):
-        return self.typ.as_dataframe() is not None
+        if self.underlying is None:
+            return self.typ.as_dataframe() is not None
+        else:
+            return self.underlying.is_has_many
 
-    @functools.cached_property
-    def is_has_many_subfeature(self):
-        return any(p.parent.is_has_many for p in self.path)
+    @property
+    def is_has_many_subfeature(self) -> bool:
+        if self._is_has_many_subfeature is None:
+            self.path = self._path
+        assert self._is_has_many_subfeature is not None
+        return self._is_has_many_subfeature
 
     @property
     def is_scalar(self):
@@ -716,17 +756,23 @@ class Feature(Generic[_TPrim, _TRich]):
         representing a particular windowed bucket. This feature is like
         any other scalar feature, and has data associated in the offline
         and online stores."""
-        return self._window_duration is not None
+        return self.window_duration is not None
 
-    @functools.cached_property
-    def window_alias_name(self) -> Optional[str]:
+    @property
+    def window_alias_name(self) -> frozenset[str]:
         """Forgive us our trespasses, as we forgive those who trespass against us."""
         if not self.is_windowed_pseudofeature:
-            return None
+            return frozenset()
 
         suffix_len = len(f"__{self.window_duration}__")
+        window_alias_formats = [
+            "{stem}_{timerep}",  # w_1d_1h_1m_1s
+            "{stem}__{timerep}__"  # w__1d_1h_1m_1s__
+        ]
+        bucketed_time_groups = []
+
         stem = self.name[:-suffix_len]
-        b = self._window_duration
+        b = self.window_duration
         assert b is not None
         days = b // 86400
         days_as_seconds = days * 86400
@@ -736,25 +782,26 @@ class Feature(Generic[_TPrim, _TRich]):
         minutes_as_seconds = minutes * 60
         seconds = b - days_as_seconds - hours_as_seconds - minutes_as_seconds
 
-        return f"{stem}_" + "".join(
-            [
-                f"{count}{unit}"
-                for count, unit in (
-                    (days, "d"),
-                    (hours, "h"),
-                    (minutes, "m"),
-                    (seconds, "s"),
-                )
-                if count > 0
-            ]
-        )
+        if (b % 3600) == 0:  # if the window is expressible in hours
+            window_as_hours = b // 3600
+            bucketed_time_groups.append(f"{window_as_hours}h")
 
-    @property
-    def window_duration(self) -> int:
-        """The window duration, in seconds."""
-        if self._window_duration is None:
-            raise ValueError(f"Feature {self} is not a windowed pseudofeature")
-        return self._window_duration
+        bucketed_time_groups.append("".join([
+            f"{count}{unit}"
+            for count, unit in (
+                (days, "d"),
+                (hours, "h"),
+                (minutes, "m"),
+                (seconds, "s"),
+            )
+            if count > 0
+        ]))
+
+        return frozenset({
+            form.format(stem=stem, timerep=bucketed_time)
+            for form in window_alias_formats
+            for bucketed_time in bucketed_time_groups
+        })
 
     @property
     def window_buckets(self) -> Optional[Set[int]]:
@@ -774,6 +821,8 @@ class Feature(Generic[_TPrim, _TRich]):
     def join(self) -> Optional[Filter]:
         # Need to manually check the pseudofeatures because they
         # do not have a features class
+        if self.underlying is not None:
+            return self.underlying.join
         if self.is_pseudofeature:
             return None
 
@@ -781,8 +830,8 @@ class Feature(Generic[_TPrim, _TRich]):
             # Join was explicitly specified
             if not callable(self._join):
                 return self._join
-            join = self._validate_join()
-            return join
+            self._join = self._validate_join()
+            return self._join
 
         # The join was NOT specified on this feature.
         # It's possible it was specified with a type that points to another feature class.
@@ -851,7 +900,7 @@ class Feature(Generic[_TPrim, _TRich]):
                 ),
                 label="invalid join",
                 range=self.lsp_error_builder.property_value_range(self.attribute_name)
-                or self.lsp_error_builder.property_range(self.attribute_name),
+                      or self.lsp_error_builder.property_range(self.attribute_name),
                 code="38",
                 raise_error=ValueError,
             )
@@ -913,6 +962,7 @@ class Feature(Generic[_TPrim, _TRich]):
                 last_for=self,
             )
             self.features_cls.features.append(last_feature)
+            setattr(self.features_cls, last_name, FeatureWrapper(last_feature))
 
         if len(self.path) == 0:
             return last_feature
@@ -941,8 +991,12 @@ class Feature(Generic[_TPrim, _TRich]):
                 ),
             )
         )
+        child_copy.underlying = child if child.underlying is None else child.underlying
         return child_copy
 
+    def dummy_feature(self) -> Feature:
+        return DUMMY_FEATURE
+
     def for_version(self, version: int) -> Feature:
         if self.version is None:
             assert self.features_cls is not None
@@ -1001,27 +1055,26 @@ class Feature(Generic[_TPrim, _TRich]):
 
     def _validate_join(self) -> Filter:
         assert callable(self._join)
-        with disable_filter_bool(True):
-            try:
-                join = self._join()
-            except Exception:
-                assert self.features_cls is not None
-                self.lsp_error_builder.add_diagnostic(
-                    message=(
-                        f"The attribute '{self.attribute_name}' on the feature class '{self.features_cls.__name__}' "
-                        f"has a join function that is incorrectly configured. "
-                        f"Ensure that the join is a lambda function that sets features from two feature classes as "
-                        f"equal. For example, ' = has_one(lambda: "
-                        f"{self.features_cls.__name__}.id == "
-                        f"OtherClass.{to_snake_case(self.features_cls.__name__)}_id"
-                        f")'"
-                    ),
-                    label="invalid join",
-                    range=self.lsp_error_builder.property_value_range(self.attribute_name)
-                    or self.lsp_error_builder.property_range(self.attribute_name),
-                    code="32",
-                    raise_error=TypeError,
-                )
+        try:
+            join = self._join()
+        except Exception:
+            assert self.features_cls is not None
+            self.lsp_error_builder.add_diagnostic(
+                message=(
+                    f"The attribute '{self.attribute_name}' on the feature class '{self.features_cls.__name__}' "
+                    f"has a join function that is incorrectly configured. "
+                    f"Ensure that the join is a lambda function that sets features from two feature classes as "
+                    f"equal. For example, ' = has_one(lambda: "
+                    f"{self.features_cls.__name__}.id == "
+                    f"OtherClass.{to_snake_case(self.features_cls.__name__)}_id"
+                    f")'"
+                ),
+                label="invalid join",
+                range=self.lsp_error_builder.property_value_range(self.attribute_name)
+                      or self.lsp_error_builder.property_range(self.attribute_name),
+                code="32",
+                raise_error=TypeError,
+            )
 
         if not self.is_has_many and not self.is_has_one:
             assert self.features_cls is not None
@@ -1032,7 +1085,7 @@ class Feature(Generic[_TPrim, _TRich]):
                 ),
                 label="invalid join",
                 range=self.lsp_error_builder.property_value_range(self.attribute_name)
-                or self.lsp_error_builder.property_range(self.attribute_name),
+                      or self.lsp_error_builder.property_range(self.attribute_name),
                 code="37",
                 raise_error=TypeError,
             )
@@ -1055,7 +1108,7 @@ class Feature(Generic[_TPrim, _TRich]):
                 ),
                 label="invalid join",
                 range=self.lsp_error_builder.property_value_range(self.attribute_name)
-                or self.lsp_error_builder.property_range(self.attribute_name),
+                      or self.lsp_error_builder.property_range(self.attribute_name),
                 code="40",
                 raise_error=TypeError,
             )
@@ -1070,7 +1123,7 @@ class Feature(Generic[_TPrim, _TRich]):
                 ),
                 label="invalid join",
                 range=self.lsp_error_builder.property_value_range(self.attribute_name)
-                or self.lsp_error_builder.property_range(self.attribute_name),
+                      or self.lsp_error_builder.property_range(self.attribute_name),
                 code="42",
                 raise_error=TypeError,
             )
@@ -1089,7 +1142,7 @@ class Feature(Generic[_TPrim, _TRich]):
                 ),
                 label="invalid join",
                 range=self.lsp_error_builder.property_value_range(self.attribute_name)
-                or self.lsp_error_builder.property_range(self.attribute_name),
+                      or self.lsp_error_builder.property_range(self.attribute_name),
                 code="43",
                 raise_error=TypeError,
             )
@@ -1109,7 +1162,7 @@ class Feature(Generic[_TPrim, _TRich]):
                     ),
                     label="invalid join",
                     range=self.lsp_error_builder.property_value_range(self.attribute_name)
-                    or self.lsp_error_builder.property_range(self.attribute_name),
+                          or self.lsp_error_builder.property_range(self.attribute_name),
                     code="44",
                     raise_error=TypeError,
                 )
@@ -1149,12 +1202,15 @@ class Feature(Generic[_TPrim, _TRich]):
         """
         FeatureConverter is sometimes not serializable, so don't include it when (un)pickling
         """
-        state = self.__dict__.copy()
-        state.pop("converter", None)
-        return state
+        return {
+            slot: None if slot == "_converter" else getattr(self, slot)
+            for slot in self.__slots__
+            if hasattr(self, slot)
+        }
 
     def __setstate__(self, state: Dict[str, Any]):
-        self.__dict__.update(state)
+        for k, v in state.items():
+            setattr(self, k, v)
 
     def __sub__(self, other: Any):
         from chalk.features.pseudofeatures import Now
@@ -1172,6 +1228,18 @@ class Feature(Generic[_TPrim, _TRich]):
         raise TypeError(f"Cannot subtract {other} from {self}")
 
 
+DUMMY_FEATURE = Feature(
+    name="__undefined__",
+    namespace="__chalk__",
+    typ=str,
+    max_staleness=None,
+    etl_offline_to_online=False,
+    primary=False,
+    is_feature_time=False,
+    is_pseudofeature=True,
+)
+
+
 def feature(
     description: Optional[str] = None,
     owner: Optional[str] = None,
