@@ -16,13 +16,13 @@ from typing import Any, Dict, List, Mapping, Sequence, Tuple, cast, Type, TypeVa
 
 import pyarrow as pa
 
-from chalk._lsp._class_finder import get_class_ast
 from chalk._lsp.error_builder import FeatureClassErrorBuilder, LSPErrorBuilder
 from chalk.features._class_property import classproperty, classproperty_support
 from chalk.features.feature_field import Feature, VersionInfo
 from chalk.features.feature_set import Features, FeatureSetBase, ClassSource
 from chalk.features.feature_time import feature_time
 from chalk.features.feature_wrapper import FeatureWrapper, unwrap_feature
+from chalk.features.namespace_context import build_namespaced_name
 from chalk.features.underscore import Underscore
 from chalk.serialization.parsed_annotation import ParsedAnnotation
 from chalk.streams import Windowed
@@ -106,6 +106,12 @@ def features(
         The name for the feature set. By default, the name of a feature is
         taken from the name of the attribute on the class, prefixed with
         the camel-cased name of the class.
+    singleton
+        If `True`, the feature set is a singleton, and there will be only
+        one instance of the class. Because there is only one instance, a
+        singleton feature class does not have a primary key. Features
+        defined on a singleton feature class are available in all resolvers,
+        regardless of whatever other features are used as inputs to the resolver.
 
     Examples
     --------
@@ -126,26 +132,12 @@ def features(
 
     def wrap(c: Type[T]) -> Type[T]:
         namespace = name if name is not None else to_snake_case(c.__name__)
-        source_info: Optional[ClassSource] = None
-        try:
-            class_source = _get_object_source(c)
-            filename = inspect.getfile(c)
-            dedent_source = class_source and textwrap.dedent(class_source)
-            try:
-                tree = get_class_ast(c)
-            except:
-                tree = None
-            source_info = ClassSource(
-                filename=filename,
-                source=class_source,
-                dedent_source=dedent_source,
-                tree=tree,
-            )
-        except:
-            pass
+        namespace = build_namespaced_name(name=namespace)
+
+        source_info = ClassSource(c)
 
         error_builder = FeatureClassErrorBuilder(
-            uri=source_info.filename if source_info is not None else "__main__",
+            uri=source_info.filename if source_info.filename is not None else "__main__",
             namespace=namespace,
             node=source_info and source_info.tree,
         )
@@ -195,7 +187,7 @@ def features(
             and (
                 source_info.filename != previous_features_class.__chalk_source_info__.filename
                 or source_info.tree.lineno != previous_features_class.__chalk_source_info__.tree.lineno
-                or source_info.source != previous_features_class.__chalk_source_info__.source
+                or source_info.class_source != previous_features_class.__chalk_source_info__.class_source
             )
         ):
             error_builder.add_diagnostic(
@@ -507,6 +499,8 @@ def _get_field(
 
     f.features_cls = cls
     f.attribute_name = annotation_name
+    if not hasattr(f, "name"):
+        f.name = annotation_name
     f.is_singleton = is_singleton
     _process_field(
         f=f,
@@ -599,13 +593,13 @@ def _process_field(
     elif f.owner is None:
         f.owner = comment_based_owner or class_owner
 
-    # Using the private variable because the max_staleness is a read-only property
-    if f._max_staleness is ...:
-        f._max_staleness = class_max_staleness
+    # The attribute is not defined if the feature intends to use the class default
+    if not hasattr(f, "max_staleness"):
+        f.max_staleness = class_max_staleness
 
     # Using the private variable because the etl_offline_to_online is a read-only property
-    if f._etl_offline_to_online is None:
-        f._etl_offline_to_online = class_etl_offline_to_online
+    if not hasattr(f, "etl_offline_to_online"):
+        f.etl_offline_to_online = class_etl_offline_to_online
     return f
 
 
@@ -704,13 +698,6 @@ def _iter_fn(_globals: Mapping[str, Any]):
     )
 
 
-def _get_object_source(obj: Union[Type, Callable]) -> Optional[str]:
-    if notebook.is_defined_in_module(obj):
-        return inspect.getsource(obj)
-    # TODO (rkargon) Try to get source from notebook cells if possible
-    return None
-
-
 def _parse_annotation_comments(source_info: ClassSource, cls_annotations: Dict[str, Any]) -> Mapping[str, str]:
     if source_info.tree is None or source_info.dedent_source is None:
         return {}
@@ -751,7 +738,7 @@ CHALK_SINGLETON_VALUE = 111
 
 def _process_class(
     cls: Type[T],
-    source_info: Optional[ClassSource],
+    source_info: ClassSource,
     error_builder: FeatureClassErrorBuilder,
     owner: Optional[str],
     tags: Tuple[str, ...],
@@ -886,7 +873,6 @@ def _process_class(
     cls.__annotations__ = cls_annotations
     del cls_annotations  # unused; set cls.__annotations__ directly
 
-    set_new_attribute(cls=cls, name="__chalk_class_definition__", value=source_info and source_info.dedent_source)
     set_new_attribute(cls=cls, name="__chalk_is_singleton__", value=singleton)
     set_new_attribute(cls=cls, name="__chalk_error_builder__", value=error_builder)
     set_new_attribute(cls=cls, name="__chalk_source_info__", value=source_info)
@@ -1077,7 +1063,7 @@ def _process_class(
                 maximum=f.version.maximum,
                 default=f.version.default,
                 reference=f.version.reference,
-                base_name=f.version.base_name
+                base_name=f.version.base_name,
             )
             f.version.reference[i] = f_i
             cls_fields.append(f_i)
