@@ -1,7 +1,6 @@
 from __future__ import annotations
 
 import collections.abc
-import contextlib
 import contextvars
 import dataclasses
 from datetime import datetime, timedelta, timezone
@@ -11,21 +10,6 @@ from typing import TYPE_CHECKING, Any, Hashable, Set, Type
 if TYPE_CHECKING:
     from chalk.features.feature_field import Feature
 
-_DISABLE_FILTER_BOOL: contextvars.ContextVar[bool] = contextvars.ContextVar("DISABLE_FILTER_BOOL", default=False)
-
-
-@contextlib.contextmanager
-def disable_filter_bool(value: bool = True):
-    token = _DISABLE_FILTER_BOOL.set(value)
-    try:
-        yield
-    finally:
-        _DISABLE_FILTER_BOOL.reset(token)
-
-
-class FilterBoolNotAllowedError(ValueError):
-    """Exception that is raised if Filter.__bool__ is called when DISABLE_FILTER_BOOL is True"""
-
 
 class Filter:
     def __init__(self, lhs: Any, operation: str, rhs: Any):
@@ -37,19 +21,19 @@ class Filter:
         # Other could also be a sequence (in the case of operation = "in")
         self.operation = operation
         if isinstance(lhs, FeatureWrapper):
-            lhs = unwrap_feature(lhs)
+            lhs = unwrap_feature(lhs, raise_error=False)
         self.lhs = lhs
         if self.operation == "in":
             if not isinstance(rhs, collections.abc.Iterable):
                 raise ValueError("The RHS must be an iterable for operation='in'")
-            rhs_unwrapped = [unwrap_feature(x) if isinstance(x, FeatureWrapper) else x for x in rhs]
+            rhs_unwrapped = [unwrap_feature(x, raise_error=False) if isinstance(x, FeatureWrapper) else x for x in rhs]
             for x in rhs_unwrapped:
                 if not isinstance(x, Hashable):
                     raise ValueError(f"RHS for operation='in' must be hashable; Found un-hashable type: {type(x)}")
-            rhs = frozenset(unwrap_feature(x) if isinstance(x, FeatureWrapper) else x for x in rhs)
+            rhs = frozenset(unwrap_feature(x, raise_error=False) if isinstance(x, FeatureWrapper) else x for x in rhs)
         else:
             if isinstance(rhs, FeatureWrapper):
-                rhs = unwrap_feature(rhs)
+                rhs = unwrap_feature(rhs, raise_error=False)
         self.rhs = rhs
 
     def __hash__(self) -> int:
@@ -78,13 +62,7 @@ class Filter:
         return f"Filter({self.lhs} {self.operation} {self.rhs})"
 
     def __bool__(self):
-        if _DISABLE_FILTER_BOOL.get():
-            raise FilterBoolNotAllowedError(
-                "Joining filters together with `and` or `or` is not allowed in the current context"
-            )
-        else:
-            # By default custom class instances are always "True"
-            return True
+        return bool(self.rhs == self.lhs)
 
 
 def _get_feature_time_index(explicit_index: Any):
