@@ -195,3 +195,19 @@ class UnderscoreUnaryOp(Underscore):
 
 
 _ = underscore = UnderscoreRoot()
+
+
+# NEED `__all__` because `_` is private and can't be auto-imported by i.e. IntelliJ.
+__all__ = [
+    "SUPPORTED_UNDERSCORE_OPS_BINARY",
+    "SUPPORTED_UNDERSCORE_OPS_UNARY",
+    "Underscore",
+    "UnderscoreAttr",
+    "UnderscoreCall",
+    "UnderscoreItem",
+    "UnderscoreBinaryOp",
+    "UnderscoreUnaryOp",
+    "UnderscoreRoot",
+    "underscore",
+    "_",
+]
