@@ -1,5 +1,6 @@
 import importlib
 import importlib.util
+import linecache
 import os
 import sys
 import traceback
@@ -32,19 +33,28 @@ def _py_path_to_module(path: Path, repo_root: Path) -> str:
         p = path.relative_to(repo_root)
     except ValueError:
         p = path
-    return str(p)[: -len(".py")].replace(os.path.join(".", ""), "").replace(os.path.sep, ".")
+    ans = str(p)[: -len(".py")].replace(os.path.join(".", ""), "").replace(os.path.sep, ".")
+    if ans.endswith(".__init__"):
+        # Do not import __init__.py directly. Instead, import the module
+        ans = ans[: -len(".__init__")]
+    return ans
 
 
 has_imported_all_files = False
 
 
 def import_all_files_once(
-    file_allowlist: Optional[List[str]] = None, project_root: Optional[Path] = None, only_sql_files: bool = False
+    file_allowlist: Optional[List[str]] = None,
+    project_root: Optional[Path] = None,
+    only_sql_files: bool = False,
+    override: bool = False,
 ) -> List[FailedImport]:
     global has_imported_all_files
     if has_imported_all_files:
         return []
-    failed = import_all_files(file_allowlist=file_allowlist, project_root=project_root, only_sql_files=only_sql_files)
+    failed = import_all_files(
+        file_allowlist=file_allowlist, project_root=project_root, only_sql_files=only_sql_files, override=override
+    )
     has_imported_all_files = True
     return failed
 
@@ -53,6 +63,8 @@ def import_all_files(
     file_allowlist: Optional[List[str]] = None,
     project_root: Optional[Path] = None,
     only_sql_files: bool = False,
+    check_ignores: bool = True,
+    override: bool = False,
 ) -> List[FailedImport]:
     if project_root is None:
         project_root = get_directory_root()
@@ -78,18 +90,23 @@ def import_all_files(
                 chalk_sql_files.append(f)
 
     if only_sql_files:
-        return import_sql_file_resolvers(project_root, chalk_sql_files)
+        return import_sql_file_resolvers(project_root, chalk_sql_files, override=override)
 
     failed_imports: List[FailedImport] = import_all_python_files_from_dir(
         project_root=project_root,
         file_allowlist=python_files,
+        check_ignores=check_ignores,
     )
     has_import_errors = len(failed_imports) > 0
-    failed_imports.extend(import_sql_file_resolvers(project_root, chalk_sql_files, has_import_errors))
+    failed_imports.extend(
+        import_sql_file_resolvers(project_root, chalk_sql_files, has_import_errors=has_import_errors, override=override)
+    )
     return failed_imports
 
 
-def import_sql_file_resolvers(path: Path, file_allowlist: Optional[List[str]] = None, has_import_errors: bool = False):
+def import_sql_file_resolvers(
+    path: Path, file_allowlist: Optional[List[str]] = None, has_import_errors: bool = False, override: bool = False
+):
     if file_allowlist is not None:
         sql_resolver_results = get_sql_file_resolvers_from_paths(
             sources=BaseSQLSource.registry, paths=file_allowlist, has_import_errors=has_import_errors
@@ -101,7 +118,7 @@ def import_sql_file_resolvers(path: Path, file_allowlist: Optional[List[str]] =
     failed_imports: List[FailedImport] = []
     for result in sql_resolver_results:
         if result.resolver:
-            result.resolver.add_to_registry(override=False)
+            result.resolver.add_to_registry(override=override)
         if result.errors:
             for error in result.errors:
                 failed_imports.append(
@@ -168,7 +185,8 @@ def _get_py_files_fast(
         if (venv_path is not None and venv_path.samefile(dirpath)) or (
             ignore_config
             and not ignore_config.has_negation
-            and ignore_config.ignored(os.path.join(str(dirpath), "#"))  # Hack to make "dir/**" match "/Users/home/dir"
+            and ignore_config.ignored(os.path.join(str(dirpath), "#"))
+            # Hack to make "dir/**" match "/Users/home/dir"
         ):
             dirnames.clear()  # Skip subdirectories
             continue  # Skip files
@@ -225,17 +243,19 @@ class ChalkImporter:
             module=module_path,
         )
         if line_number is not None:
-            self.ranges[error_file] = RangeGQL(
-                start=PositionGQL(
-                    line=line_number,
-                    character=0,
-                ),
-                end=PositionGQL(
-                    line=line_number,
-                    character=len(line) if line is not None else 0,
-                ),
-            )
-            self.short_tracebacks[error_file] = error_message
+            line = linecache.getline(str(filename), line_number)
+            if line != "":
+                self.ranges[error_file] = RangeGQL(
+                    start=PositionGQL(
+                        line=line_number,
+                        character=len(line) - len(line.lstrip()),
+                    ),
+                    end=PositionGQL(
+                        line=line_number,
+                        character=len(line) - 1 if line is not None else 0,
+                    ),
+                )
+                self.short_tracebacks[error_file] = error_message
 
     def get_failed_imports(self) -> List[FailedImport]:
         return list(self.errors.values())
@@ -296,9 +316,9 @@ def import_all_python_files_from_dir(
     # We can manually detect this by building a set of all absolute filepaths we imported, and then comparing filepaths against
     # this set before attempting to import the module again
     already_imported_files = {
-        Path(x.__file__).resolve()
-        for x in sys.modules.copy().values()
-        if hasattr(x, "__file__") and isinstance(x.__file__, str)
+        Path(v.__file__).resolve(): k
+        for (k, v) in sys.modules.copy().items()
+        if hasattr(v, "__file__") and isinstance(v.__file__, str)
     }
     try:
         venv = os.environ.get("VIRTUAL_ENV")
@@ -331,6 +351,9 @@ def import_all_python_files_from_dir(
         for filename in repo_files:
             # we want resolved_root in case repo_root contains a symlink
             if filename in already_imported_files:
+                _logger.debug(
+                    f"Skipping import of '{filename}' since it is already imported as module {already_imported_files[filename]}"
+                )
                 continue
             module_path = _py_path_to_module(filename, resolved_root)
             if module_path.startswith(".eggs") or module_path.startswith("venv") or filename.name == "setup.py":
@@ -346,7 +369,8 @@ def import_all_python_files_from_dir(
                     CHALK_IMPORTER.add_error(ex_type, ex_value, ex_traceback, filename, module_path)
                     _logger.debug(f"Failed while importing {module_path}", exc_info=True)
             else:
-                already_imported_files.add(filename)
+                _logger.debug(f"Imported '{filename}' as module {module_path}")
+                already_imported_files[filename] = module_path
     finally:
         # Let's remove our added entries in sys.path so we don't pollute it
         sys.path.pop(0)
