@@ -34,7 +34,7 @@ def load_integration_variable(
     value = None
     if integration_name is None:
         value = os.getenv(name)
-    elif has_integration(integration_name):
+    else:
         value = os.getenv(f"{integration_name}_{name}")
 
     return parser(value) if value is not None else None
