@@ -36,7 +36,7 @@ from mypy.plugins.common import add_method, deserialize_and_fixup_type
 from mypy.types import Instance, NoneType, Type, UnboundType
 
 from chalk.features import DataFrame, Features, feature, features
-from chalk.features.dataframe import DataFrameMeta
+from chalk.features.dataframe._impl import DataFrameMeta
 
 
 def _get_full_name(cls: Union[type, Callable]):
