@@ -3,7 +3,7 @@ from __future__ import annotations
 import collections
 import traceback
 from collections import defaultdict
-from typing import DefaultDict, Dict, List, Literal, Mapping, Optional, Tuple, Union
+from typing import DefaultDict, Dict, List, Mapping
 
 import pyarrow as pa
 
@@ -11,6 +11,15 @@ from chalk._lsp.error_builder import FeatureClassErrorBuilder, LSPErrorBuilder,
 from chalk.features import FeatureSetBase
 from chalk.features._encoding.serialized_dtype import deserialize_dtype
 from chalk.features.resolver import RESOLVER_REGISTRY
+from chalk.parsed._metadata import (
+    METADATA_STRING_TO_FEATURE_ATTRIBUTE_MAP,
+    METADATA_STRING_TO_RESOLVER_ATTRIBUTE_MAP,
+    MissingMetadataResolver,
+    build_feature_log_for_each_severity,
+    build_resolver_log_for_each_severity,
+    get_missing_metadata,
+    validate_feature_metadata,
+)
 from chalk.parsed.duplicate_input_gql import (
     GraphLogSeverity,
     ProjectSettingsGQL,
@@ -20,24 +29,10 @@ from chalk.parsed.duplicate_input_gql import (
     UpsertResolverGQL,
 )
 from chalk.utils.duration import parse_chalk_duration
-from chalk.utils.string import add_quotes, oxford_comma_list, to_camel_case
-
-_METADATA_STRING_TO_FEATURE_ATTRIBUTE_MAP: Dict[str, str] = {
-    "owner": "owner",
-    "tags": "tags",
-    "description": "description",
-}
-
-_METADATA_STRING_TO_RESOLVER_ATTRIBUTE_MAP: Dict[str, str] = {
-    "owner": "owner",
-    "tags": "tags",
-    "description": "doc",
-    "doc": "doc",
-    "docstring": "doc",
-}
+from chalk.utils.string import add_quotes, oxford_comma_list
 
 
-class _LogBuilder:
+class ClientLogBuilder:
     def __init__(self):
         super().__init__()
         self._logs: List[UpdateGraphError] = []
@@ -60,7 +55,7 @@ class _LogBuilder:
 
 def _validate_primary_key(
     feature: UpsertFeatureGQL,
-    builder: _LogBuilder,
+    builder: ClientLogBuilder,
     lsp_builder: FeatureClassErrorBuilder,
 ):
     if feature.scalarKind is not None and feature.scalarKind.primary:
@@ -73,8 +68,8 @@ def _validate_primary_key(
         ):
             lsp_builder.add_diagnostic(
                 label="invalid type",
-                message=f"Primary keys must be integers or strings. Feature '{feature.id.attributeName}' has a type of '{pa_dtype}'",
-                range=lsp_builder.annotation_range(feature.id.attributeName),
+                message=f"Primary keys must be integers or strings. Feature '{feature.id.fqn}' has a type of '{pa_dtype}'",
+                range=lsp_builder.annotation_range(feature.id.attributeName or feature.id.name),
                 code="23",
             )
             builder.add_error(
@@ -87,7 +82,7 @@ def _validate_primary_key(
 
 def _validate_max_staleness(
     feature: UpsertFeatureGQL,
-    builder: _LogBuilder,
+    builder: ClientLogBuilder,
     lsp_builder: FeatureClassErrorBuilder,
 ):
     if feature.maxStaleness is not None and feature.maxStaleness != "infinity":
@@ -102,7 +97,7 @@ def _validate_max_staleness(
 
 def _validate_etl_to_online(
     feature: UpsertFeatureGQL,
-    builder: _LogBuilder,
+    builder: ClientLogBuilder,
     lsp_builder: FeatureClassErrorBuilder,
 ):
     if feature.etlOfflineToOnline and feature.maxStaleness is None:
@@ -115,7 +110,7 @@ def _validate_etl_to_online(
 def _validate_no_feature_times_as_input(
     fqn_to_feature: Mapping[str, UpsertFeatureGQL],
     resolver: UpsertResolverGQL,
-    builder: _LogBuilder,
+    builder: ClientLogBuilder,
     lsp_builder: ResolverErrorBuilder,
 ):
     for i, inp in enumerate(resolver.inputs or []):
@@ -155,7 +150,7 @@ def _validate_no_feature_times_as_input(
             )
 
 
-def _validate_feature_names_unique(features: List[UpsertFeatureGQL], builder: _LogBuilder):
+def _validate_feature_names_unique(features: List[UpsertFeatureGQL], builder: ClientLogBuilder):
     counter: DefaultDict[str, int] = defaultdict(lambda: 0)
     for r in features:
         counter[r.id.fqn] += 1
@@ -171,7 +166,7 @@ def _validate_feature_names_unique(features: List[UpsertFeatureGQL], builder: _L
             )
 
 
-def _validate_resolver_names_unique(resolvers: List[UpsertResolverGQL], builder: _LogBuilder):
+def _validate_resolver_names_unique(resolvers: List[UpsertResolverGQL], builder: ClientLogBuilder):
     counter: DefaultDict[str, int] = defaultdict(lambda: 0)
     for r in resolvers:
         counter[r.fqn.split(".")[-1]] += 1
@@ -190,7 +185,7 @@ def _validate_resolver_names_unique(resolvers: List[UpsertResolverGQL], builder:
 def _validate_resolver_input(
     singleton_namespaces: set[str],
     resolver: UpsertResolverGQL,
-    builder: _LogBuilder,
+    builder: ClientLogBuilder,
     lsp_builder: ResolverErrorBuilder,
 ):
     namespaces = []
@@ -237,7 +232,9 @@ If you require features from many feature classes, reference them via their rela
         )
 
 
-def _validate_resolver_output(resolver: UpsertResolverGQL, builder: _LogBuilder, lsp_builder: ResolverErrorBuilder):
+def _validate_resolver_output(
+    resolver: UpsertResolverGQL, builder: ClientLogBuilder, lsp_builder: ResolverErrorBuilder
+):
     output = resolver.output
     features = output.features or []
     dataframes = output.dataframes or []
@@ -308,15 +305,17 @@ def _validate_resolver_output(resolver: UpsertResolverGQL, builder: _LogBuilder,
 
 
 # FIXME CHA-66 we should validate that joins are all pkey on pkey joins. no non-primary keys, no self-joins, no constants
-def _validate_joins(feature: UpsertFeatureGQL, builder: _LogBuilder, lsp_builder: FeatureClassErrorBuilder):
+def _validate_joins(feature: UpsertFeatureGQL, builder: ClientLogBuilder, lsp_builder: FeatureClassErrorBuilder):
     pass
 
 
-def _validate_feature_names(feature: UpsertFeatureGQL, builder: _LogBuilder, lsp_builder: FeatureClassErrorBuilder):
+def _validate_feature_names(
+    feature: UpsertFeatureGQL, builder: ClientLogBuilder, lsp_builder: FeatureClassErrorBuilder
+):
     if feature.id.name.startswith("__") or feature.id.name.startswith("_chalk"):
         lsp_builder.add_diagnostic(
             message="Feature names cannot begin with '_chalk' or '__'.",
-            range=lsp_builder.property_range(feature.id.attributeName),
+            range=lsp_builder.property_range(feature.id.attributeName or feature.id.name),
             label="protected name",
             code="24",
         )
@@ -341,214 +340,12 @@ def _validate_feature_names(feature: UpsertFeatureGQL, builder: _LogBuilder, lsp
         )
 
 
-def _validate_resolver_feature_cycles(
-    fqn_to_feature: Mapping[str, UpsertFeatureGQL],
-    resolver: UpsertResolverGQL,
-    builder: _LogBuilder,
-    lsp_builder: ResolverErrorBuilder,
-):
-    input_root_fqns: set[str] = set()
-    for inp in resolver.inputs or []:
-        if fqn_to_feature[inp.underlying.fqn].scalarKind and not fqn_to_feature[inp.underlying.fqn].scalarKind.primary:
-            if not inp.path:
-                input_root_fqns.add(inp.underlying.fqn)
-            else:
-                path_parsed: List[str] = []
-                path_parsed.append(inp.path[0].parent.namespace)
-                path_parsed.append(inp.path[0].parent.name)
-                for x in inp.path:
-                    path_parsed.append(x.child.name)
-                input_root_fqns.add(".".join(path_parsed))
-        # FIXME: Validate cycles for has-one and has-many features
-
-    output_features = list(resolver.output.features or [])
-    output_features += [f for df in resolver.output.dataframes for f in (df.columns or [])]
-    output_fqns: set[str] = set()
-    for f in output_features:
-        fqn = f"{f.namespace}.{f.name}"
-        root_fqn = f.fqn  # for output features, the fqn is the root fqn
-        if fqn_to_feature[fqn].scalarKind is not None and not fqn_to_feature[fqn].scalarKind.primary:
-            output_fqns.add(root_fqn)
-
-    shared_features = output_fqns & input_root_fqns
-    if len(shared_features) > 0:
-        builder.add_error(
-            header=f'Resolver "{resolver.fqn}" has the same feature in its input and output (i.e. a cycle).',
-            subheader=f"""The inputs and outputs of a resolver can't contain the same exact feature.
-The resolver "{resolver.fqn}" contains the following shared features in its inputs and outputs: {sorted(shared_features)}
-""",
-        )
-
-
-def _validate_resolver_input_and_output_namespace(resolver: UpsertResolverGQL, builder: _LogBuilder):
-    input_features = [i.underlying for i in (resolver.inputs or [])]
-    output_features = resolver.output.features or []
-    all_features = input_features + output_features
-    all_namespaces = {f.namespace for f in all_features}
-    if len(all_namespaces) > 1:
-        builder.add_warning(
-            header=f'Resolver "{resolver.fqn}" has output and input features from multiple namespaces.',
-            subheader=f"""All outputs and inputs of a resolver should be rooted in the same namespace.
-The resolver "{resolver.fqn}" references features in the namespaces {oxford_comma_list(sorted(all_namespaces))}.
-See https://docs.chalk.ai/docs/resolver-outputs for information about valid resolver outputs.
-""",
-        )
-
-
-MissingMetadata = Dict[GraphLogSeverity, List[str]]
-
-
 def _validate_metadata(
-    features: List[UpsertFeatureGQL],
-    resolvers: List[UpsertResolverGQL],
-    builder: _LogBuilder,
-    request_config: Optional[ProjectSettingsGQL],
+    features: list[UpsertFeatureGQL],
+    resolvers: list[UpsertResolverGQL],
+    builder: ClientLogBuilder,
+    request_config: ProjectSettingsGQL | None,
 ):
-    class MissingMetadataFeature:
-        def __init__(self, feature: UpsertFeatureGQL, missing_metadata: MissingMetadata):
-            super().__init__()
-            self.feature = feature
-            self.missing_metadata = missing_metadata
-
-    class MissingMetadataResolver:
-        def __init__(self, resolver: UpsertResolverGQL, missing_metadata: MissingMetadata):
-            super().__init__()
-            self.resolver = resolver
-            self.missing_metadata = missing_metadata
-
-    def get_missing_metadata(
-        entity: Union[UpsertFeatureGQL, UpsertResolverGQL],
-        attribute_map: Dict[str, str],
-        entity_type: Literal["feature", "resolver"],
-    ) -> MissingMetadata:
-        res: MissingMetadata = collections.defaultdict(list)
-        if entity_type == "feature":
-            settings = request_config.validation.feature.metadata
-        elif entity_type == "resolver":
-            settings = request_config.validation.resolver.metadata
-        else:
-            raise ValueError(f"Unrecognized type {entity_type}")
-        for setting in settings:
-            metadata_name = setting.name
-            try:
-                severity = GraphLogSeverity(setting.missing.upper())
-            except:
-                """If this fails, _validate_metadata_config should log the error"""
-                continue
-            if metadata_name in attribute_map:
-                attribute_name = attribute_map[metadata_name]
-                value = getattr(entity, attribute_name)
-                is_missing = value is None or (isinstance(value, list) and len(value) == 0)
-                if is_missing:
-                    res[severity].append(metadata_name)
-        return res
-
-    def get_missing_metadata_features_log(
-        missing_metadata_features: List[MissingMetadataFeature], severity: GraphLogSeverity
-    ) -> Union[Tuple[str, str], None]:
-        if not missing_metadata_features:
-            return None
-
-        missing_messages = []
-        feature_str = "feature"
-        missing_metadata_header_str = "missing metadata"
-        max_name_len = max([len(w.feature.id.name) for w in missing_metadata_features])
-        feature_column_width = max(max_name_len, len(feature_str)) + 1
-        get_padding = lambda s: feature_column_width - len(s)
-
-        first_feature = missing_metadata_features[0].feature
-        header = f'"{to_camel_case(first_feature.id.namespace)}" features have missing metadata'
-        subheader = f"  Filepath: {first_feature.namespacePath}\n\n"
-
-        for wrapper in missing_metadata_features:
-            missing_metadata = wrapper.missing_metadata.get(severity)
-            if not missing_metadata:
-                continue
-            padding_1 = get_padding(wrapper.feature.id.name)
-            missing_metadata_str = ", ".join(missing_metadata)
-            missing_messages.append(f"      {wrapper.feature.id.name}{' ' * padding_1}: {missing_metadata_str}")
-
-        if not missing_messages:
-            return None
-
-        padding_2 = get_padding(feature_str)
-        subheader += f"      {'-' * len(feature_str)}{' ' * padding_2}  {'-' * len(missing_metadata_header_str)}\n"
-        subheader += f"      {feature_str}{' ' * padding_2}  {missing_metadata_header_str}\n"
-        subheader += f"      {'-' * len(feature_str)}{' ' * padding_2}  {'-' * len(missing_metadata_header_str)}\n"
-        subheader += "\n".join(missing_messages)
-
-        return header, subheader
-
-    def get_missing_metadata_resolver_log(
-        missing_metadata_resolver: MissingMetadataResolver, severity: GraphLogSeverity
-    ) -> Union[Tuple[str, str], None]:
-        missing_metadata = missing_metadata_resolver.missing_metadata.get(severity)
-        if missing_metadata is None:
-            return
-        missing_metadata_str = ", ".join(missing_metadata)
-        resolver_str = "resolver"
-        missing_metadata_header_str = "missing metadata"
-        resolver = missing_metadata_resolver.resolver
-        max_name_len = len(resolver.fqn)
-        feature_column_width = max(max_name_len, len(resolver_str)) + 1
-        get_padding = lambda s: feature_column_width - len(s)
-        padding_1 = get_padding(resolver.fqn)
-
-        header = f'Resolver "{resolver.fqn}" has missing metadata'
-        subheader = f"  Filepath: {missing_metadata_resolver.resolver.filename}\n\n"
-        message = f"      {resolver.fqn}{' ' * padding_1}: {missing_metadata_str}"
-
-        padding_2 = get_padding(resolver_str)
-        subheader += f"      {'-' * len(resolver_str)}{' ' * padding_2}  {'-' * len(missing_metadata_header_str)}\n"
-        subheader += f"      {resolver_str}{' ' * padding_2}  {missing_metadata_header_str}\n"
-        subheader += f"      {'-' * len(resolver_str)}{' ' * padding_2}  {'-' * len(missing_metadata_header_str)}\n"
-        subheader += message
-
-        return header, subheader
-
-    def build_feature_log_for_each_severity(missing_metadata_features: List[MissingMetadataFeature]):
-        if not missing_metadata_features:
-            return
-
-        for severity in GraphLogSeverity:
-            log_or_none = get_missing_metadata_features_log(
-                missing_metadata_features=missing_metadata_features, severity=GraphLogSeverity(severity)
-            )
-            if log_or_none is None:
-                continue
-            header, subheader = log_or_none
-            builder.add_log(header=header, subheader=subheader, severity=GraphLogSeverity(severity))
-
-    def build_resolver_log_for_each_severity(missing_metadata_resolvers: List[MissingMetadataResolver]):
-        if not missing_metadata_resolvers:
-            return
-
-        for severity in GraphLogSeverity:
-            for resolver in missing_metadata_resolvers:
-                log_or_none = get_missing_metadata_resolver_log(
-                    missing_metadata_resolver=resolver, severity=GraphLogSeverity(severity)
-                )
-                if log_or_none is None:
-                    continue
-                header, subheader = log_or_none
-                builder.add_log(header=header, subheader=subheader, severity=GraphLogSeverity(severity))
-
-    def validate_feature_metadata(
-        namespace_features: List[UpsertFeatureGQL],
-    ) -> List[MissingMetadataFeature]:
-        wrapped_features = []
-        for nf in namespace_features:
-            missing_metadata = get_missing_metadata(
-                entity=nf,
-                attribute_map=_METADATA_STRING_TO_FEATURE_ATTRIBUTE_MAP,
-                entity_type="feature",
-            )
-            if not missing_metadata:
-                continue
-            wrapped_features.append(MissingMetadataFeature(feature=nf, missing_metadata=missing_metadata))
-
-        return wrapped_features
-
     if not (request_config and request_config.validation):
         return
 
@@ -558,19 +355,25 @@ def _validate_metadata(
             namespace_to_features[f.id.namespace].append(f)
 
         for ns_features in namespace_to_features.values():
-            wf = validate_feature_metadata(namespace_features=ns_features)
-            build_feature_log_for_each_severity(missing_metadata_features=wf)
+            wf = validate_feature_metadata(
+                settings=request_config.validation.feature.metadata, namespace_features=ns_features
+            )
+            build_feature_log_for_each_severity(builder=builder, missing_metadata_features=wf)
     if request_config.validation.resolver and request_config.validation.resolver.metadata:
-        missing_resolver_metadatas: List[MissingMetadataResolver] = []
+        missing_resolver_metadatas: list[MissingMetadataResolver] = []
         for r in resolvers:
-            metadata = get_missing_metadata(r, _METADATA_STRING_TO_RESOLVER_ATTRIBUTE_MAP, "resolver")
+            metadata = get_missing_metadata(
+                entity=r,
+                attribute_map=METADATA_STRING_TO_RESOLVER_ATTRIBUTE_MAP,
+                settings=request_config.validation.resolver.metadata,
+            )
             missing_resolver_metadatas.append(MissingMetadataResolver(resolver=r, missing_metadata=metadata))
-        build_resolver_log_for_each_severity(missing_resolver_metadatas)
+        build_resolver_log_for_each_severity(builder, missing_resolver_metadatas)
 
 
 def _validate_metadata_config(
-    builder: _LogBuilder,
-    request_config: Optional[ProjectSettingsGQL],
+    builder: ClientLogBuilder,
+    request_config: ProjectSettingsGQL | None,
 ):
     if not (request_config and request_config.validation):
         return
@@ -595,7 +398,7 @@ def _validate_metadata_config(
                     ),
                 )
 
-            if metadata_name not in _METADATA_STRING_TO_FEATURE_ATTRIBUTE_MAP:
+            if metadata_name not in METADATA_STRING_TO_FEATURE_ATTRIBUTE_MAP:
                 builder.add_warning(
                     header=f'Found invalid feature metadata "{metadata_name}" in config',
                     subheader=(
@@ -603,6 +406,7 @@ def _validate_metadata_config(
                         f" Please consider removing it from chalk.yml"
                     ),
                 )
+
     if request_config.validation.resolver and request_config.validation.resolver.metadata:
         for missing_metadata in request_config.validation.resolver.metadata:
             metadata_name = missing_metadata.name
@@ -621,7 +425,7 @@ def _validate_metadata_config(
                     ),
                 )
 
-            if metadata_name not in _METADATA_STRING_TO_RESOLVER_ATTRIBUTE_MAP:
+            if metadata_name not in METADATA_STRING_TO_RESOLVER_ATTRIBUTE_MAP:
                 builder.add_warning(
                     header=f'Found invalid feature metadata "{metadata_name}" in config',
                     subheader=(
@@ -634,7 +438,7 @@ def _validate_metadata_config(
 def _validate_namespace_primary_key(
     namespace: str,
     features: List[UpsertFeatureGQL],
-    builder: _LogBuilder,
+    builder: ClientLogBuilder,
     singleton_namespaces: set[str],
 ):
     if namespace in singleton_namespaces:
@@ -666,7 +470,7 @@ def validate_graph(request: UpsertGraphGQL) -> List[UpdateGraphError]:
     for feature in request.features or []:
         namespaces[feature.id.namespace].append(feature)
 
-    builder = _LogBuilder()
+    builder = ClientLogBuilder()
 
     # Validate the features
     _validate_feature_names_unique(request.features or [], builder)
@@ -727,7 +531,7 @@ def validate_graph(request: UpsertGraphGQL) -> List[UpdateGraphError]:
             _validate_no_feature_times_as_input(
                 fqn_to_feature=fqn_to_feature, resolver=resolver, builder=builder, lsp_builder=lsp_builder
             )
-        except Exception as e:
+        except Exception:
             err = traceback.format_exc()
             builder.add_error(
                 header=f'Failed to validate resolver "{resolver.fqn}"',
