@@ -1,14 +1,17 @@
 import collections
+import collections.abc
+import dataclasses
 import inspect
 import json
 from datetime import timedelta
-from typing import Any, Callable, ClassVar, Dict, List, Optional, Sequence, Set, Tuple, Type, Union, cast
+from typing import Any, Callable, ClassVar, Collection, Dict, List, Optional, Sequence, Set, Tuple, Type, Union, cast
 
 import pyarrow as pa
 from pydantic import BaseModel
 
 from chalk import DataFrame
 from chalk._gen.chalk.arrow.v1 import arrow_pb2 as arrow_pb
+from chalk._gen.chalk.artifacts.v1 import export_pb2 as export_pb
 from chalk._gen.chalk.expression.v1 import expression_pb2 as expr_pb
 from chalk._gen.chalk.graph.v1 import graph_pb2 as pb
 from chalk._gen.chalk.graph.v1 import sources_pb2 as sources_pb
@@ -24,16 +27,18 @@ from chalk.features import (
     TPrimitive,
     Underscore,
     ensure_feature,
+    unwrap_feature,
 )
+from chalk.features._encoding.converter import PrimitiveFeatureConverter
 from chalk.features._encoding.rich import TRich
 from chalk.features.pseudofeatures import PSEUDONAMESPACE
 from chalk.features.resolver import (
     Cron,
     OfflineResolver,
     OnlineResolver,
+    ParseInfo,
     Resolver,
     ResolverArgErrorHandler,
-    ResolverRegistry,
     SinkResolver,
     StateDescriptor,
     StreamResolver,
@@ -46,8 +51,9 @@ from chalk.features.underscore import (
     UnderscoreRoot,
     UnderscoreUnaryOp,
 )
+from chalk.parsed._proto.utils import build_failed_import, seconds_to_proto_duration, timedelta_to_proto_duration
 from chalk.parsed.expressions import is_valid_operation
-from chalk.parsed.utils import convert_raw_duration, seconds_to_proto_duration, timedelta_to_proto_duration
+from chalk.parsed.utils import convert_raw_duration
 from chalk.sql import (
     BigQuerySourceImpl,
     CloudSQLSourceImpl,
@@ -58,7 +64,7 @@ from chalk.sql import (
     SnowflakeSourceImpl,
     SQLiteSourceImpl,
 )
-from chalk.sql._internal.sql_source import BaseSQLSource
+from chalk.sql._internal.sql_source import BaseSQLSource, SQLSourceKind
 from chalk.streams import KafkaSource, KinesisSource, StreamSource
 from chalk.streams.types import (
     StreamResolverParam,
@@ -66,12 +72,22 @@ from chalk.streams.types import (
     StreamResolverParamMessage,
     StreamResolverParamMessageWindow,
 )
+from chalk.utils import paths
 from chalk.utils.collections import get_unique_item
 from chalk.utils.duration import CronTab, Duration, parse_chalk_duration
 
+_CHALK_ANON_SQL_SOURCE_PREFIX = "__chalk_anon_sql_source_"
+_CHALK_ANON_STREAM_SOURCE_PREFIX = "__chalk_anon_stream_source_"
+
+
+@dataclasses.dataclass
+class GraphConversionResult:
+    graph: pb.Graph
+    errors: List[export_pb.FailedImport]
+
 
 class ToProtoConverter:
-    _agg_to_protobuf: ClassVar[Dict[str, int]] = {
+    _agg_to_protobuf: ClassVar[Dict[str, "expr_pb.AggregateFunction"]] = {
         "sum": expr_pb.AGGREGATE_FUNCTION_SUM,
         "max": expr_pb.AGGREGATE_FUNCTION_MAX,
         "min": expr_pb.AGGREGATE_FUNCTION_MIN,
@@ -94,6 +110,16 @@ class ToProtoConverter:
         "continuous": pb.WINDOW_MODE_CONTINUOUS,
         "cdc": pb.WINDOW_MODE_CDC,
     }
+    _source_kind_to_enum: ClassVar[Dict[SQLSourceKind, "sources_pb.DatabaseSourceType"]] = {
+        SQLSourceKind.bigquery: sources_pb.DatabaseSourceType.DATABASE_SOURCE_TYPE_BIGQUERY,
+        SQLSourceKind.cloudsql: sources_pb.DatabaseSourceType.DATABASE_SOURCE_TYPE_CLOUDSQL,
+        SQLSourceKind.databricks: sources_pb.DatabaseSourceType.DATABASE_SOURCE_TYPE_DATABRICKS,
+        SQLSourceKind.mysql: sources_pb.DatabaseSourceType.DATABASE_SOURCE_TYPE_MYSQL,
+        SQLSourceKind.postgres: sources_pb.DatabaseSourceType.DATABASE_SOURCE_TYPE_POSTGRES,
+        SQLSourceKind.redshift: sources_pb.DatabaseSourceType.DATABASE_SOURCE_TYPE_REDSHIFT,
+        SQLSourceKind.snowflake: sources_pb.DatabaseSourceType.DATABASE_SOURCE_TYPE_SNOWFLAKE,
+        SQLSourceKind.sqlite: sources_pb.DatabaseSourceType.DATABASE_SOURCE_TYPE_SQLITE,
+    }
     _supported_stream_source_types: ClassVar[Tuple[Type[StreamSource], ...]] = (KafkaSource, KinesisSource)
     _supported_db_source_types: ClassVar[Tuple[Type[BaseSQLSource], ...]] = (
         BigQuerySourceImpl,
@@ -194,7 +220,9 @@ class ToProtoConverter:
             return sources_pb.StreamSource(
                 kafka=sources_pb.KafkaSource(
                     name=source_name,
-                    bootstrap_servers=source.bootstrap_server,
+                    bootstrap_servers=[source.bootstrap_server]
+                    if isinstance(source.bootstrap_server, str)
+                    else source.bootstrap_server,
                     topic=source.topic,
                     ssl_keystore_location=source.ssl_keystore_location,
                     ssl_ca_file=source.ssl_ca_file,
@@ -236,7 +264,7 @@ class ToProtoConverter:
         try:
             return ToProtoConverter._convert_stream_source(source)
         except Exception as e:
-            raise ValueError(f"Could not convert stream source '{source.name}': {e}")
+            raise ValueError(f"Could not convert stream source '{source.name}'") from e
 
     @staticmethod
     def convert_engine_args(engine_args: Dict[str, Any]) -> Dict[str, arrow_pb.ScalarValue]:
@@ -363,7 +391,7 @@ class ToProtoConverter:
         try:
             return ToProtoConverter._convert_sql_source(source)
         except Exception as e:
-            raise ValueError(f"Could not convert SQL source '{source.name}': {e}")
+            raise ValueError(f"Could not convert SQL source '{source.name}'") from e
 
     @staticmethod
     def convert_cron_filter(cron: Union[CronTab, Duration, Cron]) -> Optional[CronFilterWithFeatureArgs]:
@@ -432,9 +460,9 @@ class ToProtoConverter:
         raw_rhs = f.rhs
 
         if isinstance(raw_lhs, FeatureWrapper):
-            raw_lhs = raw_lhs._chalk_feature
+            raw_lhs = unwrap_feature(raw_lhs)
         if isinstance(raw_rhs, FeatureWrapper):
-            raw_rhs = raw_rhs._chalk_feature
+            raw_rhs = unwrap_feature(raw_rhs)
 
         converted_lhs = raw_lhs
         converted_rhs = raw_rhs
@@ -450,7 +478,9 @@ class ToProtoConverter:
 
         if isinstance(raw_lhs, Feature) and isinstance(raw_rhs, TimeDelta):
             # This means that the filter was a before() or after()
-            duration_converter = FeatureConverter(name="helper", is_nullable=False, pyarrow_dtype=pa.duration("us"))
+            duration_converter = PrimitiveFeatureConverter(
+                name="helper", is_nullable=False, pyarrow_dtype=pa.duration("us")
+            )
             converted_rhs = expr_pb.LogicalExprNode(
                 literal=duration_converter.from_primitive_to_protobuf_single(raw_rhs.to_std())
             )
@@ -476,7 +506,7 @@ class ToProtoConverter:
                     raise ValueError("rhs must be an iterable when operation is 'in'/'not in'")
                 prim_values = tuple(raw_lhs.converter.from_rich_to_primitive(x) for x in raw_rhs)
                 list_dtype = pa.large_list(raw_lhs.converter.pyarrow_dtype)
-                list_converter = FeatureConverter(name="helper", is_nullable=False, pyarrow_dtype=list_dtype)
+                list_converter = PrimitiveFeatureConverter(name="helper", is_nullable=False, pyarrow_dtype=list_dtype)
                 converted_rhs = expr_pb.LogicalExprNode(
                     literal=list_converter.from_primitive_to_protobuf_single(prim_values),
                 )
@@ -493,32 +523,32 @@ class ToProtoConverter:
     @staticmethod
     def _resolve_source_name(source: Union[BaseSQLSource, StreamSource]) -> str:
         if isinstance(source, BaseSQLSource):
-            return source.name or f"__chalk_anon_sql_source_{id(source)}"
+            return source.name or f"{_CHALK_ANON_SQL_SOURCE_PREFIX}{id(source)}"
         elif isinstance(source, StreamSource):
-            return source.name or f"__chalk_anon_stream_source_{id(source)}"
+            return source.name or f"{_CHALK_ANON_STREAM_SOURCE_PREFIX}{id(source)}"
         else:
             raise ValueError(f"Unknown source type: {type(source).__name__}")
 
     @staticmethod
     def create_database_source_reference(source: BaseSQLSource) -> sources_pb.DatabaseSourceReference:
-        if isinstance(source, BigQuerySourceImpl):
-            source_type = sources_pb.DATABASE_SOURCE_TYPE_BIGQUERY
-        elif isinstance(source, CloudSQLSourceImpl):
-            source_type = sources_pb.DATABASE_SOURCE_TYPE_CLOUDSQL
-        elif isinstance(source, DatabricksSourceImpl):
-            source_type = sources_pb.DATABASE_SOURCE_TYPE_DATABRICKS
-        elif isinstance(source, MySQLSourceImpl):
-            source_type = sources_pb.DATABASE_SOURCE_TYPE_MYSQL
-        elif isinstance(source, RedshiftSourceImpl):
-            source_type = sources_pb.DATABASE_SOURCE_TYPE_REDSHIFT
-        elif isinstance(source, SnowflakeSourceImpl):
-            source_type = sources_pb.DATABASE_SOURCE_TYPE_SNOWFLAKE
-        elif isinstance(source, SQLiteSourceImpl):
-            source_type = sources_pb.DATABASE_SOURCE_TYPE_SQLITE
-        elif isinstance(source, PostgreSQLSourceImpl):
-            source_type = sources_pb.DATABASE_SOURCE_TYPE_POSTGRES
+        if source.kind == SQLSourceKind.bigquery:
+            source_type = sources_pb.DatabaseSourceType.DATABASE_SOURCE_TYPE_BIGQUERY
+        elif source.kind == SQLSourceKind.cloudsql:
+            source_type = sources_pb.DatabaseSourceType.DATABASE_SOURCE_TYPE_CLOUDSQL
+        elif source.kind == SQLSourceKind.databricks:
+            source_type = sources_pb.DatabaseSourceType.DATABASE_SOURCE_TYPE_DATABRICKS
+        elif source.kind == SQLSourceKind.mysql:
+            source_type = sources_pb.DatabaseSourceType.DATABASE_SOURCE_TYPE_MYSQL
+        elif source.kind == SQLSourceKind.postgres:
+            source_type = sources_pb.DatabaseSourceType.DATABASE_SOURCE_TYPE_POSTGRES
+        elif source.kind == SQLSourceKind.redshift:
+            source_type = sources_pb.DatabaseSourceType.DATABASE_SOURCE_TYPE_REDSHIFT
+        elif source.kind == SQLSourceKind.snowflake:
+            source_type = sources_pb.DatabaseSourceType.DATABASE_SOURCE_TYPE_SNOWFLAKE
+        elif source.kind == SQLSourceKind.sqlite:
+            source_type = sources_pb.DatabaseSourceType.DATABASE_SOURCE_TYPE_SQLITE
         else:
-            raise ValueError(f"Unknown database source type: {type(source).__name__}")
+            raise ValueError(f"Unknown database source kind: {source.kind}")
 
         return sources_pb.DatabaseSourceReference(
             name=ToProtoConverter._resolve_source_name(source),
@@ -549,7 +579,10 @@ class ToProtoConverter:
 
     @staticmethod
     def create_function_reference(
-        fn: Callable, definition: Optional[str] = None, filename: Optional[str] = None
+        fn: Callable,
+        definition: Optional[str] = None,
+        filename: Optional[str] = None,
+        source_line: Optional[int] = None,
     ) -> pb.FunctionReference:
         module = inspect.getmodule(fn)
         return pb.FunctionReference(
@@ -691,6 +724,7 @@ class ToProtoConverter:
         max_staleness_duration = None
         if f.max_staleness is not None:
             max_staleness_duration = timedelta_to_proto_duration(convert_raw_duration(f.max_staleness))
+        assert f.joined_class is not None
         res = pb.FeatureType(
             has_many=pb.HasManyFeatureType(
                 name=f.name,
@@ -702,6 +736,7 @@ class ToProtoConverter:
                 tags=f.tags,
                 owner=f.owner,
                 description=f.description,
+                attribute_name=f.attribute_name if hasattr(f, "attribute_name") else None,
             )
         )
 
@@ -731,6 +766,7 @@ class ToProtoConverter:
                 tags=f.tags,
                 owner=f.owner,
                 description=f.description,
+                attribute_name=f.attribute_name if hasattr(f, "attribute_name") else None,
             )
         )
         return res
@@ -748,6 +784,7 @@ class ToProtoConverter:
                 tags=f.tags,
                 owner=f.owner,
                 description=f.description,
+                attribute_name=f.attribute_name if hasattr(f, "attribute_name") else None,
             )
         )
 
@@ -764,6 +801,7 @@ class ToProtoConverter:
                 namespace=f.namespace,
                 is_autogenerated=f.is_autogenerated,
                 window_durations=[seconds_to_proto_duration(d) for d in f.window_durations],
+                attribute_name=f.attribute_name if hasattr(f, "attribute_name") else None,
             )
         )
 
@@ -788,15 +826,15 @@ class ToProtoConverter:
                 max_staleness_duration=timedelta_to_proto_duration(convert_raw_duration(f._raw_max_staleness))
                 if f._raw_max_staleness is not ... and f._raw_max_staleness is not None
                 else None,
-                offline_ttl_duration=timedelta_to_proto_duration(convert_raw_duration(f._offline_ttl))
-                if f._offline_ttl is not ... and f._offline_ttl is not None
+                offline_ttl_duration=timedelta_to_proto_duration(convert_raw_duration(f.offline_ttl))
+                if hasattr(f, "offline_ttl")
                 else None,
                 window_info=pb.WindowInfo(
                     duration=seconds_to_proto_duration(f.window_duration),
                 )
-                if f.is_windowed_pseudofeature
+                if f.window_duration is not None
                 else None,
-                etl_offline_to_online=f._raw_etl_offline_to_online,
+                etl_offline_to_online=f.raw_etl_offline_to_online,
                 tags=f.tags,
                 version=pb.VersionInfo(
                     default=f.version.default,
@@ -806,13 +844,14 @@ class ToProtoConverter:
                 else None,
                 last_for=ToProtoConverter.create_feature_reference(f.last_for) if f.last_for else None,
                 default_value=f.converter.from_primitive_to_protobuf_single(f.converter.primitive_default)
-                if f.converter._primitive_default is not ...
+                if f.converter.has_default
                 else None,
                 validations=ToProtoConverter.convert_validations(f._all_validations),
                 expression=ToProtoConverter.convert_underscore(f.underscore_expression)
                 if f.underscore_expression
                 else None,
                 no_display=f.no_display,
+                attribute_name=f.attribute_name if hasattr(f, "attribute_name") else None,
             )
         )
         return res
@@ -842,13 +881,15 @@ class ToProtoConverter:
         try:
             return ToProtoConverter._convert_feature(o)
         except Exception as e:
-            raise ValueError(f"Error converting feature '{o.namespace}.{o.name}': {e}")
+            raise RuntimeError(f"Error converting feature '{o.namespace}.{o.name}'") from e
 
     @staticmethod
     def convert_dataframe(df: Type[DataFrame]) -> pb.DataFrameType:
         return pb.DataFrameType(
             root_namespace=get_unique_item(x.root_namespace for x in df.columns if x.root_namespace != PSEUDONAMESPACE),
             optional_columns=[ToProtoConverter.create_feature_reference(ensure_feature(c)) for c in df.columns],
+            # CHA-3177 -- switch to using this syntax
+            # optional_columns=() if df.__references_feature_set__ is not None else [ToProtoConverter.create_feature_reference(ensure_feature(c)) for c in df.columns],
             required_columns=(),
             filter=expr_pb.LogicalExprNode(
                 binary_expr=expr_pb.BinaryExprNode(
@@ -925,41 +966,96 @@ class ToProtoConverter:
             schedule=schedule,
             when=ToProtoConverter.convert_filter(r.when) if r.when else None,
             cron_filter=cron_filter,
-            function=ToProtoConverter.create_function_reference(r.fn, r.function_definition, r.filename),
+            function=ToProtoConverter.create_function_reference(
+                r.fn, r.function_definition, r.filename, source_line=r.source_line
+            ),
             # TODO: Include the underscore definition
         )
 
     @staticmethod
     def convert_stream_resolver_param(p: StreamResolverParam) -> pb.StreamResolverParam:
         if isinstance(p, StreamResolverParamMessage):
+            try:
+                maybe_type = ToProtoConverter.convert_rich_type_to_protobuf(p.typ)
+            except:
+                # TODO: Stream message types are often more expressive than we can
+                #       currently serialize. But we don't want to block `chalk apply`
+                #       until we absolutely must need the Arrow type to be serialized.
+                maybe_type = None
+
             return pb.StreamResolverParam(
                 message=pb.StreamResolverParamMessage(
                     name=p.name,
-                    arrow_type=ToProtoConverter.convert_rich_type_to_protobuf(p.typ),
+                    arrow_type=maybe_type,
                 )
             )
         elif isinstance(p, StreamResolverParamMessageWindow):
+            try:
+                maybe_type = ToProtoConverter.convert_rich_type_to_protobuf(p.typ)
+            except:
+                # TODO: Stream message types are often more expressive than we can
+                #       currently serialize. But we don't want to block `chalk apply`
+                #       until we absolutely must need the Arrow type to be serialized.
+                maybe_type = None
             return pb.StreamResolverParam(
                 message_window=pb.StreamResolverParamMessageWindow(
                     name=p.name,
-                    arrow_type=ToProtoConverter.convert_rich_type_to_protobuf(p.typ),
+                    arrow_type=maybe_type,
                 )
             )
         elif isinstance(p, StreamResolverParamKeyedState):
             converter = FeatureConverter(
                 name="helper", is_nullable=False, rich_type=p.typ, rich_default=p.default_value
             )
-            return pb.StreamResolverParam(
-                state=pb.ResolverState(
-                    arrow_type=converter.convert_pa_dtype_to_proto_dtype(converter.pyarrow_dtype)
-                    if converter
-                    else None,
-                    initial=converter.from_rich_to_protobuf(p.default_value) if converter else None,
-                )
-            )
+            arrow_type = None
+            if converter:
+                try:
+                    arrow_type = converter.convert_pa_dtype_to_proto_dtype(converter.pyarrow_dtype)
+                except:
+                    # TODO: Stream message types are often more expressive than we can
+                    #       currently serialize. But we don't want to block `chalk apply`
+                    #       until we absolutely must need the Arrow type to be serialized.
+                    pass
+
+            initial = None
+            if converter:
+                try:
+                    initial = converter.from_rich_to_protobuf(p.default_value)
+                except:
+                    # TODO: Stream message types are often more expressive than we can
+                    #       currently serialize. But we don't want to block `chalk apply`
+                    #       until we absolutely must need the Arrow type to be serialized.
+                    pass
+
+            return pb.StreamResolverParam(state=pb.ResolverState(arrow_type=arrow_type, initial=initial))
         else:
             raise TypeError(f"Unknown param type: {type(p).__name__}")
 
+    @classmethod
+    def convert_parse_info(cls, info: ParseInfo) -> pb.ParseInfo:
+        try:
+            maybe_input_type = ToProtoConverter.convert_rich_type_to_protobuf(info.input_type)
+        except:
+            # TODO: Stream message types are often more expressive than we can
+            #       currently serialize. But we don't want to block `chalk apply`
+            #       until we absolutely must need the Arrow type to be serialized.
+            maybe_input_type = None
+
+        try:
+            maybe_output_type = ToProtoConverter.convert_rich_type_to_protobuf(info.output_type)
+        except:
+            # TODO: Stream message types are often more expressive than we can
+            #       currently serialize. But we don't want to block `chalk apply`
+            #       until we absolutely must need the Arrow type to be serialized.
+            maybe_output_type = None
+
+        return pb.ParseInfo(
+            parse_function_input_type=maybe_input_type,
+            parse_function_output_type=maybe_output_type,
+            parse_function=ToProtoConverter.create_function_reference(info.fn),
+            is_parse_function_output_optional=info.output_is_optional,
+        )
+
     @classmethod
     def convert_stream_resolver(cls, r: StreamResolver) -> pb.StreamResolver:
         mode = None
@@ -982,14 +1078,7 @@ class ToProtoConverter:
             if r.keys is not None
             else None,
             source=ToProtoConverter.create_stream_source_reference(r.source),
-            parse_info=pb.ParseInfo(
-                parse_function_input_type=ToProtoConverter.convert_rich_type_to_protobuf(r.parse.input_type),
-                parse_function_output_type=ToProtoConverter.convert_rich_type_to_protobuf(r.parse.output_type),
-                parse_function=ToProtoConverter.create_function_reference(r.parse.fn),
-                is_parse_function_output_optional=r.parse.output_is_optional,
-            )
-            if r.parse
-            else None,
+            parse_info=ToProtoConverter.convert_parse_info(r.parse) if r.parse else None,
             mode=mode,
             environments=r.environment or [],
             timeout_duration=timedelta_to_proto_duration(r.timeout) if r.timeout is not None else None,
@@ -997,7 +1086,9 @@ class ToProtoConverter:
             owner=r.owner,
             doc=r.doc,
             machine_type=r.machine_type,
-            function=ToProtoConverter.create_function_reference(r.fn, r.function_definition, r.filename),
+            function=ToProtoConverter.create_function_reference(
+                r.fn, r.function_definition, r.filename, source_line=r.source_line
+            ),
         )
 
     @staticmethod
@@ -1027,7 +1118,9 @@ class ToProtoConverter:
             owner=r.owner,
             environments=r.environment or [],
             timeout_duration=timedelta_to_proto_duration(r.timeout) if r.timeout is not None else None,
-            function=ToProtoConverter.create_function_reference(r.fn, r.function_definition, r.filename),
+            function=ToProtoConverter.create_function_reference(
+                r.fn, r.function_definition, r.filename, source_line=r.source_line
+            ),
         )
 
     @staticmethod
@@ -1046,36 +1139,48 @@ class ToProtoConverter:
         try:
             return ToProtoConverter._convert_resolver(r)
         except Exception as e:
-            raise ValueError(f"Error converting resolver '{r.fqn}': {e}")
+            raise ValueError(f"Error converting resolver '{r.fqn}'") from e
 
     @staticmethod
     def convert_graph(
         features_registry: Dict[str, Type[Features]],
-        resolver_registry: ResolverRegistry,
-        sql_source_registry: List[BaseSQLSource],
-        stream_source_registry: List[StreamSource],
-    ) -> pb.Graph:
+        resolver_registry: Collection[Resolver],
+        sql_source_registry: Collection[BaseSQLSource],
+        stream_source_registry: Collection[StreamSource],
+    ) -> GraphConversionResult:
         feature_sets = []
-
+        failed: List[export_pb.FailedImport] = []
         for feature_set in features_registry.values():
+            features = []
+            for f in feature_set.features:
+                try:
+                    features.append(ToProtoConverter.convert_feature(f))
+                except Exception as e:
+                    failed.append(build_failed_import(e, f"feature '{f.fqn}'"))
+
             feature_sets.append(
                 pb.FeatureSet(
                     name=feature_set.namespace,
-                    features=[ToProtoConverter.convert_feature(f) for f in feature_set.features],
+                    features=features,
                     is_singleton=feature_set.__chalk_is_singleton__,
                     max_staleness_duration=timedelta_to_proto_duration(feature_set.__chalk_max_staleness__),
                     tags=feature_set.__chalk_tags__,
                     owner=feature_set.__chalk_owner__,
                     doc=feature_set.__doc__,
                     etl_offline_to_online=feature_set.__chalk_etl_offline_to_online__,
+                    class_path=paths.get_classpath_or_name(feature_set),
                 )
             )
 
         resolvers = []
         stream_resolvers = []
         sink_resolvers = []
-        for resolver in resolver_registry.get_all_resolvers():
-            converted = ToProtoConverter.convert_resolver(resolver)
+        for resolver in resolver_registry:
+            try:
+                converted = ToProtoConverter.convert_resolver(resolver)
+            except Exception as e:
+                failed.append(build_failed_import(e, f"resolver '{resolver.fqn}'"))
+                continue
             if isinstance(converted, pb.Resolver):
                 resolvers.append(converted)
             elif isinstance(converted, pb.StreamResolver):
@@ -1083,21 +1188,29 @@ class ToProtoConverter:
             elif isinstance(converted, pb.SinkResolver):
                 sink_resolvers.append(converted)
             else:
-                raise TypeError(f"Unknown resolver type: {type(converted).__name__}")
+                failed.append(
+                    build_failed_import(
+                        f"Unknown resolver protobuf type: {type(converted).__name__}",
+                        description=f"resolver '{resolver.fqn}'",
+                    )
+                )
 
-        return pb.Graph(
-            feature_sets=feature_sets,
-            resolvers=resolvers,
-            stream_resolvers=stream_resolvers,
-            sink_resolvers=sink_resolvers,
-            database_sources=[
-                ToProtoConverter.convert_sql_source(s)
-                for s in sql_source_registry
-                if isinstance(s, ToProtoConverter._supported_db_source_types)
-            ],
-            stream_sources=[
-                ToProtoConverter.convert_stream_source(s)
-                for s in stream_source_registry
-                if isinstance(s, ToProtoConverter._supported_stream_source_types)
-            ],
+        return GraphConversionResult(
+            graph=pb.Graph(
+                feature_sets=feature_sets,
+                resolvers=resolvers,
+                stream_resolvers=stream_resolvers,
+                sink_resolvers=sink_resolvers,
+                database_sources=[
+                    ToProtoConverter.convert_sql_source(s)
+                    for s in sql_source_registry
+                    if isinstance(s, ToProtoConverter._supported_db_source_types)
+                ],
+                stream_sources=[
+                    ToProtoConverter.convert_stream_source(s)
+                    for s in stream_source_registry
+                    if isinstance(s, ToProtoConverter._supported_stream_source_types)
+                ],
+            ),
+            errors=failed,
         )
