@@ -25,6 +25,25 @@ JsonValue = TypeVar("JsonValue")
 
 
 class ParsedAnnotation:
+
+    __slots__ = (
+        "_features_cls",
+        "_attribute_name",
+        "_is_nullable",
+        "_is_feature_time",
+        "_is_primary",
+        "_is_document",
+        "_underlying",
+        "_unparsed_underlying",
+        "_parsed_annotation",
+        "_as_proto",
+        "_as_document",
+        "_as_features_cls",
+        "_as_dataframe",
+        "_as_vector",
+        "_as_feature",
+    )
+
     def __init__(
         self,
         features_cls: Optional[Type[Features]] = None,
@@ -41,11 +60,11 @@ class ParsedAnnotation:
         self._is_primary = False
         self._is_document = False
         self._underlying: Optional[Union[type, Feature]] = None
+        self._unparsed_underlying = underlying
         self._parsed_annotation: Optional[type] = None
         if underlying is not None:
             if features_cls is not None and attribute_name is not None:
                 raise ValueError("If specifying the underlying, do not specify (features_cls, attribute_name)")
-            self._parse_type(underlying)
         elif features_cls is None or attribute_name is None:
             raise ValueError(
                 "If not specifying the underlying, then both the (features_cls, attribute_name) must be provided"
@@ -54,6 +73,14 @@ class ParsedAnnotation:
         # resolve any forward references in the type annotations
         # Resolution happens lazily -- after everything is imported -- to avoid circular imports
 
+        # Caching some properties for faster lookup
+        self._as_proto: Optional[Type[ProtobufMessage]] = None
+        self._as_document: Optional[Type[Document]] = None
+        self._as_features_cls: Optional[Type[Features]] = None
+        self._as_dataframe: Optional[Type[DataFrame]] = None
+        self._as_vector: Optional[Type[Vector]] = None
+        self._as_feature: Optional[Feature] = None
+
     @property
     def parsed_annotation(self) -> type:
         """The parsed type annotation. It will be parsed if needed.
@@ -67,14 +94,13 @@ class ParsedAnnotation:
         return self._parsed_annotation
 
     def __str__(self):
-        annotation = self._parsed_annotation
-        if annotation is None:
-            assert self._features_cls is not None
-            assert self._attribute_name is not None
-            annotation = self._features_cls.__annotations__[self._attribute_name]
-        if isinstance(annotation, type):
-            return annotation.__name__
-        return str(annotation)
+        if self._features_cls is not None and self._attribute_name is not None:
+            typ = self._features_cls.__annotations__[self._attribute_name]
+        else:
+            typ = self._parsed_annotation or self._underlying or self._unparsed_underlying
+        if isinstance(typ, type):
+            typ = typ.__name__
+        return str(typ)
 
     def _get_globals_for_forward_references(self) -> Dict[str, Any]:
         """
@@ -87,9 +113,7 @@ class ParsedAnnotation:
         import chalk
         from chalk.features import FeatureSetBase
 
-        feature_classes = {}
-        for feature_set in FeatureSetBase.registry.values():
-            feature_classes[feature_set.__name__] = feature_set
+        feature_classes = {feature_set.__name__: feature_set for feature_set in FeatureSetBase.registry.values()}
 
         if not getattr(self._features_cls, "__chalk_is_loaded_from_notebook__", False):
             return feature_classes
@@ -106,16 +130,28 @@ class ParsedAnnotation:
         }
 
     def _parse_annotation(self):
-        assert self._attribute_name is not None
-        assert self._features_cls is not None
-        module = sys.modules.get(self._features_cls.__module__, None)
-        if module is not None:
-            existing_globalns = dict(getattr(module, "__dict__", {}))
-            existing_globalns.update(self._get_globals_for_forward_references())
-            hints = cached_get_type_hints(self._features_cls, include_extras=True, globalns=existing_globalns)
+        if self._unparsed_underlying is None:
+            assert self._attribute_name is not None
+            assert self._features_cls is not None
+            try:
+                # First, try to get the type hints without doing anything fancy
+                # Everything should already be imported properly
+                hints = cached_get_type_hints(self._features_cls, include_extras=True)
+            except:
+                # If there was an issue with that, then we'll fallback to trying to augment the globals, automatically injecting
+                # feature sets and chalkpy imports under their respective names
+                module = sys.modules.get(self._features_cls.__module__, None)
+                existing_globalns = None
+                if module is not None:
+                    existing_globalns = dict(getattr(module, "__dict__", {}))
+                    existing_globalns.update(self._get_globals_for_forward_references())
+                try:
+                    hints = cached_get_type_hints(self._features_cls, include_extras=True, globalns=existing_globalns)
+                except Exception as e:
+                    raise TypeError(f"Could not get type hints of {self._features_cls}") from e
+            parsed_annotation = hints[self._attribute_name]
         else:
-            hints = cached_get_type_hints(self._features_cls, include_extras=True)
-        parsed_annotation = hints[self._attribute_name]
+            parsed_annotation = self._unparsed_underlying
 
         try:
             self._parse_type(parsed_annotation)
@@ -153,10 +189,18 @@ class ParsedAnnotation:
         raise TypeError(message)
 
     def _parse_type(self, annotation: Union[type, Windowed, Annotated]):
+        from chalk.features._vector import Vector
+        from chalk.features.dataframe import DataFrame
         from chalk.features.feature_field import Feature
+        from chalk.features.feature_set import Features
         from chalk.features.feature_wrapper import FeatureWrapper, unwrap_feature
         from chalk.streams import Windowed
 
+        try:
+            from google.protobuf.message import Message as ProtobufMessage
+        except ImportError:
+            ProtobufMessage = None
+
         assert self._parsed_annotation is None, "The annotation was already parsed"
         if isinstance(annotation, Windowed):
             # If it's windowed, then unwrap it immediately, because Windowed annotations are really just a proxy to the underlying type
@@ -308,18 +352,26 @@ class ParsedAnnotation:
             return
 
         self._underlying = annotation
+        if isinstance(self._underlying, type):
+            if ProtobufMessage is not None and issubclass(self._underlying, ProtobufMessage):
+                self._as_proto = self._underlying
+
+            elif issubclass(self._underlying, Features):
+                self._as_features_cls = self._underlying
+
+            elif issubclass(self._underlying, DataFrame):
+                self._as_dataframe = self._underlying
+
+            elif issubclass(self._underlying, Vector):
+                self._as_vector = self._underlying
+
+        elif isinstance(self._underlying, Feature):
+            self._as_feature = self._underlying
 
     def as_proto(self) -> Optional[Type[ProtobufMessage]]:
-        try:
-            from google.protobuf.message import Message as ProtobufMessage
-        except ImportError:
-            return None
-
         if self._parsed_annotation is None:
             self._parse_annotation()
-        if not (isinstance(self._underlying, type) and issubclass(self._underlying, ProtobufMessage)):
-            return None
-        return self._underlying
+        return self._as_proto
 
     def as_document(self) -> Optional[Type[Document]]:
         if self._parsed_annotation is None:
@@ -337,41 +389,24 @@ class ParsedAnnotation:
         return self._is_nullable
 
     def as_features_cls(self) -> Optional[Type[Features]]:
-        from chalk.features import Features
-
         if self._parsed_annotation is None:
             self._parse_annotation()
-        assert self._underlying is not None
-        if not (isinstance(self._underlying, type) and issubclass(self._underlying, Features)):
-            return None
-        return self._underlying
+        return self._as_features_cls
 
     def as_dataframe(self) -> Optional[Type[DataFrame]]:
-        from chalk.features import DataFrame
-
         if self._parsed_annotation is None:
             self._parse_annotation()
-        if not (isinstance(self._underlying, type) and issubclass(self._underlying, DataFrame)):
-            return None
-        return self._underlying
+        return self._as_dataframe
 
     def as_vector(self) -> Optional[Type[Vector]]:
-        from chalk.features._vector import Vector
-
         if self._parsed_annotation is None:
             self._parse_annotation()
-
-        if not (isinstance(self._underlying, type) and issubclass(self._underlying, Vector)):
-            return None
-
-        return self._underlying
+        return self._as_vector
 
     def as_feature(self) -> Optional[Feature]:
-        from chalk.features import Feature
-
-        if not isinstance(self._underlying, Feature):
-            return None
-        return self._underlying
+        if self._parsed_annotation is None:
+            self._parse_annotation()
+        return self._as_feature
 
     def is_primary(self) -> bool:
         if self._parsed_annotation is None:
