@@ -1,3 +1,5 @@
+from __future__ import annotations
+
 from os import PathLike
 from typing import Any, Dict, Optional, Union, overload
 
@@ -10,6 +12,8 @@ from chalk.sql._internal.integrations.postgres import PostgreSQLSourceImpl
 from chalk.sql._internal.integrations.redshift import RedshiftSourceImpl
 from chalk.sql._internal.integrations.snowflake import SnowflakeSourceImpl
 from chalk.sql._internal.integrations.sqlite import SQLiteSourceImpl
+from chalk.sql._internal.integrations.trino import TrinoSourceImpl
+from chalk.sql._internal.sql_source_group import SQLSourceGroup
 from chalk.sql.finalized_query import FinalizedChalkQuery
 from chalk.sql.protocols import (
     BaseSQLSourceProtocol,
@@ -71,13 +75,14 @@ def SnowflakeSource(*, name: str, engine_args: Optional[Dict[str, Any]] = ...) -
 @overload
 def SnowflakeSource(
     *,
-    account_identifier: str = ...,
+    name: str | None = ...,
+    account_identifier: str | None = ...,
     warehouse: str = ...,
-    user: str = ...,
-    password: str = ...,
-    db: str = ...,
-    schema: str = ...,
-    role: str = ...,
+    user: str | None = ...,
+    password: str | None = ...,
+    db: str | None = ...,
+    schema: str | None = ...,
+    role: str | None = ...,
     engine_args: Optional[Dict[str, Any]] = ...,
 ) -> BaseSQLSourceProtocol:
     """You can also configure the integration directly using environment
@@ -86,6 +91,8 @@ def SnowflakeSource(
 
     Parameters
     ----------
+    name
+        Name of the integration. Not required unless if this PostgresSQLSource is used within SQL File Resolvers.
     account_identifier
         Your Snowflake account identifier.
     warehouse
@@ -218,13 +225,14 @@ def PostgreSQLSource(
 @overload
 def PostgreSQLSource(
     *,
-    host: str = ...,
-    port: Union[int, str] = ...,
-    db: str = ...,
-    user: str = ...,
-    password: str = ...,
-    engine_args: Optional[Dict[str, Any]] = ...,
-    async_engine_args: Optional[Dict[str, Any]] = ...,
+    name: str | None = ...,
+    host: str | None = ...,
+    port: int | str | None = ...,
+    db: str | None = ...,
+    user: str | None = ...,
+    password: str | None = ...,
+    engine_args: dict[str, Any] | None = ...,
+    async_engine_args: dict[str, Any] | None = ...,
 ) -> SQLSourceWithTableIngestProtocol:
     """You can also configure the integration directly using environment
     variables on your local machine or from those added through the
@@ -232,6 +240,8 @@ def PostgreSQLSource(
 
     Parameters
     ----------
+    name
+        Name of the integration. Not required unless if this PostgresSQLSource is used within SQL File Resolvers.
     host
         Name of host to connect to.
     port
@@ -291,7 +301,14 @@ def PostgreSQLSource(
     details.
     """
     return PostgreSQLSourceImpl(
-        host, port, db, user, password, name, engine_args=engine_args, async_engine_args=async_engine_args
+        host,
+        port,
+        db,
+        user,
+        password,
+        name,
+        engine_args=engine_args,
+        async_engine_args=async_engine_args,
     )
 
 
@@ -351,6 +368,7 @@ def MySQLSource(
 @overload
 def MySQLSource(
     *,
+    name: str | None = ...,
     host: str,
     port: Union[int, str] = ...,
     db: str = ...,
@@ -366,6 +384,8 @@ def MySQLSource(
 
     Parameters
     ----------
+    name
+        Name of the integration. Not required unless if this SQL Source is used within SQL File Resolvers.
     host
         Name of host to connect to.
     port
@@ -425,7 +445,14 @@ def MySQLSource(
     details.
     """
     return MySQLSourceImpl(
-        host, port, db, user, password, name, engine_args=engine_args, async_engine_args=async_engine_args
+        host,
+        port,
+        db,
+        user,
+        password,
+        name,
+        engine_args=engine_args,
+        async_engine_args=async_engine_args,
     )
 
 
@@ -484,7 +511,12 @@ def SQLiteFileSource(
     SQLSourceWithTableIngestProtocol
         The SQL source for use in Chalk resolvers.
     """
-    return SQLiteSourceImpl(filename=filename, name=name, engine_args=engine_args, async_engine_args=async_engine_args)
+    return SQLiteSourceImpl(
+        filename=filename,
+        name=name,
+        engine_args=engine_args,
+        async_engine_args=async_engine_args,
+    )
 
 
 @overload
@@ -542,6 +574,7 @@ def RedshiftSource(
 @overload
 def RedshiftSource(
     *,
+    name: str | None = ...,
     host: str = ...,
     db: str = ...,
     user: str = ...,
@@ -555,6 +588,8 @@ def RedshiftSource(
 
     Parameters
     ----------
+    name
+        Name of the integration. Not required unless if this SQL Source is used within SQL File Resolvers.
     host
         Name of host to connect to.
     db
@@ -667,6 +702,7 @@ def BigQuerySource(
 @overload
 def BigQuerySource(
     *,
+    name: str | None = ...,
     project: Optional[str] = ...,
     dataset: Optional[str] = ...,
     location: Optional[str] = ...,
@@ -680,6 +716,8 @@ def BigQuerySource(
 
     Parameters
     ----------
+    name
+        Name of the integration. Not required unless if this SQL Source is used within SQL File Resolvers.
     project
         The name of the GCP project for the BigQuery instance.
     dataset
@@ -799,6 +837,7 @@ def CloudSQLSource(
 @overload
 def CloudSQLSource(
     *,
+    name: str | None = ...,
     instance_name: Optional[str] = ...,
     db: Optional[str] = ...,
     user: Optional[str] = ...,
@@ -812,6 +851,8 @@ def CloudSQLSource(
 
     Parameters
     ----------
+    name
+        Name of the integration. Not required unless if this SQL Source is used within SQL File Resolvers.
     instance_name
         The name of the Cloud SQL instance, as defined in your GCP console.
     db
@@ -872,6 +913,133 @@ def CloudSQLSource(
     )
 
 
+@overload
+def TrinoSource() -> BaseSQLSourceProtocol:
+    """Connect to the only configured Trino database.
+
+    If you have only one Trino connection that you'd like
+    to add to Chalk, you do not need to specify any arguments
+    to construct the source in your code.
+
+    Returns
+    -------
+    BaseSQLSourceProtocol
+        The SQL source for use in Chalk resolvers.
+
+    Examples
+    --------
+    >>> source = TrinoSource()
+    """
+    ...
+
+
+@overload
+def TrinoSource(*, name: str, engine_args: Optional[Dict[str, Any]] = ...) -> BaseSQLSourceProtocol:
+    """Chalk's injects environment variables to support data integrations.
+
+    But what happens when you have two data sources of the same kind?
+    When you create a new data source from your dashboard,
+    you have an option to provide a name for the integration.
+    You can then reference this name in the code directly.
+
+    Parameters
+    ----------
+    name
+        Name of the integration, as configured in your dashboard.
+    engine_args
+        Additional arguments to use when constructing the SQLAlchemy engine. These arguments will be
+        merged with any default arguments from the named integration.
+
+    Returns
+    -------
+    BaseSQLSourceProtocol
+        The SQL source for use in Chalk resolvers.
+
+    Examples
+    --------
+    >>> source = TrinoSource(name="RISK")
+    """
+    ...
+
+
+@overload
+def TrinoSource(
+    *,
+    name: Optional[str] = None,
+    host: Optional[str] = None,
+    port: Optional[Union[int, str]] = None,
+    catalog: Optional[str] = None,
+    schema: Optional[str] = None,
+    user: Optional[str] = None,
+    password: Optional[str] = None,
+    engine_args: Optional[Dict[str, Any]] = None,
+) -> BaseSQLSourceProtocol:
+    """You can also configure the integration directly using environment
+    variables on your local machine or from those added through the
+    generic environment variable support (https://docs.chalk.ai/docs/env-vars).
+
+    Parameters
+    ----------
+    name
+        Name of the integration. Not required unless if this SQL Source is used within SQL File Resolvers.
+    host
+        Your Trino host.
+    port
+        Port number to use.
+    catalog
+        Catalog to use.
+    schema
+        Schema to use.
+    user
+        Trino username.
+    password
+        Trino password.
+    engine_args
+        Additional arguments to use when constructing the SQLAlchemy engine.
+
+    Returns
+    -------
+    BaseSQLSourceProtocol
+        The SQL source for use in Chalk resolvers.
+
+    Examples
+    --------
+    >>> import os
+    >>> trino = TrinoSource(
+    ...     host=os.getenv("TRINO_HOST"),
+    ...     port=os.getenv("TRINO_PORT"),
+    ...     catalog=os.getenv("TRINO_CATALOG"),
+    ...     schema=os.getenv("TRINO_SCHEMA"),
+    ...     user=os.getenv("TRINO_USER"),
+    ...     password=os.getenv("TRINO_PASSWORD"),
+    ... )
+    """
+    ...
+
+
+def TrinoSource(
+    *,
+    name: Optional[str] = None,
+    host: Optional[str] = None,
+    port: Optional[Union[int, str]] = None,
+    catalog: Optional[str] = None,
+    schema: Optional[str] = None,
+    user: Optional[str] = None,
+    password: Optional[str] = None,
+    engine_args: Optional[Dict[str, Any]] = None,
+) -> BaseSQLSourceProtocol:
+    """Create a Trino data source. SQL-based data sources
+    created without arguments assume a configuration in your
+    Chalk Dashboard. Those created with the `name=` keyword
+    argument will use the configuration for the integration
+    with the given name. And finally, those created with
+    explicit arguments will use those arguments to configure
+    the data source. See the overloaded signatures for more
+    details.
+    """
+    return TrinoSourceImpl(host, port, catalog, schema, user, password, name, engine_args=engine_args)
+
+
 @overload
 def DatabricksSource() -> BaseSQLSourceProtocol:
     """Connect to the only configured Databricks database.
@@ -923,6 +1091,7 @@ def DatabricksSource(*, name: str, engine_args: Optional[Dict[str, Any]] = ...)
 @overload
 def DatabricksSource(
     *,
+    name: str | None = ...,
     host: str = ...,
     http_path: str = ...,
     access_token: str = ...,
@@ -936,6 +1105,8 @@ def DatabricksSource(
 
     Parameters
     ----------
+    name
+        Name of the integration. Not required unless if this SQL Source is used within SQL File Resolvers.
     host
         Your Databricks host.
     http_path
@@ -1009,10 +1180,12 @@ __all__ = [
     "SQLiteFileSource",
     "SQLiteInMemorySource",
     "SnowflakeSource",
+    "TrinoSource",
     "FinalizedChalkQuery",
     "ChalkQueryProtocol",
     "StringChalkQueryProtocol",
     "TableIngestProtocol",
+    "SQLSourceGroup",
     "SQLSourceWithTableIngestProtocol",
     "IncrementalSettings",
 ]
