@@ -155,6 +155,7 @@ class PostgreSQLSourceImpl(BaseSQLSource, TableIngestMixIn, SQLSourceWithTableIn
         chalk_default_engine_args = {
             "pool_size": 20,
             "max_overflow": 60,
+            "pool_recycle": 90,
             "connect_args": {
                 "keepalives": 1,
                 "keepalives_idle": 30,
@@ -434,7 +435,7 @@ class PostgreSQLSourceImpl(BaseSQLSource, TableIngestMixIn, SQLSourceWithTableIn
             features = columns_to_features(parsed_table.column_names)
 
             restricted_schema = pa.schema([pa.field(k, v.converter.pyarrow_dtype) for (k, v) in features.items()])
-            parsed_table = parsed_table.select(features.keys())
+            parsed_table = parsed_table.select(list(features.keys()))
             parsed_table = parsed_table.cast(restricted_schema)
             parsed_table = parsed_table.rename_columns([x.root_fqn for x in features.values()])
             if len(parsed_table) > 0:
