@@ -2,15 +2,21 @@ from __future__ import annotations
 
 import concurrent.futures
 import contextlib
+import dataclasses
 import functools
+import json
 import os
 import queue
 import threading
+import typing
+import uuid
 from typing import TYPE_CHECKING, Any, Callable, Mapping, NewType, Optional, Sequence, cast
 
 import orjson
 import packaging.version
+import pyarrow
 import pyarrow as pa
+import pyarrow.compute as pc
 
 from chalk.clogging import chalk_logger
 from chalk.features import Feature
@@ -79,6 +85,57 @@ def _has_new_fetch_arrow_all():
 _WorkerId = NewType("_WorkerId", int)
 
 
+def _stage_prefix(unload_job_identifier: str, snowflake_unload_stage: str) -> str:
+    return f"{snowflake_unload_stage}/{unload_job_identifier}/"
+
+
+def _rewrite_query_for_unload(sql: str, unload_job_identifier: str, snowflake_unload_stage: str):
+    rewritten_sql = sql.rstrip(";\n ")
+    prefix = _stage_prefix(unload_job_identifier, snowflake_unload_stage)
+    new_query = f"""
+    COPY INTO {prefix} FROM ({rewritten_sql}) file_format = (type = 'parquet') overwrite=true header=true; /* {json.dumps({'unload_job_identifier': unload_job_identifier})} */
+    """
+    return new_query
+
+
+class ResultHandle(typing.Protocol):
+    @property
+    def estimated_uncompressed_size(self) -> int | None:
+        return None
+
+    def to_arrow(self) -> pyarrow.Table:
+        ...
+
+
+@dataclasses.dataclass(frozen=True)
+class ResultBatchResultHandle(ResultHandle):
+    result_batch: ResultBatch
+
+    @property
+    def estimated_uncompressed_size(self) -> int | None:
+        return self.result_batch.uncompressed_size
+
+    def to_arrow(self):
+        return self.result_batch.to_arrow()
+
+
+@dataclasses.dataclass(frozen=True)
+class UnloadedStorageFileResultHandle(ResultHandle):
+    uri: str
+    compressed_size: int
+
+    @property
+    def estimated_uncompressed_size(self) -> int | None:
+        # this is a (hopefully) conservative estimate.
+        return self.compressed_size * 8
+
+    def to_arrow(self):
+        import polars as pl
+
+        chalk_logger.info(f"Loading table from {self.uri}")
+        return pl.read_parquet(source=self.uri).to_arrow()
+
+
 class SnowflakeSourceImpl(BaseSQLSource):
     def __init__(
         self,
@@ -110,9 +167,9 @@ class SnowflakeSourceImpl(BaseSQLSource):
                 "client_prefetch_threads": min((os.cpu_count() or 1) * 2, 32),
                 "client_session_keep_alive": True,
                 "application_name": "chalkai_featurepipelines",
+                "application": "chalkai_featurepipelines",
             },
         )
-
         self.account_identifier = account_identifier or load_integration_variable(
             integration_name=name, name="SNOWFLAKE_ACCOUNT_ID"
         )
@@ -153,39 +210,6 @@ class SnowflakeSourceImpl(BaseSQLSource):
             query=query,
         )
 
-    def execute_to_result_handles(
-        self,
-        finalized_query: FinalizedChalkQuery,
-        columns_to_features: Callable[[Sequence[str]], Mapping[str, Feature]],
-        connection: Optional[Connection],
-    ):
-        # these imports are safe because the only way we end up here is if we have a valid SnowflakeSource constructed,
-        # which already gates this import
-        import snowflake.connector
-        from sqlalchemy.sql import Select
-
-        if isinstance(finalized_query.query, Select):
-            validate_dtypes_for_efficient_execution(finalized_query.query, _supported_sqlalchemy_types_for_pa_querying)
-
-        with (
-            self.get_engine().connect() if connection is None else contextlib.nullcontext(connection)
-        ) as sqlalchemy_cnx:
-            con = cast(snowflake.connector.SnowflakeConnection, sqlalchemy_cnx.connection.dbapi_connection)
-            chalk_logger.info("Established connection with Snowflake")
-            sql, positional_params, named_params = self.compile_query(finalized_query)
-            assert len(positional_params) == 0, "using named param style"
-            with con.cursor() as cursor:
-                chalk_logger.info("Acquired cursor for Snowflake query. Executing.")
-                res = cursor.execute(sql, named_params)
-                chalk_logger.info("Executed Snowflake query. Fetching results.")
-                chalk_logger.info(f"Compiled query: {sql}")
-                assert res is not None
-
-                chalk_logger.info("Fetching result batches from snowflake.")
-                ans = cursor.get_result_batches()
-                assert ans is not None, "No batches returned"
-                return ans
-
     def convert_db_types(self, v: Any, converter: FeatureConverter):
         """
         Overload this if a given DB type needs custom type conversion
@@ -238,7 +262,7 @@ class SnowflakeSourceImpl(BaseSQLSource):
                     series = pa_array_to_pl_series(tbl[col_name])
                     column = series.str.json_extract(feature.converter.polars_dtype).to_arrow().cast(expected_type)
             if actual_type != expected_type:
-                column = column.cast(expected_type)
+                column = column.cast(options=pc.CastOptions(target_type=expected_type, allow_time_truncate=True))
             if isinstance(column, pa.ChunkedArray):
                 column = column.combine_chunks()
             columns.append(column)
@@ -249,7 +273,7 @@ class SnowflakeSourceImpl(BaseSQLSource):
 
     @staticmethod
     def _download_worker(
-        result_batches: list[ResultBatch],
+        result_handles: list[ResultHandle],
         lock: threading.Lock,
         sem: MultiSemaphore | None,
         pa_table_queue: queue.Queue[tuple[pa.Table, int] | _WorkerId],
@@ -258,14 +282,14 @@ class SnowflakeSourceImpl(BaseSQLSource):
         try:
             while True:
                 with lock:
-                    if len(result_batches) == 0:
+                    if len(result_handles) == 0:
                         return
-                    x = result_batches.pop()
-                weight = x.uncompressed_size
+                    x: ResultHandle = result_handles.pop()
+                weight = x.estimated_uncompressed_size
                 as_arrow = None
                 if weight is None:
                     # This is possible if the chunk is "local", which I think snowflake does for small result batches
-                    as_arrow = x.to_arrow()
+                    as_arrow: pyarrow.Table = x.to_arrow()
                     weight = as_arrow.nbytes
                 if sem:
                     if weight > sem.initial_value:
@@ -300,7 +324,10 @@ class SnowflakeSourceImpl(BaseSQLSource):
         with (
             self.get_engine().connect() if connection is None else contextlib.nullcontext(connection)
         ) as sqlalchemy_cnx:
-            con = cast(snowflake.connector.SnowflakeConnection, sqlalchemy_cnx.connection.dbapi_connection)
+            con = cast(
+                snowflake.connector.SnowflakeConnection,
+                sqlalchemy_cnx.connection.dbapi_connection,
+            )
             chalk_logger.info("Established connection with Snowflake")
             sql, positional_params, named_params = self.compile_query(finalized_query)
             assert len(positional_params) == 0, "using named param style"
@@ -314,18 +341,51 @@ class SnowflakeSourceImpl(BaseSQLSource):
                 ) in finalized_query.temp_tables.values():
                     exit_stack.enter_context(
                         self._create_temp_table(
-                            create_temp_table, temp_table, drop_temp_table, sqlalchemy_cnx, temp_value
+                            create_temp_table,
+                            temp_table,
+                            drop_temp_table,
+                            sqlalchemy_cnx,
+                            temp_value,
                         )
                     )
                 with con.cursor() as cursor:
+                    job_id = str(uuid.uuid4())
+                    if query_execution_parameters.snowflake.snowflake_unload_stage is not None:
+                        chalk_logger.info(
+                            f"Executing query to unload data to Snowflake stage for unload {job_id=} {query_execution_parameters.snowflake.snowflake_unload_stage}"
+                        )
+                        sql = _rewrite_query_for_unload(
+                            sql=sql,
+                            unload_job_identifier=job_id,
+                            snowflake_unload_stage=query_execution_parameters.snowflake.snowflake_unload_stage,
+                        )
+
                     chalk_logger.info(f"Compiled query: {repr(sql)}")
+
                     res = cursor.execute(sql, named_params)
                     chalk_logger.info("Executed Snowflake query. Fetching results.")
                     assert res is not None
 
-                    chalk_logger.info("Fetching arrow tables from Snowflake.")
-                    result_batches = cursor.get_result_batches()
-                    assert result_batches is not None
+                    if query_execution_parameters.snowflake.snowflake_unload_stage is not None:
+                        # get the uris of the files that were unloaded
+                        prefix = _stage_prefix(job_id, query_execution_parameters.snowflake.snowflake_unload_stage)
+                        stage_list_cursor = cursor.execute(f"LIST {prefix}")
+                        if stage_list_cursor is None:
+                            chalk_logger.error("Failed to enumerate unloaded files in Snowflake stage")
+                            raise ValueError("Failed to enumerate unloaded files in Snowflake stage")
+                        stage_list = stage_list_cursor.fetchall()
+                        result_handles: list[ResultHandle] = []
+                        chalk_logger.info(f"Found {len(stage_list)} files in Snowflake stage at prefix: {prefix}")
+                        for row in stage_list:
+                            # name/size/md5/last_modified
+                            result_handles.append(UnloadedStorageFileResultHandle(uri=row[0], compressed_size=row[1]))
+                    else:
+                        chalk_logger.info("Fetching arrow tables from Snowflake.")
+                        result_handles: list[ResultHandle] = [
+                            ResultBatchResultHandle(batch) for batch in (cursor.get_result_batches() or [])
+                        ]
+
+                    assert result_handles is not None
                     yielded = False
                     max_weight = query_execution_parameters.max_prefetch_size_bytes
                     if max_weight <= 0:
@@ -334,9 +394,15 @@ class SnowflakeSourceImpl(BaseSQLSource):
                     pa_table_queue: queue.Queue[tuple[pa.Table, int] | _WorkerId] = queue.Queue()
                     sem = None if max_weight is None else MultiSemaphore(max_weight)
                     assert query_execution_parameters.num_client_prefetch_threads >= 1
+
                     futures = {
                         _WorkerId(i): self.executor.submit(
-                            self._download_worker, result_batches, lock, sem, pa_table_queue, _WorkerId(i)
+                            self._download_worker,
+                            result_handles,
+                            lock,
+                            sem,
+                            pa_table_queue,
+                            _WorkerId(i),
                         )
                         for i in range(query_execution_parameters.num_client_prefetch_threads)
                     }
@@ -367,7 +433,8 @@ class SnowflakeSourceImpl(BaseSQLSource):
                         if schema is not None:
                             features = columns_to_features(schema.names)
                             yield pa.RecordBatch.from_arrays(
-                                arrays=[[] for _ in features], names=[x.root_fqn for x in features.values()]
+                                arrays=[[] for _ in features],
+                                names=[x.root_fqn for x in features.values()],
                             )
                             return
                         else:
