@@ -1,5 +1,6 @@
 from __future__ import annotations
 
+import contextlib
 import dataclasses
 import inspect
 import json
@@ -14,6 +15,8 @@ from yaml.scanner import ScannerError
 from chalk import OfflineResolver, OnlineResolver
 from chalk._lsp.error_builder import SQLFileResolverErrorBuilder
 from chalk.features import DataFrame, Feature, Features, FeatureSetBase
+from chalk.features.namespace_context import build_namespaced_name
+from chalk.features.namespace_context import namespace as namespace_ctx
 from chalk.features.resolver import ResolverArgErrorHandler, StreamResolver
 from chalk.sql._internal.integrations.bigquery import BigQuerySourceImpl
 from chalk.sql._internal.integrations.cloudsql import CloudSQLSourceImpl
@@ -106,6 +109,7 @@ class IncrementalSettings(BaseModel):
 class CommentDict(BaseModel):
     source: str
     resolves: str
+    namespace: Optional[str]
     incremental: Optional[IncrementalSettings]
     tags: Optional[List[str]]
     environment: Optional[List[str]]
@@ -303,213 +307,233 @@ def get_sql_file_resolver(
             args=None,
         )
 
-    # validate inputs and outputs as real features in graph
-    inputs: List[Feature] = []
-    for arg in glot_result.args.values():
-        try:
-            inputs.append(Feature.from_root_fqn(arg))
-        except Exception:
-            message = f"SQL file resolver references an input feature '{arg}' which does not exist."
-            if arg in FeatureSetBase.registry:
-                message += f" It appears '{arg}' is a feature class, not a feature."
+    with (
+        namespace_ctx(parsed.comment_dict.namespace)
+        if parsed.comment_dict and parsed.comment_dict.namespace
+        else contextlib.nullcontext()
+    ):
+        # validate inputs and outputs as real features in graph
+        inputs: List[Feature] = []
+        for arg in glot_result.args.values():
+            try:
+                inputs.append(Feature.from_root_fqn(build_namespaced_name(name=arg)))
+            except Exception:
+                message = f"SQL file resolver references an input feature '{arg}' which does not exist."
+                if arg in FeatureSetBase.registry:
+                    message += f" It appears '{arg}' is a feature class, not a feature."
 
-            if not has_import_errors:
-                error_builder.add_diagnostic(
-                    message=message,
-                    code="152",
-                    label="input feature not recognized",
-                    range=error_builder.variable_range_by_name(arg),
-                )
-                errors.append(
-                    ResolverError(
-                        display=message,
-                        path=path,
-                        parameter=arg,
+                if not has_import_errors:
+                    error_builder.add_diagnostic_with_spellcheck(
+                        spellcheck_item=arg,
+                        spellcheck_candidates=[
+                            feature.fqn
+                            for feature in FeatureSetBase.registry[
+                                build_namespaced_name(name=parsed.namespace)
+                            ].features
+                        ],
+                        message=message,
+                        code="152",
+                        label="input feature not recognized",
+                        range=error_builder.variable_range_by_name(arg),
                     )
-                )
-    outputs: List[Feature] = []
-    query_fields: Dict[str, str] = {}
-    for variable, output in parsed.fields.items():
-        if output.endswith("*"):
-            features = FeatureSetBase.registry[parsed.namespace].features
-            for feature in features:
-                if (
-                    feature.is_scalar
-                    and not feature.is_autogenerated
-                    and not feature.is_pseudofeature
-                    and not feature.is_windowed
-                ):
-                    outputs.append(feature)
-            continue
-
-        unrecognized_output = False
-        try:
-            feature = Feature.from_root_fqn(output)
-            outputs.append(feature)
-            query_fields[variable] = output
-        except Exception:
-            if parsed.comment_dict.fields is not None:
-                split = output.split(".", 1)
-                namespace = split[0]
-                output_name = split[1]
-                if output_name in parsed.comment_dict.fields:
-                    field = parsed.comment_dict.fields.get(output_name)
-                    output = f"{namespace}.{field}"
-                    try:
-                        feature = Feature.from_root_fqn(output)
+                    errors.append(
+                        ResolverError(
+                            display=message,
+                            path=path,
+                            parameter=arg,
+                        )
+                    )
+        outputs: List[Feature] = []
+        query_fields: Dict[str, str] = {}
+        for variable, output in parsed.fields.items():
+            if output.endswith("*"):
+                features = FeatureSetBase.registry[build_namespaced_name(name=parsed.namespace)].features
+                for feature in features:
+                    if (
+                        feature.is_scalar
+                        and not feature.is_autogenerated
+                        and not feature.is_pseudofeature
+                        and not feature.is_windowed
+                    ):
                         outputs.append(feature)
-                        query_fields[output_name] = output
-                    except:
+                continue
+
+            unrecognized_output = False
+            try:
+                feature = Feature.from_root_fqn(output)
+                outputs.append(feature)
+                query_fields[variable] = output
+            except Exception:
+                if parsed.comment_dict.fields is not None:
+                    split = output.split(".", 1)
+                    namespace = split[0]
+                    output_name = split[1]
+                    if output_name in parsed.comment_dict.fields:
+                        field = parsed.comment_dict.fields.get(output_name)
+                        output = f"{namespace}.{field}"
+                        try:
+                            feature = Feature.from_root_fqn(build_namespaced_name(name=output))
+                            outputs.append(feature)
+                            query_fields[output_name] = output
+                        except:
+                            unrecognized_output = True
+                    else:
                         unrecognized_output = True
                 else:
                     unrecognized_output = True
-            else:
-                unrecognized_output = True
-        if unrecognized_output:
-            message = f"SQL file resolver references an output feature '{output}' which does not exist. "
-            if not has_import_errors:
-                value = output.split(".", 1)[1]
-                assert glot_result.glot is not None, f"Failed to parse {glot_result.sql_string}"
-                error_builder.add_diagnostic(
-                    message=message,
-                    code="153",
-                    label="output feature not recognized",
-                    range=error_builder.value_range_by_name(glot_result.glot, value),
-                )
-                errors.append(
-                    ResolverError(
-                        display=message,
-                        path=path,
-                        parameter=output,
+            if unrecognized_output:
+                message = f"SQL file resolver references an output feature '{output}' which does not exist. "
+
+                if not has_import_errors:
+                    value = output.split(".", 1)[1]
+                    assert glot_result.glot is not None, f"Failed to parse {glot_result.sql_string}"
+                    error_builder.add_diagnostic_with_spellcheck(
+                        spellcheck_item=output,
+                        spellcheck_candidates=[
+                            feature.fqn
+                            for feature in FeatureSetBase.registry[
+                                build_namespaced_name(name=parsed.namespace)
+                            ].features
+                        ],
+                        message=message,
+                        code="153",
+                        label="output feature not recognized",
+                        range=error_builder.value_range_by_name(glot_result.glot, value),
                     )
+                    errors.append(
+                        ResolverError(
+                            display=message,
+                            path=path,
+                            parameter=output,
+                        )
+                    )
+        if len(outputs) == 0:
+            message = (
+                "SQL file resolver has no detected outputs. Make sure that all select outputs are aliased to features "
+                "defined within the feature set referenced by the `resolves` parameter."
+            )
+            if has_import_errors:
+                return ResolverResult(
+                    resolver=None,
+                    errors=[],
+                    db=parsed.source,
+                    fields=None,
+                    args=None,
                 )
-    if len(outputs) == 0:
-        message = (
-            "SQL file resolver has no detected outputs. Make sure that all select outputs are aliased to features "
-            "defined within the feature set referenced by the `resolves` parameter."
-        )
-        if has_import_errors:
+            error_builder.add_diagnostic(
+                message=message,
+                code="154",
+                label="no outputs detected",
+                range=error_builder.sql_range(),
+            )
+            errors.append(ResolverError(display=message, path=path, parameter=None))
+
+        resolver_type_str = parsed.comment_dict.type if parsed.comment_dict.type else "online"
+        resolver_type = _RESOLVER_TYPES[resolver_type_str]
+
+        if resolver_type == StreamResolver:
+            return _get_stream_resolver(path, glot_result, parsed, outputs, error_builder)
+
+        incremental_dict = parsed.comment_dict.incremental.dict() if parsed.comment_dict.incremental else None
+        return_one = parsed.comment_dict.count
+        source = parsed.source
+        assert isinstance(source, BaseSQLSource), f"source {source} is not configured. Is the driver installed?"
+
+        # function for online resolver to process
+        def fn(
+            *input_values: Any,
+            database: BaseSQLSource = source,
+            sql_query: str = parsed.sql_string,
+            field_dict: Dict[str, str] = query_fields,
+            args_dict: Dict[str, str] = glot_result.args,
+            incremental: Optional[Dict[str, Any]] = incremental_dict,
+        ):
+            arg_dict = {arg: input_value for input_value, arg in zip(input_values, args_dict.keys())}
+            func = database.query_string(
+                query=sql_query,
+                fields=field_dict,
+                args=arg_dict,
+            )
+            if incremental:
+                func = func.incremental(**incremental)
+            elif return_one in [1, "one"]:
+                func = func.one()
+            elif return_one == "one_or_none":
+                func = func.one_or_none()
+            return func
+
+        if errors:
             return ResolverResult(
-                resolver=None,
-                errors=[],
-                db=parsed.source,
-                fields=None,
-                args=None,
+                resolver=None, errors=errors, db=parsed.source, fields=parsed.fields, args=glot_result.args
             )
-        error_builder.add_diagnostic(
-            message=message,
-            code="154",
-            label="no outputs detected",
-            range=error_builder.sql_range(),
-        )
-        errors.append(ResolverError(display=message, path=path, parameter=None))
+        if return_one is None:
+            # If the root ns of the inputs is the same as what is being resolved, then it is always return one
+            input_root_namespace = get_unique_item_if_exists(x.root_namespace for x in inputs)
+            output_root_namespace = get_unique_item(x.root_namespace for x in outputs)
+            if input_root_namespace is not None and input_root_namespace == output_root_namespace:
+                return_one = "one"
 
-    resolver_type_str = parsed.comment_dict.type if parsed.comment_dict.type else "online"
-    resolver_type = _RESOLVER_TYPES[resolver_type_str]
+        if return_one:
+            output = Features[tuple(outputs)]
+        else:
+            output = Features[DataFrame[tuple(outputs)]]
 
-    if resolver_type == StreamResolver:
-        return _get_stream_resolver(path, glot_result, parsed, outputs, error_builder)
+        default_args: List[Optional[ResolverArgErrorHandler]] = [
+            None if default_value is ... else ResolverArgErrorHandler(default_value)
+            for default_value in glot_result.default_args
+        ]
 
-    incremental_dict = parsed.comment_dict.incremental.dict() if parsed.comment_dict.incremental else None
-    return_one = parsed.comment_dict.count
-    source = parsed.source
-    assert isinstance(source, BaseSQLSource), f"source {source} is not configured. Is the driver installed?"
-
-    # function for online resolver to process
-    def fn(
-        *input_values: Any,
-        database: BaseSQLSource = source,
-        sql_query: str = parsed.sql_string,
-        field_dict: Dict[str, str] = query_fields,
-        args_dict: Dict[str, str] = glot_result.args,
-        incremental: Optional[Dict[str, Any]] = incremental_dict,
-    ):
-        arg_dict = {arg: input_value for input_value, arg in zip(input_values, args_dict.keys())}
-        func = database.query_string(
-            query=sql_query,
-            fields=field_dict,
-            args=arg_dict,
-        )
-        if incremental:
-            func = func.incremental(**incremental)
-        elif return_one in [1, "one"]:
-            func = func.one()
-        elif return_one == "one_or_none":
-            func = func.one_or_none()
-        return func
-
-    if errors:
-        return ResolverResult(
-            resolver=None, errors=errors, db=parsed.source, fields=parsed.fields, args=glot_result.args
-        )
-    if return_one is None:
-        # If the root ns of the inputs is the same as what is being resolved, then it is always return one
-        input_root_namespace = get_unique_item_if_exists(x.root_namespace for x in inputs)
-        output_root_namespace = get_unique_item(x.root_namespace for x in outputs)
-        if input_root_namespace is not None and input_root_namespace == output_root_namespace:
-            return_one = "one"
-
-    if return_one:
-        output = Features[tuple(outputs)]
-    else:
-        output = Features[DataFrame[tuple(outputs)]]
-
-    default_args: List[Optional[ResolverArgErrorHandler]] = [
-        None if default_value is ... else ResolverArgErrorHandler(default_value)
-        for default_value in glot_result.default_args
-    ]
-
-    filename = os.path.basename(path)
-    # attempt to instantiate the resolver
-    try:
-        assert resolver_type in (OnlineResolver, OfflineResolver)
-        assert isinstance(parsed.source, BaseSQLSource)
-        resolver = resolver_type(
-            filename=path,
-            function_definition=sql_string_result.sql_string,
-            fqn=filename.replace(".chalk.sql", ""),
-            doc=parsed.docstring,
-            inputs=inputs,
-            output=output,
-            fn=fn,
-            environment=parsed.comment_dict.environment,
-            tags=parsed.comment_dict.tags,
-            cron=parsed.comment_dict.cron,
-            machine_type=parsed.comment_dict.machine_type,
-            when=None,
-            state=None,
-            default_args=default_args,
-            owner=parsed.comment_dict.owner,
-            timeout=parsed.comment_dict.timeout,
-            is_sql_file_resolver=True,
-            data_sources=[parsed.source],
-            name=None,
-            source_line=None,
-            lsp_builder=get_resolver_error_builder(fn),
-            underscore=None,
-        )
-    except Exception as e:
-        message = f"SQL file resolver '{resolver_type_str.capitalize()}'  could not be instantiated, {e}"
-        error_builder.add_diagnostic(
-            message=message,
-            code="155",
-            label="resolver instantiation failed",
-            range=error_builder.full_range(),
-        )
-        errors.append(
-            ResolverError(
-                display=message,
-                path=path,
-                parameter=None,
+        filename = os.path.basename(path)
+        # attempt to instantiate the resolver
+        try:
+            assert resolver_type in (OnlineResolver, OfflineResolver)
+            assert isinstance(parsed.source, BaseSQLSource)
+            resolver = resolver_type(
+                filename=path,
+                function_definition=sql_string_result.sql_string,
+                fqn=filename.replace(".chalk.sql", ""),
+                doc=parsed.docstring,
+                inputs=inputs,
+                output=output,
+                fn=fn,
+                environment=parsed.comment_dict.environment,
+                tags=parsed.comment_dict.tags,
+                cron=parsed.comment_dict.cron,
+                machine_type=parsed.comment_dict.machine_type,
+                when=None,
+                state=None,
+                default_args=default_args,
+                owner=parsed.comment_dict.owner,
+                timeout=parsed.comment_dict.timeout,
+                is_sql_file_resolver=True,
+                data_sources=[parsed.source],
+                source_line=None,
+                lsp_builder=get_resolver_error_builder(fn),
+                underscore=None,
+                parse=None,
+            )
+        except Exception as e:
+            message = f"SQL file resolver '{resolver_type_str.capitalize()}'  could not be instantiated, {e}"
+            error_builder.add_diagnostic(
+                message=message,
+                code="155",
+                label="resolver instantiation failed",
+                range=error_builder.full_range(),
+            )
+            errors.append(
+                ResolverError(
+                    display=message,
+                    path=path,
+                    parameter=None,
+                )
+            )
+            return ResolverResult(
+                resolver=None, errors=errors, db=parsed.source, fields=parsed.fields, args=glot_result.args
             )
-        )
-        return ResolverResult(
-            resolver=None, errors=errors, db=parsed.source, fields=parsed.fields, args=glot_result.args
-        )
 
-    return ResolverResult(
-        resolver=resolver, errors=errors, db=parsed.source, fields=parsed.fields, args=glot_result.args
-    )
+        return ResolverResult(
+            resolver=resolver, errors=errors, db=parsed.source, fields=parsed.fields, args=glot_result.args
+        )
 
 
 def _get_sql_string(path: str) -> SQLStringResult:
@@ -684,8 +708,8 @@ def _get_sql_glot(
     if "source" not in comment_dict:
         """This would be caught by the following BaseModel.parse_obj() but the error message is bad"""
         message = (
-            f"The datasource is a required field for SQL file resolvers. "
-            f"Please add a comment ' -- source: my_name'  where 'my_name' refers to a named integration. "
+            "The datasource is a required field for SQL file resolvers. "
+            "Please add a comment '-- source: my_name'  where 'my_name' refers to a named integration."
         )
         error_builder.add_diagnostic(
             message=message,
@@ -713,8 +737,8 @@ def _get_sql_glot(
     if "resolves" not in comment_dict:
         """This would be caught by the following BaseModel.parse_obj() but the error message is bad"""
         message = (
-            f"A feature class must be specified for SQL file resolvers. "
-            f"Please add a comment ' -- resolves: my_name'  where 'my_name' refers to a feature class."
+            "A feature class must be specified for SQL file resolvers. "
+            "Please add a comment '-- resolves: my_name'  where 'my_name' refers to a feature class."
         )
         error_builder.add_diagnostic(
             message=message,
@@ -913,6 +937,12 @@ def _parse_glot(
         for possible_source in sources:
             source_type = _SOURCES.get(source_name)
             if source_type is not None and isinstance(possible_source, source_type):
+                # verifies whether the name of the datasource is the same as the
+                # data source, if so, we should return the datasource even if
+                # there are duplicates.
+                if possible_source.name in _SOURCES:
+                    source = possible_source
+                    break
                 if source:
                     message = (
                         f"SQL file resolver refers to '{source_name}' when more than one {source_name}-type source "
@@ -954,20 +984,16 @@ def _parse_glot(
             )
         )
     # get resolver fields: which columns selected will match to which chalk feature?
-    namespace = comment_dict.resolves
+    namespace = build_namespaced_name(namespace=comment_dict.namespace, name=to_snake_case(comment_dict.resolves))
     if namespace not in FeatureSetBase.registry:
-        snaked_namespace = to_snake_case(namespace)
-        if snaked_namespace in FeatureSetBase.registry:
-            namespace = snaked_namespace
-        else:
-            message = f"No @features class with the name '{namespace}'"
-            error_builder.add_diagnostic(
-                message=message,
-                code="151",
-                label="Unrecognized namespace ",
-                range=error_builder.comment_range_by_key("resolves"),
-            )
-            errors.append(ResolverError(display=message, path=path, parameter=source_name))
+        message = f"No @features class with the name '{namespace}'"
+        error_builder.add_diagnostic(
+            message=message,
+            code="151",
+            label="Unrecognized namespace ",
+            range=error_builder.comment_range_by_key("resolves"),
+        )
+        errors.append(ResolverError(display=message, path=path, parameter=source_name))
 
     if len(errors) > 0:
         return ParseResult(
@@ -1047,7 +1073,6 @@ def _get_stream_resolver(
             keys=None,  # TODO implement parse and keys for sql file resolvers?
             timestamp=None,
             source_line=None,
-            name=None,
             tags=None,
             lsp_builder=get_resolver_error_builder(fn),
         )
