@@ -156,6 +156,7 @@ class SQLSourceKind(str, Enum):
     redshift = "redshift"
     snowflake = "snowflake"
     sqlite = "sqlite"
+    trino = "trino"
 
 
 class BaseSQLSource(BaseSQLSourceProtocol):
@@ -188,10 +189,11 @@ class BaseSQLSource(BaseSQLSourceProtocol):
             for k, v in self._load_env_engine_args(name=self.name).items():
                 engine_args.setdefault(k, v)
                 async_engine_args.setdefault(k, v)
-        engine_args.setdefault("pool_pre_ping", env_var_bool("USE_CLIENT_POOL_PRE_PING"))
-        engine_args.setdefault("echo", CHALK_QUERY_LOGGING)
-        async_engine_args.setdefault("pool_pre_ping", env_var_bool("USE_CLIENT_POOL_PRE_PING"))
-        async_engine_args.setdefault("echo", CHALK_QUERY_LOGGING)
+        if getattr(self, "kind", None) != SQLSourceKind.trino:
+            engine_args.setdefault("pool_pre_ping", env_var_bool("USE_CLIENT_POOL_PRE_PING"))
+            engine_args.setdefault("echo", CHALK_QUERY_LOGGING)
+            async_engine_args.setdefault("pool_pre_ping", env_var_bool("USE_CLIENT_POOL_PRE_PING"))
+            async_engine_args.setdefault("echo", CHALK_QUERY_LOGGING)
         self._engine_args = engine_args
         self._async_engine_args = async_engine_args
         self._engine = None
@@ -227,6 +229,8 @@ class BaseSQLSource(BaseSQLSourceProtocol):
                 assert frame is not None
                 caller_frame = frame.f_back
                 assert caller_frame is not None
+                while caller_frame.f_globals["__name__"].startswith("chalk.sql") and caller_frame.f_back is not None:
+                    caller_frame = caller_frame.f_back
                 dir_path = os.path.dirname(os.path.realpath(inspect.getframeinfo(caller_frame).filename))
                 path = str(path)
                 relative_path = os.path.join(dir_path, path)
@@ -510,7 +514,7 @@ class BaseSQLSource(BaseSQLSourceProtocol):
                     data[features[k].root_fqn].append(self.convert_db_types(v, features[k].converter))
             # Only keep the columns provided by the schema
             schema = pa.schema([pa.field(v.root_fqn, v.converter.pyarrow_dtype) for v in features.values()])
-            return pa.RecordBatch.from_pydict(data, schema=schema)
+            return _convert_to_record_batch(data, schema=schema)
 
     def _execute_query_inefficient(
         self,
@@ -551,7 +555,7 @@ class BaseSQLSource(BaseSQLSourceProtocol):
                         data[features[k].root_fqn].append(self.convert_db_types(v, features[k].converter))
                 # Only keep the columns provided by the schema
                 schema = pa.schema([pa.field(v.root_fqn, v.converter.pyarrow_dtype) for v in features.values()])
-                return pa.RecordBatch.from_pydict(data, schema=schema)
+        return _convert_to_record_batch(data, schema=schema)
 
     def _execute_query_efficient(
         self,
@@ -711,3 +715,23 @@ def _extract_features(feature_set: Features, prefix: str, targets: List[Label],
 
 
 insert_size_limit = 10_000
+
+
+def _convert_to_record_batch(
+    data: Mapping[str, List[Any]],
+    schema: pa.Schema,
+) -> pa.RecordBatch:
+    """
+    Converts the provided dict of fqn --> values into a `pa.RecordBatch`.
+    This method will detect validation errors and report which FQN failed to be parsed,
+    and the target Pyarrow type, which makes debugging much easier.
+    """
+    data_arrays: dict[str, pa.Array] = dict()
+    for k, v in data.items():
+        try:
+            data_arrays[k] = pa.array(v, type=schema.field(k).type)
+        except Exception as e:
+            raise ValueError(
+                f"Unable to convert values for feature '{k}' into pyarrow type '{schema.field(k).type}': {e}"
+            )
+    return pa.RecordBatch.from_pydict(data_arrays, schema=schema)
