@@ -21,7 +21,7 @@ from typing import (
 
 import pyarrow as pa
 
-from chalk.features import DataFrame, Feature, FeatureConverter, Features
+from chalk.features import DataFrame, Feature, Features
 from chalk.sql._internal.incremental import IncrementalSettings
 from chalk.sql._internal.query_execution_parameters import (
     QueryExecutionParameters,
@@ -333,10 +333,11 @@ class FinalizedChalkQuery:
         A stream of `pa.RecordBatch`
         """
         from chalk.sql._internal.sql_source import BaseSQLSource
+        from chalk.sql._internal.sql_source_group import SQLSourceGroup
 
         self._check_incremental_settings()
         col_to_features = self._get_col_to_feature(expected_features)
-        assert isinstance(self.source, BaseSQLSource)
+        assert isinstance(self.source, (BaseSQLSource, SQLSourceGroup)), f"Expected BaseSQLSource, got {self.source}"
 
         if query_execution_parameters is None:
             query_execution_parameters = query_execution_parameters_from_env_vars()
@@ -487,39 +488,6 @@ class FinalizedChalkQuery:
             pa_table = pa_table.slice(0, 1)
         return pa_table
 
-    def execute_to_result_handles(
-        self,
-        expected_features: Optional[Sequence[Feature]] = None,
-        connection: Optional[Connection] = None,
-        query_execution_parameters: Optional[QueryExecutionParameters] = None,
-    ):
-        if not hasattr(self.source, "execute_to_result_handles"):
-            raise NotImplementedError
-
-        if self.incremental_settings is not None:
-            # FIXME: Move the incrementalization logic here, so then the `execute` and `execute_to_dataframe`
-            # methods can take the hwm timestamp as a paramater, to allow for direct execution
-            warnings.warn(
-                (
-                    "This query specified an incremental configuration, which has not been applied. "
-                    "This is likely because the resolver is being executed directly. "
-                    "The query will be attempted without any high-water-mark timestamp. "
-                    "This will attempt to select all data, "
-                    "or if the filters depend on the incremental timestamp, "
-                    "will result in a query execution error. "
-                )
-            )
-        expected_feature_root_fqns = (
-            None if expected_features is None else frozenset(x.root_fqn for x in expected_features)
-        )
-
-        def converter_getters(column_names: List[str]) -> Dict[str, FeatureConverter]:
-            # First map the column names to determine the feature fqns
-            col_name_mapping = self.get_col_name_mapping(tuple(column_names), expected_feature_root_fqns)
-            return {k: Feature.from_root_fqn(v).converter for (k, v) in col_name_mapping.items()}
-
-        return self.source.execute_to_result_handles(self, converter_getters, connection)  # pyright: ignore
-
     def execute_to_dataframe(
         self,
         expected_features: Optional[Sequence[Feature]] = None,
