@@ -1,41 +1,17 @@
 from __future__ import annotations
 
 from os import PathLike
-from typing import (
-    TYPE_CHECKING,
-    Any,
-    Callable,
-    Dict,
-    List,
-    Literal,
-    Mapping,
-    Optional,
-    Protocol,
-    Sequence,
-    Tuple,
-    Type,
-    TypeVar,
-    Union,
-    overload,
-)
+from typing import TYPE_CHECKING, Any, Dict, List, Literal, Mapping, Optional, Protocol, Type, TypeVar, Union, overload
 
-import pyarrow as pa
-
-from chalk.features import DataFrame, Feature, FeatureConverter, Features
+from chalk.features import DataFrame, Feature, Features
 from chalk.sql._internal.incremental import IncrementalSettings
 from chalk.utils.duration import Duration
 
 if TYPE_CHECKING:
     import sqlalchemy
     import sqlalchemy.ext.asyncio
-    from sqlalchemy.engine import Connection
-    from sqlalchemy.orm import Session
 
-    from chalk.sql.finalized_query import (
-        DataframeFinalizedChalkQuery,
-        FinalizedChalkQuery,
-        SingletonFinalizedChalkQuery,
-    )
+    from chalk.sql.finalized_query import DataframeFinalizedChalkQuery, SingletonFinalizedChalkQuery
 
 TTableIngestProtocol = TypeVar("TTableIngestProtocol", bound="TableIngestProtocol")
 
@@ -513,85 +489,6 @@ class BaseSQLSourceProtocol(Protocol):
         """
         ...
 
-    def raw_session(self) -> Session:
-        ...
-
-    def execute_query(
-        self,
-        finalized_query: FinalizedChalkQuery,
-        columns_to_converters: Callable[[List[str]], Dict[str, FeatureConverter]],
-        connection: Optional[Connection] = None,
-        attempt_efficient_execution: bool = True,
-        polars_read_csv: bool = False,
-    ) -> pa.Table:
-        """Execute a query to a `pa.Table`.
-
-        Parameters
-        ----------
-        finalized_query
-            The query to execute
-        columns_to_converters
-            A function that, given the list of column names returned from the database,
-            produces a mapping of columns to converters. Columns not in the resulting dictionary should be ignored.
-        connection
-            A connection to execute the query under. If not specified, then a new connection is acquired.
-        attempt_efficient_execution
-            The executor will attempt to avoid the use of SQLAlchemy and use more efficient methods to
-            retrieve data.
-
-        Returns
-        -------
-        pa.Table
-            A `pa.Table` containing the results.
-        """
-        ...
-
-    async def async_execute_query(
-        self,
-        finalized_query: FinalizedChalkQuery,
-        columns_to_converters: Callable[[List[str]], Dict[str, FeatureConverter]],
-        connection: Optional[sqlalchemy.ext.asyncio.AsyncConnection] = None,
-        attempt_efficient_execution: bool = True,
-        polars_read_csv: bool = False,
-    ) -> pa.Table:
-        """Execute a query to a `pa.Table`.
-
-        Parameters
-        ----------
-        finalized_query
-            The query to execute
-        columns_to_converters
-            A function that, given the list of column names returned from the database,
-            produces a mapping of columns to converters. Columns not in the resulting dictionary should be ignored.
-        connection
-            A connection to execute the query under. If not specified, then a new connection is acquired.
-
-        Returns
-        -------
-        pa.Table
-            A `pa.Table` containing the results.
-        """
-        ...
-
-    def compile_query(self, finalized_query: FinalizedChalkQuery) -> Tuple[str, Sequence[Any], Mapping[str, Any]]:
-        """Compile a query into a string and the (positional parameters, named parameters)
-
-        Returns
-        -------
-        tuple[str, Sequence[Any], dict[str, Any]]
-            The compiled SQL query.
-        """
-        ...
-
-    def get_sqlglot_dialect(self) -> Union[str, None]:
-        """Returns dialect for sqlglot operations.  Also happens to be the name of the source.
-
-        Returns
-        -------
-        Union[str, None]
-            The dialect if applicable to sqlglot
-        """
-
 
 class TableIngestProtocol(Protocol):
     def with_table(
