@@ -4,14 +4,20 @@ import dataclasses
 from typing import TYPE_CHECKING, Any, List, Sequence, Set, Type, Union
 
 import pyarrow
-from pydantic import BaseModel
 from typing_extensions import TypeAlias
 
-from chalk.features import DataFrame
 from chalk.utils import AnyDataclass
 
 if TYPE_CHECKING:
     import polars as pl
+    from pydantic import BaseModel
+
+    from chalk.features import DataFrame
+else:
+    try:
+        from pydantic.v1 import BaseModel
+    except ImportError:
+        from pydantic import BaseModel
 
 
 @dataclasses.dataclass
@@ -31,7 +37,7 @@ StreamResolverWindowType: TypeAlias = Union[
     Type[AnyDataclass],
     Type[pyarrow.Table],
     Type["pl.DataFrame"],
-    Type[DataFrame],
+    Type["DataFrame"],
     Any,  # The annotation value is likely going to be a GenericAlias, which messes with pydantic's validation
 ]
 
