@@ -1,7 +1,7 @@
 from __future__ import annotations
 
 import collections.abc
-from typing import Any, Dict, Iterable, List, Sequence, Set, Tuple, TypeVar, Union, cast, overload
+from typing import Any, Dict, Generic, Iterable, List, Optional, Sequence, Set, Tuple, TypeVar, Union, cast, overload
 
 from typing_extensions import Annotated, get_args, get_origin
 
@@ -14,6 +14,7 @@ _NoneType = type(None)
 _AnnotatedType = type(Annotated[int, ""])
 
 T = TypeVar("T")
+U = TypeVar("U")
 
 
 def unwrap_annotated_if_needed(typ: Any):
@@ -180,3 +181,50 @@ def get_unique_item_if_exists(iterable: Iterable[T], name: str | None = None) ->
     if item is ...:
         return None
     return item
+
+
+class BidirectionalMap(Generic[T, U]):
+    """
+    Represents a bijection between two sets of elements, with fast lookup for each direction.
+    Under the hood keeps two dictionaries, one from T-->U and another from U-->T
+    Constructor enforces that the mapping is a bijection.
+    """
+
+    def __init__(self, value_pairs: Sequence[Tuple[T, U]]):
+        """
+        Takes a sequence of pairs (t,u) to build the mapping t_i <--> u_i. Must be a bijection.
+        :param value_pairs:
+        """
+        super().__init__()
+        self._t_to_u: Dict[T, U] = dict(value_pairs)
+        self._u_to_t: Dict[U, T] = dict([(v, k) for (k, v) in value_pairs])
+        if (len(value_pairs) != len(self._t_to_u)) or (len(value_pairs) != len(self._u_to_t)):
+            raise ValueError("Mapping is not a bijection.")
+
+    def get(self, t: T, default: Optional[U] = None) -> Optional[U]:
+        return self._t_to_u.get(t, default)
+
+    def contains(self, t: T) -> bool:
+        return t in self._t_to_u
+
+    def get_reverse(self, u: U, default: Optional[T] = None) -> Optional[T]:
+        return self._u_to_t.get(u, default)
+
+    def contains_reverse(self, u: U) -> bool:
+        return u in self._u_to_t
+
+    def insert(self, t: T, u: U):
+        """
+        Raises if either element already exists.
+        """
+        if self.contains(t):
+            raise ValueError(f"Element {t} already exists in forward mapping.")
+        if self.contains_reverse(u):
+            raise ValueError(f"Element {t} already exists in reverse mapping.")
+
+    def __len__(self):
+        return len(self._t_to_u)
+
+    @classmethod
+    def empty(cls):
+        return cls([])
