@@ -1,12 +1,14 @@
 from __future__ import annotations
 
+import io
 import pathlib
 from io import BytesIO
-from typing import TYPE_CHECKING, Any, BinaryIO, Union
+from typing import TYPE_CHECKING, Any, BinaryIO, Literal, Union
 
 import numpy as np
 import pyarrow as pa
 import pyarrow.compute as pc
+import pyarrow.ipc as pa_ipc
 
 from chalk.features._encoding.pyarrow import is_map_in_dtype_tree, pyarrow_to_polars
 from chalk.utils.log_with_context import get_logger
@@ -491,3 +493,24 @@ if TYPE_CHECKING:
     read_parquet = pl.read_parquet
 else:
     read_parquet = _read_parquet
+
+
+def record_batch_to_arrow_ipc(rb: pa.RecordBatch, compression: Literal["lz4", "zstd", "uncompressed"] = "lz4"):
+    dest = BytesIO()
+    writer = pa_ipc.RecordBatchFileWriter(dest, rb.schema, options=pa_ipc.IpcWriteOptions(compression=compression))
+    writer.write_batch(rb)
+    writer.close()
+    dest.seek(0)
+    feather_bytes = dest.read()
+    return feather_bytes
+
+
+def arrow_ipc_to_record_batch(b: bytes) -> pa.RecordBatch:
+    bio = io.BytesIO(b)
+    reader = pa_ipc.RecordBatchFileReader(bio)
+    t: pa.Table = reader.read_all()
+    return pa_table_to_recordbatch(t)
+
+
+def pa_table_to_recordbatch(t: pa.Table) -> pa.RecordBatch:
+    return pa.record_batch([c.combine_chunks() for c in t.columns], t.column_names)
