@@ -1,6 +1,6 @@
 import builtins
 import types
-from typing import Any, Dict, List, Mapping, Optional, Tuple, Type, Union
+from typing import Any, Dict, List, Mapping, Optional, Type
 
 
 # A sentinel object to detect if a parameter is supplied or not.
