@@ -1,18 +1,23 @@
+from __future__ import annotations
+
+import inspect
 import os
-import sys
 from pathlib import Path
-from typing import Any, List, Optional
+from typing import List, Optional
 
 
-def get_classpath(x: object) -> Path:
-    filepath = sys.modules[x.__module__].__file__
-    assert filepath is not None
+def get_classpath(x: object) -> Path | None:
+    module = inspect.getmodule(x)
+    if module is None:
+        return
+    filepath = module.__file__
+    if filepath is None:
+        return
     return Path(os.path.abspath(filepath))
 
 
-def get_classpath_or_name(x: Any) -> str:
-    module = sys.modules[x.__module__]
-    return getattr(module, "__file__", None) or getattr(module, "__name__")
+def get_classpath_or_name(x: object) -> str | None:
+    return str(get_classpath(x)) or getattr(x, "__module__", None)
 
 
 def get_directory_root() -> Optional[Path]:
@@ -26,7 +31,7 @@ def get_directory_root() -> Optional[Path]:
         current = current.parent
 
 
-def _search_recursively_for_file(base: Path, filename: str) -> List[Path]:
+def search_recursively_for_file(base: Path, filename: str) -> List[Path]:
     ans = []
     assert base.is_dir()
     while True:
