@@ -62,7 +62,7 @@ class StubGenerator:
             item = unwrap_feature(item).fqn
         if isinstance(item, type):
             item = f"{item.__module__}.{item.__name__}"
-        if not isinstance(item, str):  # pyright: ignore[reportUnnecessaryIsInstance]
+        if not isinstance(item, str):
             raise TypeError(f"Invalid type for {item}: {type(item)}")
         item_split = tuple(item.split("."))
         if item_split[0] == "builtins":
@@ -157,11 +157,15 @@ class StubGenerator:
                         # still match feature classes that have a circular reference
                         protocol_annotation="Any" if (f.typ.as_features_cls() is not None) else protocol_annotation,
                     )
-        if features_cls.__name__ in self._parsed_feature_classes:
+        name = features_cls.__chalk_namespace__.replace("::", "_")
+        name_parts = name.split("_")
+        name = "".join(x.title() for x in name_parts)
+        if name in self._parsed_feature_classes:
+            # We could have duplicates because of namespacing. That is fine because we do protocol matching so the actual name doesn't matter
             raise ValueError(
-                f"Unable to generate stubs due to multiple features sets called {features_cls.__name__}. Feature set names must be unique. Please rename one of these feature sets."
+                f"Unable to generate stubs due to multiple features sets named '{features_cls.__chalk_namespace__}'. Feature set names must be unique. Please rename one of these feature sets."
             )
-        self._parsed_feature_classes[features_cls.__name__] = ParsedFeaturesClass(
+        self._parsed_feature_classes[name] = ParsedFeaturesClass(
             module=features_cls.__module__,
             annotations=annotations,
         )
